package com.cloud.aop;

import com.cloud.framework.core.annotation.DispersedTransaction;
import com.cloud.framework.core.annotation.LuceneUpdate;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;
import org.springframework.stereotype.Component;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import java.lang.reflect.Method;

/**
 * @Author sj
 * Date on 2020/9/10  11:06
 * @Version 1.0
 * 多服务事务管理
 */
@Slf4j
@Aspect
@Component
public class DispersedTransactionAop {
    @Autowired
    private DataSourceTransactionManager dataSourceTransactionManager;

    @Autowired
    private TransactionDefinition transactionDefinition;


    @Pointcut("@annotation(com.cloud.framework.core.annotation.DispersedTransaction)")
    public void DispersedTransactionAopPointCut() {
    }

    @Around(value = "DispersedTransactionAopPointCut()")
    public Object aroundNotice(ProceedingJoinPoint pjp) throws Throwable {
        Signature signature = pjp.getSignature();
        MethodSignature methodSignature = (MethodSignature) signature;
        //获取目标方法
        Method targetMethod = methodSignature.getMethod();
        //判断有没有统一事务管理注解
        if (targetMethod.isAnnotationPresent(DispersedTransaction.class)) {
            //声明手动事务
            TransactionStatus transactionStatus = null;
            try {
                //开启手动事务
                transactionStatus = dataSourceTransactionManager.getTransaction(transactionDefinition);
                //调用执行目标方法(result为目标方法执行结果)
                Object result = pjp.proceed();
                //没有出现异常提交手动事务
                dataSourceTransactionManager.commit(transactionStatus);
                return result;
            } catch (Exception e) {
                if (transactionStatus != null) {
                    //事务回滚
                    dataSourceTransactionManager.rollback(transactionStatus);
                }
                throw new Exception(e);
            }
        } else {
            return pjp.proceed();
        }
    }


}
