package com.cloud.framework.core.pagination;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import java.util.List;

/**
 * @Author sj
 * Date on 2020/10/21  15:53
 * @Version 1.0
 * android分页返回值对象
 */
@Data
public class ApiPageResult<T> {

    @ApiModelProperty("总页数")
    private Long totalPage;

    @ApiModelProperty("分页数据")
    private List<T> records;

    @ApiModelProperty("偏移值")
    private Long offset;

    @ApiModelProperty("查询条数")
    private Long pageSize;


    public ApiPageResult() {

    }

    public ApiPageResult(Paging<T> page) {
        if(page.getTotal()!=0) {
            this.totalPage = (long) Math.ceil((double)page.getTotal() / (double)page.getPageSize());
        }else{
            this.totalPage=0L;
        }
        this.records = page.getRecords();
        this.offset = page.getPageIndex()-1;
        this.pageSize = page.getPageSize();
    }

}
