package com.cloud.generator;

import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.generator.config.po.TableFill;
import com.baomidou.mybatisplus.generator.config.rules.DateType;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.cloud.generator.config.GeneratorStrategy;
import com.cloud.generator.constant.GeneratorConstant;
import com.cloud.generator.properties.GeneratorProperties;
import org.springframework.stereotype.Component;

import java.util.ArrayList;

/**
 * spring-boot-plus代码生成器入口类
 *
 * @author cyty
 * @date 2019-10-22
 **/
@Component
public class SpringBootPlusGenerator {

    /**
     * 生成代码
     *
     * @param args
     */
    public static void main(String[] args) {
        // 循环生成
        String[] tables = {
                "user_authentication",
                "company_authentication",
        };
        int i = 0;
        for (String table : tables) {
            // 设置需要生成的表名称生成代码
            create(table, "id");
            i = i + 1;
        }
        System.out.println(i);

    }

    public static void create(String tableName, String id) {

        GeneratorProperties generatorProperties = new GeneratorProperties();
        // 设置基本信息
        generatorProperties
                .setMavenModuleName("mobile-server")
                .setParentPackage("com.cloud")
                .setModuleName("web")
                .setAuthor("zyl")
                .setFileOverride(true);

        // 设置表信息
        generatorProperties.addTable(tableName, id);

        // 设置表前缀
        // generatorProperties.setTablePrefix(Arrays.asList("tb_"));

        // 数据源配置
        generatorProperties.getDataSourceConfig()
                .setDbType(DbType.MYSQL)
                .setUsername("recruitment")
                .setPassword("SNXBpepn2KxxZXZy")
                .setDriverName("com.mysql.jdbc.Driver")
                .setUrl("jdbc:mysql://47.108.174.85:3306/recruitment?useUnicode=true&characterEncoding=UTF-8&useSSL=false");
        // 生成配置
        generatorProperties.getGeneratorConfig()
                .setGeneratorStrategy(GeneratorStrategy.ALL)
                .setGeneratorEntity(true)
                .setGeneratorController(true)
                .setGeneratorService(true)
                .setGeneratorServiceImpl(true)
                .setGeneratorMapper(true)
                .setGeneratorMapperXml(true)
                .setGeneratorPageParam(true)
                .setGeneratorQueryVo(true)
                .setRequiresPermissions(false)
                .setPageListOrder(true)
                .setParamValidation(true)
                .setSwaggerTags(true)
                .setOperationLog(true);

        /**
         * 只生成entity
         */
//        generatorProperties.getGeneratorConfig()
//                .setGeneratorStrategy(GeneratorStrategy.SINGLE)
//                .setGeneratorEntity(true)
//                .setGeneratorController(false)
//                .setGeneratorService(false)
//                .setGeneratorServiceImpl(false)
//                .setGeneratorMapper(false)
//                .setGeneratorMapperXml(false)
//                .setGeneratorPageParam(false)
//                .setGeneratorQueryVo(true)
//                .setRequiresPermissions(false)
//                .setPageListOrder(false)
//                .setParamValidation(false)
//                .setSwaggerTags(true)
//                .setOperationLog(true);
        // 全局配置
        generatorProperties.getMybatisPlusGeneratorConfig().getGlobalConfig()
                .setOpen(false)
                .setSwagger2(true)
                .setIdType(IdType.AUTO)
                .setDateType(DateType.ONLY_DATE);
        // 自动填充配置
        TableFill gmtCreate = new TableFill("create_time", FieldFill.INSERT);
        TableFill gmtModified = new TableFill("update_time",
                FieldFill.INSERT_UPDATE);
        ArrayList<TableFill> tableFills = new ArrayList<>();
        tableFills.add(gmtCreate);
        tableFills.add(gmtModified);


        // 策略配置
        generatorProperties.getMybatisPlusGeneratorConfig().getStrategyConfig()
                .setNaming(NamingStrategy.underline_to_camel)
                .setColumnNaming(NamingStrategy.underline_to_camel)
                .setEntityLombokModel(true)
                .setRestControllerStyle(true)
                .setControllerMappingHyphenStyle(true)
                .setVersionFieldName(GeneratorConstant.VERSION)
                .setTableFillList(tableFills)
                .setLogicDeleteFieldName(GeneratorConstant.DELETED);


        // 生成代码
        CodeGenerator codeGenerator = new CodeGenerator();
        codeGenerator.generator(generatorProperties);

    }

}
