package com.cloud.web.commond.resolver;

import com.cloud.framework.common.api.ApiCode;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.config.AndroidConfig;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.util.EncryptUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.stereotype.Component;
import org.springframework.web.bind.support.WebDataBinderFactory;
import org.springframework.web.context.request.NativeWebRequest;
import org.springframework.web.method.support.HandlerMethodArgumentResolver;
import org.springframework.web.method.support.ModelAndViewContainer;

/**
 * @Author sj
 * Date on 2020/9/15  11:58
 * @Version 1.0
 * 自定义参数解析注入
 */
@Component
public class AuthUserMethodArgumentResolver implements HandlerMethodArgumentResolver {

    @Autowired
    private EncryptUtil encryptUtil;

    @Override
    public boolean supportsParameter(MethodParameter methodParameter) {
        if (methodParameter.hasParameterAnnotation(AnalysisUserId.class)) {
            return true;
        }
        return false;
    }

    @Override
    public Object resolveArgument(MethodParameter methodParameter, ModelAndViewContainer modelAndViewContainer, NativeWebRequest nativeWebRequest, WebDataBinderFactory webDataBinderFactory) throws Exception {
        if (methodParameter.hasParameterAnnotation(AnalysisUserId.class)) {
            String token = nativeWebRequest.getHeader(AndroidConfig.TOKEN_NAME);
            //解密token获取用户编号
            Long id = Long.parseLong(encryptUtil.Base64Decode(encryptUtil.XORdecode(token, AndroidConfig.SALF)).split("-")[0]);
            if (id == null || id <= 0) {
                throw new AndroidBusinessException(ApiCode.UNKNOWN_EXCEPTION);
            }
            return id;
        } else {
            return null;
        }
    }
}
