package com.cloud.web.controller;

import com.cloud.web.entity.ApplePriceSetting;
import com.cloud.web.service.ApplePriceSettingService;
import lombok.extern.slf4j.Slf4j;
import com.cloud.web.param.ApplePriceSettingPageParam;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.web.vo.ApplePriceSettingQueryVo;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.common.param.IdParam;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import com.cloud.framework.core.validator.groups.Add;
import com.cloud.framework.core.validator.groups.Update;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 控制器
 *
 * @author zyl
 * @since 2022-07-19
 */
@Slf4j
@RestController
@RequestMapping("/applePriceSetting")
@Module("web")
@Api(value = "API：苹果支付", tags = {"API：苹果支付"})
public class ApplePriceSettingController extends BaseController {

    @Autowired
    private ApplePriceSettingService applePriceSettingService;

    /**
     * 添加
     */
    @PostMapping("/add")
    @OperationLog(name = "添加", type = OperationLogType.ADD)
    @ApiOperation(value = "添加", response = ApiResult.class)
    public ApiResult<Boolean> addApplePriceSetting(@Validated(Add.class) @RequestBody ApplePriceSetting applePriceSetting) throws Exception {
        boolean flag = applePriceSettingService.saveApplePriceSetting(applePriceSetting);
        return ApiResult.result(flag);
    }

    /**
     * 修改
     */
    @PostMapping("/update")
    @OperationLog(name = "修改", type = OperationLogType.UPDATE)
    @ApiOperation(value = "修改", response = ApiResult.class)
    public ApiResult<Boolean> updateApplePriceSetting(@Validated(Update.class) @RequestBody ApplePriceSetting applePriceSetting) throws Exception {
        boolean flag = applePriceSettingService.updateApplePriceSetting(applePriceSetting);
        return ApiResult.result(flag);
    }

    /**
     * 删除
     */
    @PostMapping("/delete/{id}")
    @OperationLog(name = "删除", type = OperationLogType.DELETE)
    @ApiOperation(value = "删除", response = ApiResult.class)
    public ApiResult<Boolean> deleteApplePriceSetting(@PathVariable("id") Long id) throws Exception {
        boolean flag = applePriceSettingService.deleteApplePriceSetting(id);
        return ApiResult.result(flag);
    }

    /**
     * 获取详情
     */
    @GetMapping("/info")
    @OperationLog(name = "详情", type = OperationLogType.INFO)
    @ApiOperation(value = "详情", response = ApplePriceSettingQueryVo.class)
    public ApiResult<ApplePriceSettingQueryVo> getApplePriceSetting(Long id) throws Exception {
        ApplePriceSettingQueryVo applePriceSettingQueryVo = applePriceSettingService.getApplePriceSettingById(id);
        return ApiResult.ok(applePriceSettingQueryVo);
    }

    /**
     * 分页列表
     */
    @PostMapping("/getPageList")
    @OperationLog(name = "分页列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "分页列表", response = ApplePriceSettingQueryVo.class)
    public ApiResult<Paging<ApplePriceSetting>> getApplePriceSettingPageList(@Validated @RequestBody ApplePriceSettingPageParam applePriceSettingPageParam) throws Exception {
        Paging<ApplePriceSetting> paging = applePriceSettingService.getApplePriceSettingPageList(applePriceSettingPageParam);
        return ApiResult.ok(paging);
    }

}

