package com.cloud.web.controller;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.entity.Draw;
import com.cloud.web.entity.User;
import com.cloud.web.params.page.MyDocumentPageParam;
import com.cloud.web.service.DrawDocumentService;
import com.cloud.web.service.DrawService;
import com.cloud.web.service.UserService;
import com.cloud.web.vo.DrawDocumentQueryVo;
import com.cloud.web.vo.DrawVo;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.*;

/**
 *  控制器
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@RestController
@RequestMapping("/app/draw")
@Module("web")
@Api(value = "API：抽奖管理", tags = {"API：抽奖管理"})
@ResponseResult
public class DrawController extends BaseController {

    @Autowired
    private DrawService drawService;
    @Autowired
    private DrawDocumentService drawDocumentService;
    @Autowired
    private UserService userService;

    /**
     * 分页列表
     */
    @GetMapping("/getList")
    @OperationLog(name = "奖品列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "奖品列表", response = Draw.class)
    public Map<String,Object> getDrawPageList(@AnalysisUserId Long userId) throws Exception {
        Map map = new HashMap<String,Object>();
        List<Draw> drawList = drawService.getList();
        User user = userService.getById(userId);
        map.put("drawList",drawList);
        map.put("drawNumber",user.getDrawNumber());
        map.put("integral",user.getIntegral());
        return map;
    }

    /**
     * 抽奖
     */
    @GetMapping("/getDraw")
    @OperationLog(name = "抽奖", type = OperationLogType.PAGE)
    @ApiOperation(value = "抽奖", response = DrawVo.class)
    public DrawVo getDraw(@AnalysisUserId Long userId) throws Exception {
        DrawVo drawVo = drawService.raffle(userId);
        return drawVo;
    }

    @GetMapping("/page/getDrawDocument")
    @OperationLog(name = "分页获取抽奖记录列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "分页获取抽奖记录列表", response = DrawDocumentQueryVo.class)
    public ApiPageResult<DrawDocumentQueryVo> getList(@AnalysisUserId Long userId, @Validated MyDocumentPageParam myDocumentPageParam) throws Exception {
        myDocumentPageParam.setUserId(userId);
        return drawDocumentService.getList(myDocumentPageParam);
    }


}

