package com.cloud.web.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.framework.core.annotation.WebLoginToken;
import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.entity.*;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.param.AddPayrollRecordsParam;
import com.cloud.web.params.page.MyAnnouncementPageParam;
import com.cloud.web.params.page.MyBlacklistRecordsPageParam;
import com.cloud.web.params.param.InsertCompanyAuthenticationParam;
import com.cloud.web.params.param.InsertUserAuthenticationParam;
import com.cloud.web.service.*;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.*;


/**
 * 最新接口控制器
 *
 * @author
 */
@Slf4j
@RestController
@RequestMapping("/app/new")
@Module("web")
@Api(value = "API：新接口", tags = {"API：新接口"})
@ResponseResult
public class NewSectionController extends BaseController {

    @Autowired
    private BaseSettingService baseSettingService;
    @Autowired
    private AnnouncementService announcementService;
    @Autowired
    private PostService postService;
    @Autowired
    private UserService userService;
    @Autowired
    private PayrollRecordsService payrollRecordsService;
    @Autowired
    private BlacklistRecordsService blacklistRecordsService;
    @Autowired
    private ApplyService applyService;
    @Autowired
    private CompanyAuthenticationService companyAuthenticationService;
    @Autowired
    private UserAuthenticationService userAuthenticationService;
    @Autowired
    private DescribeFaceVerifyService describeFaceVerifyService;

    /**
     * @Function: 基础配置
     */
    @GetMapping("/getBaseSetting")
    @OperationLog(name = "基础配置", type = OperationLogType.INFO)
    @ApiOperation(value = "基础配置", response = BaseSetting.class)
    @WebLoginToken
    public BaseSetting getBaseSetting(HttpServletRequest request) {
        BaseSetting baseSetting = baseSettingService.getOne();
        if (StringUtils.isNotBlank(baseSetting.getAppVersion())) {

            log.info("当前安装版本====》》》" + baseSetting.getAppVersion());
            System.out.println("当前安装版本====》》》" + baseSetting.getAppVersion());
            String version = request.getHeader("version");
            if (StringUtils.isNotBlank(version)) {
                log.info("APP当前安装版本====》》》" + version);
                System.out.println("APP当前安装版本====》》》" + version);
                if (version.length() < 4 && !baseSetting.getAppVersion().equals(version)) {
                    baseSetting.setIsRenewal(1);
                }
                if (version.length() >= 4 && !baseSetting.getIosVersion().equals(version)) {
                    baseSetting.setIsRenewal(1);
                }
            }
        }
        return baseSetting;
    }


    /**
     * 公告详情
     */
    @GetMapping("/info")
    @OperationLog(name = "公告详情", type = OperationLogType.INFO)
    @ApiOperation(value = "公告详情", response = AnnouncementQueryVo.class)
    @WebLoginToken
    public ApiResult<AnnouncementQueryVo> getAnnouncement(Long id) throws Exception {
        AnnouncementQueryVo announcementQueryVo = announcementService.getAnnouncementById(id);
        return ApiResult.ok(announcementQueryVo);
    }

    /**
     * 公告分页列表
     */
    @GetMapping("/getPageList")
    @OperationLog(name = "公告分页列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "公告分页列表", response = AnnouncementQueryVo.class)
    @WebLoginToken
    public ApiPageResult<AnnouncementQueryVo> getAnnouncementPageList(@Validated MyAnnouncementPageParam myAnnouncementPageParam) throws Exception {
        ApiPageResult<AnnouncementQueryVo> paging = announcementService.getAnnouncementPageLists(myAnnouncementPageParam);
        return paging;
    }


    /**
     * 企业职位列表
     */
    @GetMapping("/getPostList")
    @OperationLog(name = "企业职位列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "企业职位列表", response = Post.class)
    public List<Post> getPostList(@AnalysisUserId Long userId) {
        User user = userService.getById(userId);
        if (user != null && user.getCompanyId() != null) {
            List<Post> posts = postService.list(new LambdaQueryWrapper<Post>().eq(Post::getCompanyId, user.getCompanyId()));
            return posts;
        }
        return new ArrayList<>();
    }


    /**
     * 工资发放
     */
    @PostMapping("/addPayrollRecords")
    @OperationLog(name = "工资发放", type = OperationLogType.ADD)
    @ApiOperation(value = "工资发放", response = ApiResult.class)
    public ApiResult<Boolean> addPayrollRecords(@AnalysisUserId Long userId, AddPayrollRecordsParam addPayrollRecordsParam) throws Exception {
        if (userId != null) {
            addPayrollRecordsParam.setUserId(userId);
            boolean flag = payrollRecordsService.addPayrollRecords(addPayrollRecordsParam);
            return ApiResult.result(flag);
        }
        return ApiResult.result(false);
    }


    /**
     * 黑名单列表
     */
    @GetMapping("/getBlacklistRecords")
    @OperationLog(name = "黑名单列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "黑名单列表", response = BlacklistRecordsQueryVo.class)
    @WebLoginToken
    public ApiPageResult<BlacklistRecordsQueryVo> addBlacklistRecords(@Validated MyBlacklistRecordsPageParam myAnnouncementPageParam) throws Exception {
        ApiPageResult<BlacklistRecordsQueryVo> paging = blacklistRecordsService.getBlacklistRecordsPageLists(myAnnouncementPageParam);
        List<BlacklistRecordsQueryVo> blacklistRecordsQueryVos = paging.getRecords();
        if (blacklistRecordsQueryVos != null && blacklistRecordsQueryVos.size() > 0) {
            for (BlacklistRecordsQueryVo blacklistRecordsQueryVo : blacklistRecordsQueryVos) {
                if (StringUtils.isNotBlank(blacklistRecordsQueryVo.getUserPhone())) {
                    String phone1 = blacklistRecordsQueryVo.getUserPhone().substring(0, 3);
                    String phone2 = blacklistRecordsQueryVo.getUserPhone().substring(7);
                    blacklistRecordsQueryVo.setUserPhone(phone1 + "****" + phone2);
                }
            }
        }
        return paging;
    }


    /**
     * 移除黑名单
     */
    @GetMapping("/removeBlacklistRecords")
    @OperationLog(name = "移除黑名单", type = OperationLogType.DELETE)
    @ApiOperation(value = "移除黑名单", response = ApiResult.class)
    @WebLoginToken
    public Boolean removeBlacklistRecords(String ids) {
        if (StringUtils.isNotBlank(ids)) {
            String[] idArr = ids.split(",");
            for (String id : idArr) {
                blacklistRecordsService.deleteById(id);
            }
            return true;
        }
        return false;
    }


    /**
     * 用户搜索
     */
    @GetMapping("/getUser")
    @OperationLog(name = "用户搜索", type = OperationLogType.LIST)
    @ApiOperation(value = "用户搜索", response = UserVo.class)
    @WebLoginToken
    public UserVo getUser(String query) {
        UserVo userVo = new UserVo();
        if (StringUtils.isEmpty(query)) {
            throw new AndroidBusinessException(ApiCode.PARAMETER_ISNULL_ERR);
        }
        List<User> userList = userService.list(new LambdaQueryWrapper<User>().like(User::getUserName, query));
        if (userList != null && userList.size() == 1) {
            User user = userList.get(0);
            BeanUtils.copyProperties(user, userVo);
            BlacklistRecords blacklistRecords = blacklistRecordsService.getOne(new LambdaQueryWrapper<BlacklistRecords>().eq(BlacklistRecords::getUserId, user.getId()));
            if (blacklistRecords != null) {
                userVo.setIsBlack(1);
                if (StringUtils.isNotBlank(blacklistRecords.getRemark())) {
                    userVo.setRemark(blacklistRecords.getRemark());
                }
            }
            return userVo;
        }
        List<User> userLists = userService.list(new LambdaQueryWrapper<User>().like(User::getPhone, query));
        if (userList != null && userLists.size() == 1) {
            User user = userList.get(0);
            BeanUtils.copyProperties(user, userVo);
            BlacklistRecords blacklistRecords = blacklistRecordsService.getOne(new LambdaQueryWrapper<BlacklistRecords>().eq(BlacklistRecords::getUserId, user.getId()));
            if (blacklistRecords != null) {
                userVo.setIsBlack(1);
                if (StringUtils.isNotBlank(blacklistRecords.getRemark())) {
                    userVo.setRemark(blacklistRecords.getRemark());
                }
            }
            return userVo;
        }
        throw new AndroidBusinessException(ApiCode.INEXISTENCE);
    }


    /**
     * 添加黑名单
     */
    @PostMapping("/addBlacklistRecords")
    @OperationLog(name = "添加黑名单", type = OperationLogType.ADD)
    @ApiOperation(value = "添加黑名单", response = ApiResult.class)
    public Boolean addBlacklistRecords(@AnalysisUserId Long userId, AddBlacklistRecordsVo addBlacklistRecordsVo) {
        if (userId != null) {
            addBlacklistRecordsVo.setUserId(userId.intValue());
            return blacklistRecordsService.addBlacklistRecords(addBlacklistRecordsVo);
        }
        return false;
    }


    @GetMapping("/getApplyTotal")
    @OperationLog(name = "职位申请：待审核统计", type = OperationLogType.LIST)
    @ApiOperation(value = "职位申请：待审核统计", response = ApiResult.class)
    public int getApply(@AnalysisUserId Long userId) {
        User user = userService.getById(userId);
        if (user != null) {
            LambdaQueryWrapper<Apply> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(Apply::getCompanyId, user.getCompanyId()).eq(Apply::getStatus, 1);
            return applyService.count(wrapper);
        }
        return 0;
    }


    /**
     * 企业认证
     */
    @PostMapping("/addCompanyAuthentication")
    @OperationLog(name = "企业认证", type = OperationLogType.ADD)
    @ApiOperation(value = "企业认证", response = ApiResult.class)
    public ApiResult<Boolean> addCompanyAuthentication(@AnalysisUserId Long userId, InsertCompanyAuthenticationParam insertCompanyAuthenticationParam) throws Exception {
        if (userId != null) {
            insertCompanyAuthenticationParam.setUserId(userId);
            boolean flag = companyAuthenticationService.addCompanyAuthentication(insertCompanyAuthenticationParam);
            return ApiResult.result(flag);
        }
        return ApiResult.result(false);
    }

    /**
     * 企业认证详情
     */
    @GetMapping("/getCompanyAuthentication")
    @OperationLog(name = "企业认证详情", type = OperationLogType.INFO)
    @ApiOperation(value = "企业认证详情", response = CompanyAuthentication.class)
    public CompanyAuthentication addCompanyAuthentication(@AnalysisUserId Long userId) {
        if (userId != null) {
            CompanyAuthentication companyAuthentication = companyAuthenticationService.getOne(new LambdaQueryWrapper<CompanyAuthentication>().eq(CompanyAuthentication::getUserId, userId));
            return companyAuthentication;
        }
        return null;
    }

    /**
     * 用户身份信息提取
     */
    @PostMapping("/getIdCard")
    @OperationLog(name = "用户身份信息提取", type = OperationLogType.ADD)
    @ApiOperation(value = "用户身份信息提取", response = IdCardVo.class)
    public IdCardVo getIdCard(InsertUserAuthenticationParam insertUserAuthenticationParam) throws Exception {
        return userAuthenticationService.getIdCard(insertUserAuthenticationParam);
    }


    /**
     * 用户实名认证
     */
    @PostMapping("/addUserAuthentication")
    @OperationLog(name = "用户实名认证", type = OperationLogType.ADD)
    @ApiOperation(value = "用户实名认证", response = ApiResult.class)
    public ApiResult<String> addUserAuthentication(@AnalysisUserId Long userId, InsertUserAuthenticationParam insertUserAuthenticationParam) throws Exception {
        if (userId != null) {
            insertUserAuthenticationParam.setUserId(userId);
            String cid = userAuthenticationService.addUserAuthentication(insertUserAuthenticationParam);
            return ApiResult.ok(cid);
        }
        return null;
    }


    /**
     * 用户实名认证结果
     */
    @GetMapping("/getDescribeFaceVerify")
    @OperationLog(name = "用户实名认证结果", type = OperationLogType.OTHER_QUERY)
    @ApiOperation(value = "用户实名认证结果", response = ApiResult.class)
    public String getDescribeFaceVerify(@AnalysisUserId Long userId, String certifyId) throws Exception {
        return describeFaceVerifyService.getDescribeFaceVerify(userId, certifyId);
    }


    /**
     * 用户实名认证详情
     */
    @GetMapping("/getUserAuthentication")
    @OperationLog(name = "用户实名认证详情", type = OperationLogType.INFO)
    @ApiOperation(value = "用户实名认证详情", response = UserAuthentication.class)
    public UserAuthentication getUserAuthentication(@AnalysisUserId Long userId) {
        if (userId != null) {
            UserAuthentication userAuthentication = userAuthenticationService.getOne(new LambdaQueryWrapper<UserAuthentication>().eq(UserAuthentication::getUserId, userId));
            return userAuthentication;
        }
        return null;
    }

}

