package com.cloud.web.controller;

import com.cloud.web.entity.OrderBuy;
import com.cloud.web.service.OrderBuyService;
import lombok.extern.slf4j.Slf4j;
import com.cloud.web.param.OrderBuyPageParam;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.web.vo.OrderBuyQueryVo;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.common.param.IdParam;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import com.cloud.framework.core.validator.groups.Add;
import com.cloud.framework.core.validator.groups.Update;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 *  控制器
 *
 * @author zyl
 * @since 2021-11-10
 */
@Slf4j
@RestController
@RequestMapping("/orderBuy")
@Module("web")
@Api(value = "API：职位购买信息", tags = {"API：职位购买信息"})
public class OrderBuyController extends BaseController {

    @Autowired
    private OrderBuyService orderBuyService;

    /**
     * 添加
     */
    @PostMapping("/add")
    @OperationLog(name = "添加", type = OperationLogType.ADD)
    @ApiOperation(value = "添加", response = ApiResult.class)
    public ApiResult<Boolean> addOrderBuy(@Validated(Add.class) @RequestBody OrderBuy orderBuy) throws Exception {
        boolean flag = orderBuyService.saveOrderBuy(orderBuy);
        return ApiResult.result(flag);
    }

    /**
     * 修改
     */
    @PostMapping("/update")
    @OperationLog(name = "修改", type = OperationLogType.UPDATE)
    @ApiOperation(value = "修改", response = ApiResult.class)
    public ApiResult<Boolean> updateOrderBuy(@Validated(Update.class) @RequestBody OrderBuy orderBuy) throws Exception {
        boolean flag = orderBuyService.updateOrderBuy(orderBuy);
        return ApiResult.result(flag);
    }

    /**
     * 删除
     */
    @PostMapping("/delete/{id}")
    @OperationLog(name = "删除", type = OperationLogType.DELETE)
    @ApiOperation(value = "删除", response = ApiResult.class)
    public ApiResult<Boolean> deleteOrderBuy(@PathVariable("id") Long id) throws Exception {
        boolean flag = orderBuyService.deleteOrderBuy(id);
        return ApiResult.result(flag);
    }

    /**
     * 获取详情
     */
    @GetMapping("/info/{id}")
    @OperationLog(name = "详情", type = OperationLogType.INFO)
    @ApiOperation(value = "详情", response = OrderBuyQueryVo.class)
    public ApiResult<OrderBuyQueryVo> getOrderBuy(@PathVariable("id") Long id) throws Exception {
        OrderBuyQueryVo orderBuyQueryVo = orderBuyService.getOrderBuyById(id);
        return ApiResult.ok(orderBuyQueryVo);
    }

    /**
     * 分页列表
     */
    @PostMapping("/getPageList")
    @OperationLog(name = "分页列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "分页列表", response = OrderBuyQueryVo.class)
    public ApiResult<Paging<OrderBuy>> getOrderBuyPageList(@Validated @RequestBody OrderBuyPageParam orderBuyPageParam) throws Exception {
        Paging<OrderBuy> paging = orderBuyService.getOrderBuyPageList(orderBuyPageParam);
        return ApiResult.ok(paging);
    }

}

