package com.cloud.web.controller;

import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.framework.core.annotation.WebLoginToken;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.entity.BaseSetting;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.modules.pay.wechat.service.WeChatPayServices;
import com.cloud.web.service.BaseSettingService;
import com.cloud.web.service.MemberRecordsService;
import com.cloud.web.util.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("/app/pay")
@Module("web")
@Api(value = "API：微信支付", tags = {"API：微信支付"})
@ResponseResult
public class PayController extends BaseController {

    @Autowired
    private MemberRecordsService memberRecordsService;
    @Autowired
    private WeChatPayServices weChatPayServices;
    @Autowired
    private BaseSettingService baseSettingService;

    /**
     * @Function: 价格查询
     */
    @GetMapping("/getPrice")
    @ResponseBody
    @OperationLog(name = "会员价格", type = OperationLogType.PAY)
    @ApiOperation(value = "会员价格", response = BaseSetting.class)
    public BaseSetting getPrice(HttpServletRequest request) {
        BaseSetting baseSetting = baseSettingService.getOne();
        log.info("当前安装版本====》》》" + baseSetting.getAppVersion());
        System.out.println("当前安装版本====》》》" + baseSetting.getAppVersion());
        String version = request.getHeader("version");
        if (StringUtils.isNotBlank(version)) {
            log.info("APP当前安装版本====》》》" + version);
            System.out.println("APP当前安装版本====》》》" + version);
            if (!baseSetting.getAppVersion().equals(version)) {
                baseSetting.setIsRenewal(1);
            }
        }
        return baseSetting;
    }

    /**
     * @Function: 去支付
     */
    @PostMapping("/pay")
    @ResponseBody
    @OperationLog(name = "微信支付(开通会员)", type = OperationLogType.PAY)
    @ApiOperation(value = "微信支付（开通会员）", response = ApiResult.class)
    public Map<String, Object> goWeChatPay(@AnalysisUserId Long userId) throws Exception {
        Map<String, Object> map = memberRecordsService.goWeChatPay(userId, 1, getRequest());
        if (Integer.parseInt(map.get("code").toString()) == 500) {
            return null;
        }
        return (Map) map.get("data");
    }


    /**
     * @Function: 去支付
     */
    @PostMapping("/pays")
    @ResponseBody
    @OperationLog(name = "微信支付(开通推荐管)", type = OperationLogType.PAY)
    @ApiOperation(value = "微信支付（开通推荐管）", response = ApiResult.class)
    public Map<String, Object> goWeChatPays(@AnalysisUserId Long userId) throws Exception {
        Map<String, Object> map = memberRecordsService.goWeChatPay(userId, 2, getRequest());
        if (Integer.parseInt(map.get("code").toString()) == 500) {
            return null;
        }
        return (Map) map.get("data");
    }

    /**
     * @Function: 微信支付回调
     */
    @PostMapping("/call/back")
    @OperationLog(name = "微信支付回调", type = OperationLogType.PAY)
    @ApiOperation(value = "微信支付回调", response = String.class)
    @WebLoginToken
    public void weChatNotify(HttpServletRequest request) throws IOException {
        request.setCharacterEncoding("UTF-8");
        getResponse().setCharacterEncoding("UTF-8");
        getResponse().setContentType("application/xml; charset=utf-8");
        PrintWriter writer = null;
        String returnXML;
        try {
            returnXML = weChatPayServices.weChatNotify(request);
            writer = getResponse().getWriter();
            writer.write(returnXML);
            writer.flush();
        } catch (Exception e) {
            throw new AndroidBusinessException(e.getMessage(), ApiCode.PAY_ERR);
        } finally {
            if (writer != null) {
                writer.close();
            }
        }
    }

}
