package com.cloud.web.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.framework.core.annotation.WebLoginToken;
import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.entity.Apply;
import com.cloud.web.entity.Servicer;
import com.cloud.web.entity.User;
import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.param.IntroUserPageParam;
import com.cloud.web.param.UserPageParam;
import com.cloud.web.params.page.MyHelpPageParam;
import com.cloud.web.params.page.MyMessagePageParam;
import com.cloud.web.service.*;
import com.cloud.web.vo.*;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.log.annotation.Module;
import io.swagger.annotations.Api;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 控制器
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@RestController
@RequestMapping("/app/article")
@Module("web")
@Api(value = "API：个人中心", tags = {"API：个人中心"})
@ResponseResult
public class PersonController extends BaseController {

    @Autowired
    private MessageService messageService;
    @Autowired
    private HelpCenterService helpCenterService;
    @Autowired
    private ServicerService servicerService;
    @Autowired
    private ArticleService articleService;
    @Autowired
    private UserService userService;
    @Autowired
    private ApplyService applyService;

    @GetMapping("/getList")
    @ApiOperation(value = "获取客服列表", response = Servicer.class)
    public List<Servicer> getList() throws Exception {
        return servicerService.getServicer();
    }

    @GetMapping("/page/getMessages")
    @OperationLog(name = "分页获取消息列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "分页获取消息列表", response = MessageQueryVo.class)
    public ApiPageResult<MessageQueryVo> getList(@AnalysisUserId Long userId, @Validated MyMessagePageParam myMessagePageParam) throws Exception {
        myMessagePageParam.setUserId(userId);
        return messageService.getMessageQueryVo(myMessagePageParam);
    }

    @GetMapping("/page/getHelps")
    @OperationLog(name = "分页获取帮助列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "分页获取帮助列表", response = HelpCenterQueryVo.class)
    public ApiPageResult<HelpCenterQueryVo> getHelps(@Validated MyHelpPageParam myHelpPageParam) throws Exception {
        return helpCenterService.getHelpCenterQueryVo(myHelpPageParam);
    }

    @GetMapping("/get/type")
    @WebLoginToken
    public MyArticleQueryVo getMyArticleQueryVo(Integer type) throws Exception {
        if (type == null || type <= 0) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_VERIFY_ERR);
        }
        return articleService.getMyArticleQueryVo(type);
    }

    /**
     * 获取用户详情
     */
    @GetMapping("/userInfo")
    @OperationLog(name = "用户详情", type = OperationLogType.INFO)
    @ApiOperation(value = "用户详情", response = UserQueryVo.class)
    public UserQueryVo getUserInfo(@AnalysisUserId Long userId) throws Exception {
        if (userId == null || userId <= 0) {
            throw new AndroidVerifyException(ApiCode.USER_TOKEN_OVERDUE_ERR);
        }
        UserQueryVo userQueryVo = userService.getUserById(userId);
        if (userQueryVo == null) {
            throw new AndroidVerifyException(ApiCode.NOT_LOGIN);
        }
        userQueryVo.setShowIntegral(0);
        List<Apply> applies = applyService.list(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, userId).eq(Apply::getStatus, 3).orderByDesc(Apply::getCreateTime));
        if (applies != null && applies.size() > 0) {
            userQueryVo.setIsApply(true);
        } else {
            userQueryVo.setIsApply(false);
        }
        return userQueryVo;
    }

    /**
     * 邀请记录
     */
    @GetMapping("/getIntroRecord")
    @OperationLog(name = "邀请记录", type = OperationLogType.INFO)
    @ApiOperation(value = "邀请记录", response = IntroUserVo.class)
    public Map getIntroRecord(@AnalysisUserId Long userId, IntroUserPageParam introUserPageParam) throws Exception {
        Map map = new HashMap();
        if (userId == null || userId <= 0) {
            throw new AndroidVerifyException(ApiCode.USER_TOKEN_OVERDUE_ERR);
        }
        UserQueryVo userQueryVo = userService.getUserById(userId);
        if (userQueryVo != null && userQueryVo.getPhone() != null) {
            introUserPageParam.setPhone(userQueryVo.getPhone());
        } else {
            introUserPageParam.setPhone(String.valueOf(0));
        }
        ApiPageResult<IntroUserVo> introUserVoList = userService.getIntroRecord(introUserPageParam);
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getInvitationCode, introUserPageParam.getPhone());
        int count = userService.count(lambdaQueryWrapper);
        map.put("introUserVoList", introUserVoList);
        map.put("count", count);
        return map;
    }

}

