package com.cloud.web.controller;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.entity.*;
import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.params.page.MyPostPageParam;
import com.cloud.web.params.param.InsertApplyParam;
import com.cloud.web.service.*;
import com.cloud.web.vo.PostInfoVo;
import com.cloud.web.vo.PostQueryVo;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 控制器
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@RestController
@RequestMapping("/app/post")
@Module("web")
@Api(value = "API：职位管理", tags = {"API：职位管理"})
@ResponseResult
public class PostController extends BaseController {

    @Autowired
    private PostService postService;
    @Autowired
    private ApplyService applyService;
    @Autowired
    private HotSearchService hotSearchService;

    /**
     * 获取详情
     */
    @GetMapping("/info")
    @OperationLog(name = "详情", type = OperationLogType.INFO)
    @ApiOperation(value = "详情", response = PostInfoVo.class)
    public PostInfoVo getPost(@AnalysisUserId Long userId, Integer postId) throws Exception {
        if (postId == null || postId <= 0) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_VERIFY_ERR);
        }
        PostInfoVo postInfoVo = postService.getOne(postId, userId);
        return postInfoVo;
    }

    @GetMapping("/page/getList")
    @OperationLog(name = "分页获取职位列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "分页获取职位列表", response = PostQueryVo.class)
    public ApiPageResult<PostQueryVo> getList(@AnalysisUserId Long userId, @Validated MyPostPageParam postPageParam) throws Exception {
        postPageParam.setUserId(userId);
        ApiPageResult<PostQueryVo> postQueryVoApiPageResult = postService.getPostVo(postPageParam);
        List<PostQueryVo> postQueryVos = postQueryVoApiPageResult.getRecords();
        if (postQueryVos != null && postQueryVos.size() > 0) {
            for (PostQueryVo postQueryVo : postQueryVos) {
                Apply apply = applyService.getOne(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, userId).eq(Apply::getPostId, postQueryVo.getId()));
                if (apply != null) {
                    postQueryVo.setIsBuy(apply.getIsBuy());
                }
                String updateTime = postQueryVo.getUpdateTime().substring(0, 16);
                postQueryVo.setUpdateTime(updateTime);
            }
        }
        return postQueryVoApiPageResult;
    }

    @PostMapping("/add/apply")
    @OperationLog(name = "报名", type = OperationLogType.UPDATE)
    @ApiOperation(value = "报名", response = ApiResult.class)
    public Boolean addApply(@AnalysisUserId Long userId, @Validated InsertApplyParam insertApplyParam) throws Exception {
        insertApplyParam.setUserId(userId);
        applyService.addApply(insertApplyParam);
        return null;
    }

    @GetMapping("/getSearch")
    @OperationLog(name = "热门搜索", type = OperationLogType.LIST)
    @ApiOperation(value = "热门搜索", response = HotSearch.class)
    public List<HotSearch> getSearch() {
        return hotSearchService.getSearch();
    }

}

