package com.cloud.web.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.framework.core.annotation.WebLoginToken;
import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import com.cloud.system.service.UpLoadService;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.entity.*;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.modules.send.utils.NoteSendUtil;
import com.cloud.web.param.*;
import com.cloud.web.params.page.MyPostPageParam;
import com.cloud.web.params.param.*;
import com.cloud.web.service.*;
import com.cloud.web.util.IosVerifyUtil;
import com.cloud.web.util.RedisUtils;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.ApplePayVo;
import com.cloud.web.vo.LoginTokenVo;
import com.cloud.web.vo.PostQueryVo;
import com.cloud.web.vo.PostVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author 石家庄赵子龙
 */
@Slf4j
@RestController
@RequestMapping("/app/second")
@Module("web")
@Api(value = "API：二开接口", tags = {"API：二开接口"})
@ResponseResult
public class SecondSectionController extends BaseController {

    @Autowired
    private MemberRecordsService memberRecordsService;
    @Autowired
    private PostService postService;
    @Autowired
    private UserService userService;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private ApplyService applyService;
    @Autowired
    private BaseSettingService baseSettingService;
    @Autowired
    private PunchCardService punchCardService;
    @Resource
    private UpLoadService upLoadService;
    @Autowired
    private IntegralBillService integralBillService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private OrderBuyService orderBuyService;
    @Autowired
    private IosVerifyUtil iosVerifyUtil;
    @Autowired
    private ApplePriceSettingService applePriceSettingService;
    @Autowired
    private NoteSendUtil sendUtil;

    @PostMapping("/register")
    @OperationLog(name = "注册：用户注册", type = OperationLogType.ADD)
    @ApiOperation(value = "注册：用户注册", response = ApiResult.class)
    @WebLoginToken
    public Map<String, String> register(@Validated RegistersParam registersParam) throws Exception {
        String token = userService.registerUsers(registersParam);
        Map<String, String> map = new HashMap<>(1);
        map.put("authToken", token);
        return map;
    }

    @PostMapping("/login")
    @OperationLog(name = "登录：用户登录", type = OperationLogType.INFO)
    @ApiOperation(value = "登录：用户登录", response = LoginTokenVo.class)
    @WebLoginToken
    public LoginTokenVo login(@Validated UserLoginsParam userLoginsParam) throws Exception {
        LoginTokenVo loginTokenVo = userService.getOneByPhones(userLoginsParam);
        return loginTokenVo;
    }

    @PostMapping("/loginByCode")
    @OperationLog(name = "登录：用户登录(验证码)", type = OperationLogType.INFO)
    @ApiOperation(value = "登录：用户登录(验证码)", response = LoginTokenVo.class)
    @WebLoginToken
    public LoginTokenVo loginByCode(@Validated UserLoginsParam userLoginsParam) throws Exception {
        LoginTokenVo loginTokenVo = userService.getOneByPhoneCode(userLoginsParam);
        return loginTokenVo;
    }

    @PostMapping("/sendNote")
    @OperationLog(name = "验证码：发送验证码给指定手机号码", type = OperationLogType.NOTE_MESSAGING)
    @ApiOperation(value = "验证码：发送验证码给指定手机号码", response = ApiResult.class)
    @WebLoginToken
    public void getCode(@RequestParam String phone) throws Exception {
        try {
            if (!StringUtils.isPhone(phone) || StringUtils.isBlank(phone)) {
                new AndroidVerifyException(ApiCode.PARAMETER_PHONE_VERIFY_ERR);
            }
            int code = Integer.parseInt(NoteSendUtil.getRandom(6));
            //将当前用户phone作为key，随机数字验证码作为value，存储十五分钟过期
            sendUtil.sendSms(phone, code);
            redisUtils.set(phone, code, 900);
        } catch (Exception e) {
            throw new AndroidBusinessException(e.getMessage(), ApiCode.SEND_NOTE_ERR);
        }
    }

    @PostMapping("/modifyPassword")
    @OperationLog(name = "忘记密码", type = OperationLogType.UPDATE)
    @ApiOperation(value = "忘记密码", response = ApiResult.class)
    @WebLoginToken
    public String modifyPassword(@Validated ModifyPasswordParam modifyPasswordParam) {
        userService.modifyPassword(modifyPasswordParam);
        return null;
    }

    @PostMapping("/updateCompany")
    @OperationLog(name = "编辑企业号信息", type = OperationLogType.UPDATE)
    @ApiOperation(value = "编辑企业号信息", response = ApiResult.class)
    public void updateCompany(@AnalysisUserId Long userId, @Validated UpdateCompanyParam updateCompanyParam) throws Exception {
        userService.updateCompany(updateCompanyParam, userId);
    }

    @GetMapping("/getPrice")
    @OperationLog(name = "支付：价格", type = OperationLogType.PAY)
    @ApiOperation(value = "支付：价格", response = BaseSetting.class)
    @WebLoginToken
    public BaseSetting getPrice(HttpServletRequest request) {
        BaseSetting baseSetting = baseSettingService.getOne();
        log.info("当前安装版本====》》》" + baseSetting.getAppVersion());
        System.out.println("当前安装版本====》》》" + baseSetting.getAppVersion());
        String version = request.getHeader("version");
        if (StringUtils.isNotBlank(version)) {
            log.info("APP当前安装版本====》》》" + version);
            System.out.println("APP当前安装版本====》》》" + version);
            if (!baseSetting.getAppVersion().equals(version)) {
                baseSetting.setIsRenewal(1);
            }
        }
        return baseSetting;
    }

    /**
     * @Function: 去支付
     */
    @PostMapping("/wechatPays")
    @OperationLog(name = "支付：微信支付(开通企业号)", type = OperationLogType.PAY)
    @ApiOperation(value = "支付：微信支付（开通企业号）", response = ApiResult.class)
    public Map<String, Object> wechatPays(@AnalysisUserId Long userId) {
        Map<String, Object> map = memberRecordsService.goWeChatPay(userId, 3, getRequest());
        if (Integer.parseInt(map.get("code").toString()) == 500) {
            return null;
        }
        return (Map) map.get("data");
    }


    @PostMapping("/rechargePays")
    @OperationLog(name = "支付：微信支付(企业积分充值)", type = OperationLogType.PAY)
    @ApiOperation(value = "支付：微信支付（企业积分充值）", response = ApiResult.class)
    public Map<String, Object> rechargePays(@AnalysisUserId Long userId, BigDecimal rechargeAmount) {
        redisUtils.set("recharge:" + userId, rechargeAmount);
        Map<String, Object> map = memberRecordsService.goWeChatPay(userId, 4, getRequest());
        if (Integer.parseInt(map.get("code").toString()) == 500) {
            return null;
        }
        return (Map) map.get("data");
    }

    @GetMapping("/getPageList")
    @OperationLog(name = "职位：分页获取职位列表", type = OperationLogType.PAGE)
    @ApiOperation(value = "职位：分页获取职位列表", response = PostQueryVo.class)
    public ApiPageResult<PostQueryVo> getList(@AnalysisUserId Long userId, @Validated MyPostPageParam postPageParam) throws Exception {
        postPageParam.setUserId(userId);
        ApiPageResult<PostQueryVo> postQueryVoApiPageResult = postService.getPostVos(postPageParam);
        List<PostQueryVo> postQueryVos = postQueryVoApiPageResult.getRecords();
        if (postQueryVos != null && postQueryVos.size() > 0) {
            for (PostQueryVo postQueryVo : postQueryVos) {
                String updateTime = postQueryVo.getUpdateTime().substring(0, 16);
                postQueryVo.setUpdateTime(updateTime);
            }
        }
        return postQueryVoApiPageResult;
    }

    @PostMapping("/addOrUpdatePost")
    @OperationLog(name = "职位：创建/修改职位", type = OperationLogType.ADD)
    @ApiOperation(value = "职位：创建/修改职位", response = ApiResult.class)
    public Boolean addOrUpdatePost(@AnalysisUserId Long userId, AddPostParam addPostParam) throws Exception {
        //企业用户
        User user = userService.getById(userId);
        if (addPostParam.getPhone().length() != 11) {
            throw new AndroidVerifyException(ApiCode.PHONE_ERR);
        }
        Post post = new Post();
        BeanUtils.copyProperties(addPostParam, post);
        //图片上传
        if (addPostParam.getImages() != null) {
            String images = upLoadService.uploadToOOS(addPostParam.getImages());
            if (StringUtils.isNotBlank(images)) {
                post.setImage(images);
            }
        }
        //去掉地址所带“-”
        if (StringUtils.isNotBlank(addPostParam.getAddress())) {
            String address = addPostParam.getAddress().replace("-", "");
            post.setAddress(address);
        }
        if (user != null) {
            if (user.getCompanyId() == 0) {
                throw new AndroidVerifyException(ApiCode.NOT_PREFECT);
            }
            if (StringUtils.isBlank(addPostParam.getName()) || StringUtils.isBlank(addPostParam.getCompensation()) || StringUtils.isBlank(addPostParam.getAddress()) || StringUtils.isBlank(addPostParam.getRecruiter())
                    || StringUtils.isBlank(addPostParam.getDetails()) || StringUtils.isBlank(addPostParam.getOpenDay()) || "开始时间".equals(addPostParam.getOpenDay()) || StringUtils.isBlank(addPostParam.getEndDay()) || "结束时间".equals(addPostParam.getEndDay())) {
                throw new AndroidVerifyException(ApiCode.PREFECT_INFORMATION);
            }
            Company company = companyService.getById(user.getCompanyId());
            if (company != null) {
                post.setCompanyId(company.getId());
                post.setCompanyName(company.getName());
            }
            //修改后改为待审核
            post.setAuditStatus(1);
            post.setUpdateTime(new Date());
            if (post.getId() != null) {
                Post oldPost = postService.getById(post.getId());
                //扣除用户积分
                int amount;
                int days;
                int amounts;
                if (post.getType().equals("1")) {
                    amount = post.getPeopleNumber() * post.getCharge().intValue();
                    days = post.getDayNumber() * 30 + 30;
                    post.setDescription("推荐官推荐1人入职" + post.getDayNumber() + "天可得" + post.getCharge().intValue() + "元");
                } else {
                    amount = post.getPeopleNumber() * post.getDayNumber() * post.getCharge().intValue();
                    days = post.getDayNumber() + 30;
                    post.setDescription("推荐官推荐1人入职" + post.getDayNumber() + "天可得" + post.getCharge().intValue() + "元");
                }
                if (oldPost.getType().equals("1")) {
                    amounts = oldPost.getPeopleNumber() * oldPost.getCharge().intValue();
                } else {
                    amounts = oldPost.getPeopleNumber() * oldPost.getDayNumber() * oldPost.getCharge().intValue();
                }
                int a = amount - amounts;
                if (a > 0) {
                    if (user.getIntegral() < a) {
                        throw new AndroidVerifyException(ApiCode.INTEGRAL_LIMITED);
                    }
                    post.setTotalCharge(amount);
                    //截至日期
                    Calendar calendar = Calendar.getInstance();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                    calendar.add(Calendar.DATE, days);
                    String dateStr = sdf.format(calendar.getTime());
                    Date deadline = sdf.parse(dateStr);
                    post.setDeadline(deadline);
                    int integral = user.getIntegral() - a;
                    user.setIntegral(integral);
                    userService.updateUser(user);

                    //积分记录
                    IntegralBill integralBill = new IntegralBill();
                    integralBill.setTitle("企业职位发布");
                    integralBill.setAmount("-" + a);
                    integralBill.setInOut(2);
                    integralBill.setUid(user.getId());
                    integralBill.setUname(user.getUserName());
                    integralBill.setPhone(user.getPhone());
                    integralBill.setCreateTime(new Date());
                    integralBill.setType(5);
                    integralBillService.save(integralBill);
                }
                if (a < 0) {
                    int b = amounts - amount;
                    post.setTotalCharge(amount);
                    //截至日期
                    Calendar calendar = Calendar.getInstance();
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                    calendar.add(Calendar.DATE, days);
                    String dateStr = sdf.format(calendar.getTime());
                    Date deadline = sdf.parse(dateStr);
                    post.setDeadline(deadline);
                    int integral = user.getIntegral() + b;
                    user.setIntegral(integral);
                    userService.updateUser(user);

                    //积分记录
                    IntegralBill integralBill = new IntegralBill();
                    integralBill.setTitle("企业职位发布");
                    integralBill.setAmount("+" + b);
                    integralBill.setInOut(1);
                    integralBill.setUid(user.getId());
                    integralBill.setUname(user.getUserName());
                    integralBill.setPhone(user.getPhone());
                    integralBill.setCreateTime(new Date());
                    integralBill.setType(5);
                    integralBillService.save(integralBill);
                }
                postService.updatePost(post);
            } else {
                //扣除用户积分
                int amount;
                int days;
                if (post.getType().equals("1")) {
                    amount = post.getPeopleNumber() * post.getCharge().intValue();
                    days = post.getDayNumber() * 30 + 30;
                    post.setDescription("推荐官推荐1人入职" + post.getDayNumber() + "天可得" + post.getCharge().intValue() + "元");
                } else {
                    amount = post.getPeopleNumber() * post.getDayNumber() * post.getCharge().intValue();
                    days = post.getDayNumber() + 30;
                    post.setDescription("推荐官推荐1人入职" + post.getDayNumber() + "天可得" + post.getCharge().intValue() + "元");
                }
                if (user.getIntegral() < amount) {
                    throw new AndroidVerifyException(ApiCode.INTEGRAL_LIMITED);
                }
                post.setTotalCharge(amount);
                postService.savePost(post);
                int integral = user.getIntegral() - amount;
                user.setIntegral(integral);

                //截至日期
                Calendar calendar = Calendar.getInstance();
                SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
                calendar.add(Calendar.DATE, days);
                String dateStr = sdf.format(calendar.getTime());
                Date deadline = sdf.parse(dateStr);
                post.setDeadline(deadline);
                userService.updateUser(user);

                //积分记录
                IntegralBill integralBill = new IntegralBill();
                integralBill.setTitle("企业职位发布");
                integralBill.setAmount("-" + amount);
                integralBill.setInOut(2);
                integralBill.setUid(user.getId());
                integralBill.setUname(user.getUserName());
                integralBill.setPhone(user.getPhone());
                integralBill.setCreateTime(new Date());
                integralBill.setType(5);
                integralBillService.save(integralBill);
            }
        }
        return null;
    }

    @PostMapping("/postUpOrDown")
    @OperationLog(name = "职位：职位上架/下架", type = OperationLogType.ADD)
    @ApiOperation(value = "职位：职位上架/下架", response = ApiResult.class)
    @WebLoginToken
    public Boolean postUpOrDown(@AnalysisUserId Long userId, @Validated PostUpOrDownParam postUpOrDownParam) {
        //企业用户
//        User user = userService.getById(userId);
//        if (user != null) {
//            if (user.getIntegral() <= 0) {
//                throw new AndroidVerifyException(ApiCode.INTEGRAL_LIMITED);
//            }
//        }
        Post post = postService.getById(postUpOrDownParam.getId());
        if (post != null) {
            post.setStatus(postUpOrDownParam.getStatus());
            post.setUpdateTime(new Date());
            postService.saveOrUpdate(post);
        }
        return null;
    }

    /**
     * 获取职位详情
     */
    @GetMapping("/info")
    @OperationLog(name = "职位：获取职位详情", type = OperationLogType.INFO)
    @ApiOperation(value = "职位：获取职位详情", response = PostVo.class)
    @WebLoginToken
    public PostVo getPost(Integer postId) {
        if (postId == null || postId <= 0) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_VERIFY_ERR);
        }
        Post post = postService.getById(postId);
        PostVo postVo = new PostVo();
        BeanUtils.copyProperties(post, postVo);
        if (postVo.getVideo().equals("null")) {
            postVo.setVideo(null);
        }
        return postVo;
    }

    /**
     * 删除职位
     */
    @GetMapping("/deletePost")
    @OperationLog(name = "职位：删除职位", type = OperationLogType.DELETE)
    @ApiOperation(value = "职位：删除职位", response = ApiResult.class)
    @WebLoginToken
    public Boolean deletePost(Integer postId) {
        if (postId == null || postId <= 0) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_VERIFY_ERR);
        }
        postService.delete(postId);
        return null;
    }

    @GetMapping("/getApply")
    @OperationLog(name = "职位申请：获取申请列表(status : 1 待审核 3 审核通过)", type = OperationLogType.LIST)
    @ApiOperation(value = "职位申请：获取申请列表(status : 1 待审核 3 审核通过)", response = ApplyParam.class)
    public List<ApplyParam> getApply(@AnalysisUserId Long userId, @RequestParam("status") Integer status, Integer postId) {
        User user = userService.getById(userId);
        List<ApplyParam> applyParams = new LinkedList<>();
        if (user != null) {
            LambdaQueryWrapper<Apply> wrapper = new LambdaQueryWrapper<>();
            wrapper.eq(Apply::getCompanyId, user.getCompanyId()).eq(Apply::getStatus, status).orderByDesc(Apply::getCreateTime);
            if (status == 3 && postId != null && postId > 0) {
                wrapper.eq(Apply::getPostId, postId);
            }
            List<Apply> applyList = applyService.list(wrapper);
            if (applyList != null && applyList.size() > 0) {
                for (Apply apply : applyList) {
                    ApplyParam applyParam = new ApplyParam();
                    User users = userService.getById(apply.getUserId());
                    if (users != null) {
                        BeanUtils.copyProperties(users, applyParam);
                    }
                    applyParam.setId(apply.getId());
                    applyParam.setPostId(apply.getPostId());
                    applyParam.setPostName(apply.getPostName());
                    applyParam.setSignUpTime(apply.getCreateTime());
                    applyParam.setIpAddress(apply.getIpAddress());
                    applyParam.setUserId(apply.getUserId());
                    applyParams.add(applyParam);
                }
            }
        }
        return applyParams;
    }

    @GetMapping("/getApplyInfo")
    @OperationLog(name = "职位申请：申请详情", type = OperationLogType.LIST)
    @ApiOperation(value = "职位申请：申请详情", response = ApplyParam.class)
    @WebLoginToken
    public ApplyParam getApplyInfo(String id) {
        Apply apply = applyService.getById(id);
        if (apply != null) {
            ApplyParam applyParam = new ApplyParam();
            User user = userService.getById(apply.getUserId());
            if (user != null) {
                BeanUtils.copyProperties(user, applyParam);
            }
            applyParam.setId(apply.getId());
            applyParam.setPostId(apply.getPostId());
            applyParam.setPostName(apply.getPostName());
            applyParam.setSignUpTime(apply.getCreateTime());
            applyParam.setIpAddress(apply.getIpAddress());
            applyParam.setUserId(apply.getUserId());
            applyParam.setPhone(apply.getUserPhone());
            return applyParam;
        }
        return null;
    }

    @PostMapping("/auditApply")
    @OperationLog(name = "职位申请：申请审核", type = OperationLogType.LIST)
    @ApiOperation(value = "职位申请：申请审核", response = ApiResult.class)
    @WebLoginToken
    public Boolean auditApply(@Validated ApplyAuditParam applyAuditParam) {
        Apply apply = applyService.getById(applyAuditParam.getId());
        if (apply != null) {
            if (applyAuditParam.getStatus() == 2) {
                applyService.delete(apply.getId().longValue());
            } else {
                List<Apply> applies = applyService.list(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, apply.getUserId()).eq(Apply::getStatus, 3));
                if (applies != null && applies.size() > 0) {
                    throw new AndroidVerifyException(ApiCode.REPETITION_APPLY);
                }
                BeanUtils.copyProperties(applyAuditParam, apply);
                applyService.saveOrUpdate(apply);
            }
        }
        return null;
    }

    @GetMapping("/deleteApply")
    @OperationLog(name = "职位申请：删除报名申请", type = OperationLogType.DELETE)
    @ApiOperation(value = "职位申请：删除报名申请", response = ApiResult.class)
    @WebLoginToken
    public Boolean deleteApply(String id) {
        applyService.delete(Long.valueOf(id));
        return null;
    }

    @GetMapping("/deleteUserApply")
    @OperationLog(name = "职位申请：用户取消报名申请", type = OperationLogType.DELETE)
    @ApiOperation(value = "职位申请：用户取消报名申请", response = ApiResult.class)
    public Boolean deleteUserApply(@AnalysisUserId Long userId, String postId) {
        Apply apply = applyService.getOne(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, userId).eq(Apply::getPostId, postId));
        if (apply != null) {
            applyService.delete(Long.valueOf(apply.getId()));
        }
        return null;
    }

    @GetMapping("/getPunchMessage")
    @OperationLog(name = "打卡：获取打卡信息", type = OperationLogType.INFO)
    @ApiOperation(value = "打卡：获取打卡信息", response = PostLocationParam.class)
    public PostLocationParam getPunchMessage(@AnalysisUserId Long userId) {
        List<Apply> applies = applyService.list(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, userId).eq(Apply::getStatus, 3).orderByDesc(Apply::getCreateTime));
        PostLocationParam postLocationParam = new PostLocationParam();
        //是否自动打卡
        postLocationParam.setIsAutoPunch(1);
        //打卡范围
        BaseSetting baseSetting = baseSettingService.getOne();
        postLocationParam.setRadius(baseSetting.getClockDistance());
        //默认打卡地址
        postLocationParam.setLongitude("116.397451");
        postLocationParam.setLatitude("39.909187");
        if (applies != null && applies.size() > 0) {
            Apply apply = applies.get(0);
            Post post = postService.getById(apply.getPostId());
            if (post != null) {
                BeanUtils.copyProperties(post, postLocationParam);
            }
        }
        //判断是否打卡
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String date = sdf.format(new Date());
        SimpleDateFormat sdf1 = new SimpleDateFormat("HH:mm");
        List<PunchCard> punchCards = punchCardService.list(new LambdaQueryWrapper<PunchCard>().eq(PunchCard::getUserId, userId).like(PunchCard::getCreateTime, date).eq(PunchCard::getClockPeriod, 1));
        if (punchCards != null && punchCards.size() > 0) {
            postLocationParam.setMPunchTime(sdf1.format(punchCards.get(0).getCreateTime()));
        }
        List<PunchCard> punchCardss = punchCardService.list(new LambdaQueryWrapper<PunchCard>().eq(PunchCard::getUserId, userId).like(PunchCard::getCreateTime, date).eq(PunchCard::getClockPeriod, 2));
        if (punchCardss != null && punchCardss.size() > 0) {
            postLocationParam.setAPunchTime(sdf1.format(punchCardss.get(0).getCreateTime()));
        }
        return postLocationParam;
    }

    @PostMapping("/punchCard")
    @OperationLog(name = "打卡：打卡", type = OperationLogType.INFO)
    @ApiOperation(value = "打卡：打卡", response = ApiResult.class)
    public Map punchCard(@AnalysisUserId Long userId, @Validated PunchCardParam punchCardParam) throws Exception {
        List<Apply> applies = applyService.list(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, userId).eq(Apply::getStatus, 3).orderByDesc(Apply::getCreateTime));
        Map map = new HashMap();
        map.put("punchType", 0);
        if (applies != null && applies.size() > 0) {
            Apply apply = applies.get(0);
            BaseSetting baseSetting = baseSettingService.getOne();
            PunchCard punchCard = new PunchCard();
            BeanUtils.copyProperties(apply, punchCard);

            Post post = postService.getById(apply.getPostId());
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
            String date = sdf.format(new Date());

            int count = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getUserId, userId).eq(PunchCard::getClockPeriod, 1));
            if (count < 1) {
                //上班打卡
                punchCard.setClockPeriod(1);
                redisUtils.set(userId.toString(), true, 180);
            } else {
                int mCount = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getUserId, userId).eq(PunchCard::getClockPeriod, 1));
                int aCount = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getUserId, userId).eq(PunchCard::getClockPeriod, 2));
                Object obj = redisUtils.get(userId.toString());
                //判断是否已打下班卡
                if (aCount == 0 && StringUtils.isEmpty(obj)) {
                    //下班打卡
                    punchCard.setClockPeriod(2);
                    //完成上班打卡，赠送积分
                    if (mCount > 0) {
                        User user = userService.getById(userId);
                        if (user != null) {
                            //推荐人查询
                            User users = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getPhone, user.getInvitationCode()));
                            if (users != null && users.getIsReferrer() == 2) {
                                if (post.getType().equals("2") && post.getTotalCharge() >= post.getCharge().intValue()) {
                                    users.setIntegral(users.getIntegral() + post.getCharge().intValue());
                                    userService.updateUser(user);
                                    post.setTotalCharge(post.getTotalCharge() - post.getCharge().intValue());
                                    postService.updatePost(post);
                                    //积分记录
                                    IntegralBill integralBill = new IntegralBill();
                                    integralBill.setTitle("推荐打卡奖励");
                                    integralBill.setAmount("+" + post.getCharge());
                                    integralBill.setInOut(1);
                                    integralBill.setUid(users.getId());
                                    integralBill.setUname(users.getUserName());
                                    integralBill.setPhone(users.getPhone());
                                    integralBill.setCreateTime(new Date());
                                    integralBill.setType(4);
                                    integralBillService.save(integralBill);
                                }

                                //企业号
                                User cUser = userService.getOne(new LambdaQueryWrapper<User>().eq(User::getCompanyId, post.getCompanyId()));
                                if (cUser != null && cUser.getIntegral() >= baseSetting.getClockReferrerIntegral()) {
                                    users.setIntegral(users.getIntegral() + baseSetting.getClockReferrerIntegral());
                                    userService.updateUser(user);
                                    cUser.setIntegral(cUser.getIntegral() - baseSetting.getClockReferrerIntegral());
                                    userService.updateUser(cUser);
                                    //积分记录
                                    IntegralBill integralBill2 = new IntegralBill();
                                    integralBill2.setTitle("推荐打卡奖励");
                                    integralBill2.setAmount("+" + baseSetting.getClockReferrerIntegral());
                                    integralBill2.setInOut(1);
                                    integralBill2.setUid(users.getId());
                                    integralBill2.setUname(users.getUserName());
                                    integralBill2.setPhone(users.getPhone());
                                    integralBill2.setCreateTime(new Date());
                                    integralBill2.setType(4);
                                    integralBillService.save(integralBill2);
                                    IntegralBill integralBill3 = new IntegralBill();
                                    integralBill3.setTitle("推荐打卡奖励消耗");
                                    integralBill3.setAmount("-" + baseSetting.getClockReferrerIntegral());
                                    integralBill3.setInOut(2);
                                    integralBill3.setUid(cUser.getId());
                                    integralBill3.setUname(cUser.getUserName());
                                    integralBill3.setPhone(cUser.getPhone());
                                    integralBill3.setCreateTime(new Date());
                                    integralBill3.setType(4);
                                    integralBillService.save(integralBill3);
                                }
                            }
                        }
                    }
                } else {
                    throw new AndroidVerifyException(ApiCode.PUNCH_SUCCESS);
                }
            }
            BeanUtils.copyProperties(punchCardParam, punchCard);
            punchCardService.savePunchCard(punchCard);
            map.put("punchType", punchCard.getPunchType());
        }
        return map;
    }

    @PostMapping("/punchRecord")
    @OperationLog(name = "打卡：考勤统计", type = OperationLogType.INFO)
    @ApiOperation(value = "打卡：考勤统计", response = ApiResult.class)
    public Map punchRecord(@AnalysisUserId Long userId, Integer postId) {
        Map map = new HashMap();
        map.put("totalNumber", 0);
        map.put("tardyNumber", 0);
        map.put("earlyNumber", 0);
        map.put("lackNumber", 0);
        User user = userService.getById(userId);
        if (user != null && user.getCompanyId() > 0) {
            Company company = companyService.getById(user.getCompanyId());
            if (company != null) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                String date = sdf.format(new Date());
                if (postId != null && postId > 0) {
                    int totalNumber = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getCompanyId, company.getId()).eq(PunchCard::getPostId, postId));
                    map.put("totalNumber", totalNumber);
                    int tardyNumber = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getCompanyId, company.getId()).eq(PunchCard::getPostId, postId).eq(PunchCard::getPunchType, 3));
                    map.put("tardyNumber", tardyNumber);
                    int earlyNumber = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getCompanyId, company.getId()).eq(PunchCard::getPostId, postId).eq(PunchCard::getPunchType, 2));
                    map.put("earlyNumber", earlyNumber);
                    //理论总打卡次数
                    int total = applyService.count(new LambdaQueryWrapper<Apply>().eq(Apply::getCompanyId, company.getId()).eq(Apply::getPostId, postId).eq(Apply::getStatus, 3));
                    int lackNumber = total * 2 - totalNumber;
                    map.put("lackNumber", lackNumber);
                } else {
                    int totalNumber = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getCompanyId, company.getId()));
                    map.put("totalNumber", totalNumber);
                    int tardyNumber = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getCompanyId, company.getId()).eq(PunchCard::getPunchType, 3));
                    map.put("tardyNumber", tardyNumber);
                    int earlyNumber = punchCardService.count(new LambdaQueryWrapper<PunchCard>().like(PunchCard::getCreateTime, date).eq(PunchCard::getCompanyId, company.getId()).eq(PunchCard::getPunchType, 2));
                    map.put("earlyNumber", earlyNumber);
                    //理论总打卡次数
                    int total = applyService.count(new LambdaQueryWrapper<Apply>().eq(Apply::getCompanyId, company.getId()).eq(Apply::getStatus, 3));
                    int lackNumber = total * 2 - totalNumber;
                    map.put("lackNumber", lackNumber);
                }
                return map;
            }
        }
        return map;
    }


    /**
     * 解析时间为Date格式
     *
     * @param time 时间字符串
     * @return Date
     */
    private static Date parseDate(String time) {
        String[] temp = time.split(":");
        Calendar calendar = Calendar.getInstance();
        calendar.set(Calendar.HOUR_OF_DAY, Integer.parseInt(temp[0]));
        calendar.set(Calendar.MINUTE, Integer.parseInt(temp[1]));
        calendar.set(Calendar.SECOND, 0);
        return calendar.getTime();
    }


    @GetMapping("/getPunch")
    @OperationLog(name = "打卡记录", type = OperationLogType.DELETE)
    @ApiOperation(value = "打卡记录", response = PunchCard.class)
    public List<PunchCard> getPunch(@AnalysisUserId Long userId) {
        User user = userService.getById(userId);
        if (user != null) {
            Company company = companyService.getById(user.getCompanyId());
            if (company != null) {
                SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                String date = sdf.format(new Date());
                List<PunchCard> punchCards = punchCardService.list(new LambdaQueryWrapper<PunchCard>().eq(PunchCard::getCompanyId, company.getId()).orderByDesc(PunchCard::getCreateTime).groupBy(PunchCard::getUserId));
                if (punchCards != null && punchCards.size() > 0) {
                    for (PunchCard punchCard : punchCards) {
                        List<PunchCard> punchCards1 = punchCardService.list(new LambdaQueryWrapper<PunchCard>().eq(PunchCard::getCompanyId, company.getId()).eq(PunchCard::getUserId, punchCard.getUserId()).like(PunchCard::getCreateTime, date).eq(PunchCard::getClockPeriod, 1));
                        List<PunchCard> punchCards2 = punchCardService.list(new LambdaQueryWrapper<PunchCard>().eq(PunchCard::getCompanyId, company.getId()).eq(PunchCard::getUserId, punchCard.getUserId()).like(PunchCard::getCreateTime, date).eq(PunchCard::getClockPeriod, 2).orderByDesc(PunchCard::getCreateTime));
                        SimpleDateFormat sdf1 = new SimpleDateFormat("HH:mm");
                        SimpleDateFormat sdf2 = new SimpleDateFormat("yyyy年MM月dd日");
                        String dateStr = sdf2.format(new Date());
                        if (punchCards1 != null && punchCards1.size() > 0) {
                            String punchTime = dateStr + ":" + sdf1.format(punchCards1.get(0).getCreateTime()) + "-";
                            punchCard.setPunchTime(punchTime);
                        }
                        if (punchCards2 != null && punchCards2.size() > 0) {
                            String punchTime = dateStr + ":" + "-" + sdf1.format(punchCards2.get(0).getCreateTime());
                            punchCard.setPunchTime(punchTime);
                        }
                        if (punchCards1 != null && punchCards1.size() > 0 && punchCards2 != null && punchCards2.size() > 0) {
                            String punchTime = dateStr + ":" + sdf1.format(punchCards1.get(0).getCreateTime()) + "-" + sdf1.format(punchCards2.get(0).getCreateTime());
                            punchCard.setPunchTime(punchTime);
                        }
                    }
                }
                return punchCards;
            }
        }
        return null;
    }


    @GetMapping("/buyPostInfo")
    @OperationLog(name = "购买职位信息", type = OperationLogType.ADD)
    @ApiOperation(value = "购买职位信息", response = ApiResult.class)
    public Boolean deleteApply(@AnalysisUserId Long userId, String postId) throws Exception {
        User user = userService.getById(userId);
        BaseSetting baseSetting = baseSettingService.getOne();
        if (user != null) {
            int count = orderBuyService.count(new LambdaQueryWrapper<OrderBuy>().eq(OrderBuy::getUserId, userId).eq(OrderBuy::getPostId, postId).eq(OrderBuy::getDeleted, 0));
            if (count == 0) {
                if (user.getIntegral() < baseSetting.getConsumeIntegral()) {
                    throw new AndroidVerifyException(ApiCode.INTEGRAL_INSUFFIVIENT);
                }
                //减去积分
                user.setIntegral(user.getIntegral() - baseSetting.getConsumeIntegral());
                userService.updateUser(user);
                //添加购买记录
                OrderBuy order = new OrderBuy();
                order.setUserId(userId.intValue());
                order.setPostId(Integer.valueOf(postId));
                orderBuyService.save(order);
                //积分记录
                IntegralBill integralBill = new IntegralBill();
                integralBill.setTitle("用户购买职位信息");
                integralBill.setAmount("-" + baseSetting.getConsumeIntegral());
                integralBill.setInOut(2);
                integralBill.setUid(user.getId());
                integralBill.setUname(user.getUserName());
                integralBill.setPhone(user.getPhone());
                integralBill.setCreateTime(new Date());
                integralBill.setType(4);
                integralBillService.save(integralBill);
            }
        }
        return null;
    }


    @OperationLog(name = "客户端内购支付", type = OperationLogType.ADD)
    @ApiOperation(value = "客户端内购支付", response = ApiResult.class)
    @PostMapping("/applePay")
    public void applePay(HttpServletRequest request, @AnalysisUserId Long userId, @Validated ApplePayVo applePayVo) throws Exception {
        String receipt_data = applePayVo.getReceiptData();
        String transactionID = applePayVo.getTransactionID();
        String verifyResult = iosVerifyUtil.buyAppVerify(receipt_data, 1);
        //苹果服务器没有返回验证结果
        if (verifyResult == null) {
            throw new AndroidVerifyException(ApiCode.PAY_ERR);
        } else {
            // 苹果验证有返回结果
            JSONObject job = JSONObject.parseObject(verifyResult);
            System.out.println("这里是苹果返回的结果：" + job.toString());
            String states = job.getString("status");
            //获取苹果系统服务器验证结果数据处理自己的业务
            applePayVo.setUserId(userId);
            getAppPay(states, job, transactionID, request, applePayVo);
        }
    }

    @OperationLog(name = "苹果价格设置", type = OperationLogType.ADD)
    @ApiOperation(value = "苹果价格设置", response = ApiResult.class)
    @PostMapping("/getApplePrice")
    @WebLoginToken
    public List<ApplePriceSetting> getApplePrice() {
        List<ApplePriceSetting> applePriceSettings = applePriceSettingService.list();
        return applePriceSettings;
    }

    /**
     * 苹果支付验证后
     **/
    public void getAppPay(String states, JSONObject job, String transactionID, HttpServletRequest request, ApplePayVo applePayVo) throws Exception {
        if ("0".equals(states)) {
            //app端所提供的收据是有效的    验证成功
            String r_receipt = job.getString("receipt");
            JSONObject returnJson = JSONObject.parseObject(r_receipt);
            String in_app = returnJson.getString("in_app");
//            JSONObject in_appJson = JSONObject.parseObject(in_app.substring(1, in_app.length() - 1));
            JSONArray jsonArray = JSON.parseArray(in_app);
            JSONObject in_appJson = JSONObject.parseObject(jsonArray.getString(0));
            String transactionId = in_appJson.getString("transaction_id");
            //产品id
//            String productId = in_appJson.getString("product_id");
            String productId = applePayVo.getProductId();
            //苹果支付订单状态
            String in_app_ownership_type = in_appJson.getString("in_app_ownership_type");
            //如果获取验证后的支付订单单号与app端传来的支付订单号一致并且状态为已支付状态则处理自己的业务 transactionID.equals(transactionId) &&
            if (in_app_ownership_type.equals("PURCHASED")) {
                memberRecordsService.applePay(applePayVo.getUserId(), applePayVo.getProductType(), productId, request);
            }

        }
    }

    @OperationLog(name = "我的报名", type = OperationLogType.LIST)
    @ApiOperation(value = "我的报名", response = ApiResult.class)
    @GetMapping("/getMyApply")
    public List<Apply> getMyApply(@AnalysisUserId Long userId) {
        List<Apply> applies = applyService.list(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, userId).orderByDesc(Apply::getCreateTime));
        return applies;
    }


    @OperationLog(name = "推荐列表", type = OperationLogType.LIST)
    @ApiOperation(value = "推荐列表", response = ApiResult.class)
    @GetMapping("/getRecommendList")
    public List<User> getRecommendList(@AnalysisUserId Long userId) {
        User user = userService.getById(userId);
        if (user != null) {
            List<User> userList = userService.list(new LambdaQueryWrapper<User>().eq(User::getInvitationCode, user.getPhone()).orderByDesc(User::getCreateTime));
            if (userList != null && userList.size() > 0) {
                for (User u : userList) {
                    int count = applyService.count(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, u.getId()).eq(Apply::getStatus, 3));
                    if (count > 0) {
                        u.setApplyStatus(3);
                    } else {
                        int counts = applyService.count(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId, u.getId()).eq(Apply::getStatus, 1));
                        if (counts > 0) {
                            u.setApplyStatus(2);
                        }
                    }

                    if (StringUtils.isNotBlank(u.getPhone())) {
                        String phone = u.getPhone().substring(0, 7);
                        u.setPhone(phone + "****");
                    }
                }
            }
            return userList;
        }
        return new ArrayList<>();
    }

}