package com.cloud.web.controller;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.core.annotation.WebLoginToken;
import com.cloud.framework.util.SpringContextHolder;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.entity.User;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.modules.send.utils.NoteSendUtil;
import com.cloud.web.param.WeChatBody;
import com.cloud.web.params.param.RegisterParam;
import com.cloud.web.params.param.UpdateUserParam;
import com.cloud.web.params.param.UserLoginParam;
import com.cloud.web.service.UserService;
import com.cloud.web.util.RedisUtils;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.LoginTokenVo;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.common.controller.BaseController;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import org.springframework.validation.annotation.Validated;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.HashMap;
import java.util.Map;

/**
 * 控制器
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@RestController
@RequestMapping("/app/user")
@Module("web")
@Api(value = "API：会员中心", tags = {"API：会员中心"})
@ResponseResult
public class UserController extends BaseController {

    @Autowired
    private UserService userService;
    @Autowired
    private NoteSendUtil sendUtil;
    private RedisUtils redisUtils = SpringContextHolder.getBean(RedisUtils.class);

    @PostMapping("/register")
    @OperationLog(name = "用户注册", type = OperationLogType.ADD)
    @ApiOperation(value = "用户注册", response = ApiResult.class)
    @WebLoginToken
    public Map<String, String> register(@Validated RegisterParam registerParam) throws Exception {
        String token = userService.registerUser(registerParam);
        Map<String, String> map = new HashMap<>(1);
        map.put("authToken", token);
        return map;
    }

    @PostMapping("/sendNote")
    @OperationLog(name = "验证码：发送验证码给指定手机号码", type = OperationLogType.NOTE_MESSAGING)
    @ApiOperation(value = "验证码：发送验证码给指定手机号码", response = ApiResult.class)
    @WebLoginToken
    public void getCode(@RequestParam String phone) throws Exception {
        try {
            if (!StringUtils.isPhone(phone) || StringUtils.isBlank(phone)) {
                new AndroidVerifyException(ApiCode.PARAMETER_PHONE_VERIFY_ERR);
            }
            int code = Integer.parseInt(NoteSendUtil.getRandom(6));
            //将当前用户phone作为key，随机数字验证码作为value，存储十五分钟过期
            sendUtil.sendSms(phone, code);
            redisUtils.set(phone, code, 900);
        } catch (Exception e) {
            throw new AndroidBusinessException(e.getMessage(), ApiCode.SEND_NOTE_ERR);
        }
    }

    @PostMapping("/login")
    @OperationLog(name = "登录：用户登录", type = OperationLogType.INFO)
    @ApiOperation(value = "登录：用户登录", response = LoginTokenVo.class)
    @WebLoginToken
    public LoginTokenVo login(@Validated UserLoginParam userLoginParam) throws Exception {
        LoginTokenVo loginTokenVo = userService.getOneByPhone(userLoginParam);
        return loginTokenVo;
    }

    @PostMapping("/update/message")
    @OperationLog(name = "更新会员信息", type = OperationLogType.UPDATE)
    @ApiOperation(value = "更新会员信息", response = ApiResult.class)
    public void updateUserMessage(@AnalysisUserId Long userId, @Validated UpdateUserParam updateUserParam) throws Exception {
        userService.updateUser(updateUserParam, userId);
    }


    @ApiOperation(value = "获取授权地址")
    @PostMapping("authorizeAddress")
    @WebLoginToken
    public String authorizeAddress(@RequestBody WeChatBody weChatBody) {
        String openAppId = "wxa3355ee024c4b87d";
        if (StringUtils.isNotBlank(openAppId)) {
            String url = "https://open.weixin.qq.com/connect/oauth2/authorize?appid=";
            String redirect_uri = weChatBody.getRedirectUri();
            StringBuilder sb = new StringBuilder(url);
            sb.append(openAppId);
            sb.append("&redirect_uri=").append(redirect_uri);
            sb.append("&response_type=code&scope=").append(weChatBody.getScope());
            sb.append("&state=").append("STATE");
            sb.append("#wechat_redirect");
            System.out.println(sb.toString());
            return sb.toString();
        }
        return null;
    }


    @ApiOperation(value = "微信授权")
    @GetMapping("/authorization")
    public String authorization(@AnalysisUserId Long userId, String code) throws Exception {
        String openAppId = "wxa3355ee024c4b87d";
        String openSecret = "b35f4d5d9da382493d15f5880944df11";
        //获取access_token
        String url = "https://api.weixin.qq.com/sns/oauth2/access_token" +
                "?appid=" + openAppId +
                "&secret=" + openSecret +
                "&code=" + code +
                "&grant_type=authorization_code";
        String result = HttpUtil.get(url);
        //返回结果的json对象
        JSONObject resultObject = JSON.parseObject(result);
        String unionid = "";
        String openId = resultObject.getString("openid");
        if (resultObject.containsKey("unionid")) {
            unionid = resultObject.getString("unionid");
        }

        if (StringUtils.isNotBlank(unionid) && StringUtils.isNotBlank(openId)) {
            if (userId != null) {
                User user = userService.getById(userId);
                if (user != null) {
                    //请求获取userInfo
                    String infoUrl = "https://api.weixin.qq.com/sns/userinfo" +
                            "?access_token=" + resultObject.getString("access_token") +
                            "&openid=" + openId +
                            "&lang=zh_CN";
                    String resultInfo = HttpUtil.get(infoUrl);

                    //返回结果的json对象
                    JSONObject userObject = JSON.parseObject(resultInfo);
                    user.setHeadImg(userObject.getString("headimgurl"));
                    user.setUnionid(unionid);
                    user.setOpenid(openId);
                    userService.updateUser(user);
                }
            }
        }
        return null;
    }

}

