package com.cloud.web.entity;

import com.baomidou.mybatisplus.annotation.IdType;

import java.util.Date;

import com.baomidou.mybatisplus.annotation.Version;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.FieldFill;
import com.cloud.framework.common.entity.BaseEntity;
import com.baomidou.mybatisplus.annotation.TableLogic;
import com.baomidou.mybatisplus.annotation.TableField;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;

import com.cloud.framework.core.validator.groups.Update;

/**
 * @author zyl
 * @since 2021-01-08
 */
@Data
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "User对象")
public class User extends BaseEntity {
    private static final long serialVersionUID = 1L;

    @NotNull(message = "id不能为空", groups = {Update.class})
    @ApiModelProperty("id")
    @TableId(value = "id", type = IdType.AUTO)
    private Integer id;

    @ApiModelProperty("手机号")
    private String phone;

    @ApiModelProperty("密码")
    private String password;

    @ApiModelProperty("头像")
    private String headImg;

    @ApiModelProperty("用户名")
    private String userName;

    @ApiModelProperty("真实姓名")
    private String realName;

    @ApiModelProperty("身份证号")
    private String idCard;

    @ApiModelProperty("性别")
    private Integer sex;

    @ApiModelProperty("年龄")
    private Integer age;

    @ApiModelProperty("地址")
    private String address;

    @ApiModelProperty("工作经验")
    private String experience;

    @ApiModelProperty("用户昵称")
    private String nickName;

    @ApiModelProperty("自我评价")
    private String selfEvaluation;

    @ApiModelProperty("用户积分")
    private Integer integral;

    @ApiModelProperty("是否开通会员（1 未开通 2 开通）")
    private Integer isMember;

    @ApiModelProperty("邀请码")
    private String invitationCode;

    @ApiModelProperty("会员开通时间")
    private Date openingDate;

    @ApiModelProperty("创建者")
    private String createBy;

    @ApiModelProperty("创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty("更新者")
    private String updateBy;

    @ApiModelProperty("更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty("是否删除")
    @TableLogic
    private Boolean deleted;

    @ApiModelProperty("抽奖次数")
    private Integer drawNumber;

    @ApiModelProperty("是否开通推荐管（1 未开通 2 开通）")
    private Integer isReferrer;

    @ApiModelProperty("是否完善信息（1 否 2 是）")
    private Integer isChange;

    @ApiModelProperty("绑定公司ID")
    private Integer companyId;

    @ApiModelProperty("公司名称")
    private String companyName;

    @ApiModelProperty("是否是企业号（1 否 2 是）")
    private Integer isCompany;

    @ApiModelProperty("公司名logo")
    private String companyLogo;

    @ApiModelProperty("公司简介")
    private String companyIntro;

    @ApiModelProperty("简介图")
    private String image;

    @ApiModelProperty("unionid")
    private String unionid;

    @ApiModelProperty("openid")
    private String openid;

    @ApiModelProperty("状态 1 已邀请 2 已报名 3 已录用")
    @TableField(exist = false)
    private Integer applyStatus = 1;

    @ApiModelProperty("是否实名认证（0 否 1 是）")
    private Integer isUserAuthentication;

    @ApiModelProperty("是否企业认证（0 否 1 是）")
    private Integer isCompanyAuthentication;
}
