package com.cloud.web.interceptors;

import com.alibaba.fastjson.JSONObject;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.core.annotation.WebLoginToken;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.config.AndroidConfig;
import com.cloud.web.util.RedisUtils;
import com.cloud.web.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/***
 * 移动端登录拦截器
 */
@Component
@Slf4j
public class GlobalInterceptors implements HandlerInterceptor {

    private RedisUtils redisUtils;

    public GlobalInterceptors(RedisUtils redisUtils) {
        this.redisUtils = redisUtils;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        request.setCharacterEncoding("utf-8");
        String requestURI = request.getRequestURI();
        if (requestURI.equals(AndroidConfig.DEFAULT_INTERCEPTORS_URL + "wechat/call/back")) {
            return true;
        }
        //默认只拦截移动端接口
        if (!requestURI.contains(AndroidConfig.DEFAULT_INTERCEPTORS_URL)) {
            return true;
        }
        log.info("移动端请求的URI：{}，Method：{}", requestURI, request.getMethod());
        // 请求的方法是否有跳过验证注解
        boolean haveTokenAnnotation = handler.getClass().isAssignableFrom(HandlerMethod.class);
        if (haveTokenAnnotation) {
            // 检测是否有 @WebLoginToken 注解
            WebLoginToken webLoginToken = ((HandlerMethod) handler).getMethodAnnotation(WebLoginToken.class);
            if (webLoginToken != null) {
                return true;
            }
        }
        Object jsonObject;
        // 获取用户唯一标识
        String token = request.getHeader(AndroidConfig.TOKEN_NAME);
        //如请求头中没有token 在从请求参数中获取一遍
        if (StringUtils.isBlank(token)) {
            token = request.getParameter(AndroidConfig.TOKEN_NAME);
        }
        if (StringUtils.isNotBlank(token)) {
            boolean flag = redisUtils.hasKey(token);
            // 有效
            if (flag) {
                return true;
            } else {
                jsonObject = JSONObject.toJSON(ApiResult.fail(ApiCode.USER_TOKEN_VERIFY_ERR));
            }
        } else {
            jsonObject = JSONObject.toJSON(ApiResult.fail(ApiCode.USER_TOKEN_ISNULL_ERR));
        }
        response.setContentType(MediaType.APPLICATION_JSON_VALUE);
        response.setCharacterEncoding("UTF-8");
        response.getWriter().print(jsonObject);
        response.setStatus(HttpStatus.OK.value());
        return false;
    }
}
