package com.cloud.web.interceptors;

import com.alibaba.fastjson.JSONObject;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.web.commond.anno.AnalysisUserId;
import com.cloud.web.commond.anno.ResponseResult;
import com.cloud.web.config.AndroidConfig;
import com.cloud.web.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;

/**
 * @Author sj
 * Date on 2020/10/21  16:23
 * @Version 1.0
 * 接口请求返回值封装拦截器
 */
@Slf4j
@Component
public class ResponseResultInterceptors implements HandlerInterceptor {
    private static final String RESPONSE_RESULT_ANN = "RESPONSE_RESULT_ANN";

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        if (handler instanceof HandlerMethod) {
            final HandlerMethod handlerMethod = (HandlerMethod) handler;
            final Class<?> clazz = handlerMethod.getBeanType();
            final Method method = handlerMethod.getMethod();
            //判断类上方是否有封装返回值注解
            if (clazz.isAnnotationPresent(ResponseResult.class)) {
                request.setAttribute(RESPONSE_RESULT_ANN, clazz.getAnnotation(ResponseResult.class));
            } else if (method.isAnnotationPresent(ResponseResult.class)) {
                request.setAttribute(RESPONSE_RESULT_ANN, clazz.getAnnotation(ResponseResult.class));
            }
        }
        return true;
    }
}
