package com.cloud.web.modules.pay.wechat.service.impl;

import com.cloud.web.entity.MemberRecords;
import com.cloud.web.entity.User;
import com.cloud.web.modules.pay.wechat.config.WxpayConfig;
import com.cloud.web.modules.pay.wechat.model.Unifiedorder;
import com.cloud.web.modules.pay.wechat.model.UnifiedorderResult;
import com.cloud.web.modules.pay.wechat.service.WeChatPayServices;
import com.cloud.web.modules.pay.wechat.util.HttpXmlUtils;
import com.cloud.web.modules.pay.wechat.util.ParseXMLUtils;
import com.cloud.web.modules.pay.wechat.util.WXSignUtils;
import com.cloud.web.service.MemberRecordsService;
import com.cloud.web.service.UserService;
import com.cloud.web.util.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.math.BigDecimal;
import java.util.*;

/**
 * @Author sj
 * Date on 2020/9/21  20:45
 * @Version 1.0
 */
@Service
@Slf4j
public class WeChatPayServiceImpl implements WeChatPayServices {
    @Autowired
    private WxpayConfig wxpayConfig;
    @Autowired
    private MemberRecordsService memberRecordsService;
    @Autowired
    private UserService userService;

    @Override
    public Map<String, Object> goWeChatPay(MemberRecords pay, HttpServletRequest request) {
        Map<String, Object> map = new HashMap<>(20);
        int total_fee = pay.getMoney().multiply(new BigDecimal(100)).intValue();

        //测试
        User user = userService.getById(pay.getUid());
        if (user != null && user.getPhone().equals("18380438130")) {
            total_fee = 1;
        }

        // 微信开放平台审核通过的应用APPID
        String nonce_str = StringUtils.getRandomString(30);
        String spbill_create_ip = WXSignUtils.getRemortIP(request);
        String notify_url = wxpayConfig.getNotify_url();
        String trade_type = "APP";
        // 参数：开始生成签名
        SortedMap<Object, Object> parameters = new TreeMap<Object, Object>();
        parameters.put("appid", wxpayConfig.getAppid());
        parameters.put("body", "会员充值");
        parameters.put("mch_id", wxpayConfig.getMch_id());
        parameters.put("nonce_str", nonce_str);
        parameters.put("notify_url", notify_url);
        parameters.put("out_trade_no", pay.getOrderId());
        parameters.put("spbill_create_ip", spbill_create_ip);
        parameters.put("total_fee", total_fee);
        parameters.put("trade_type", trade_type);
        String sign = WXSignUtils.createSign(parameters);
        log.info("price:" + total_fee);
        log.info("appid是：" + wxpayConfig.getAppid());
        log.info("mch_id是：" + wxpayConfig.getMch_id());
        log.info("spbill_create_ip====" + spbill_create_ip);
        log.info("notify_url是：" + notify_url);
        log.info("签名是：" + sign);
        Unifiedorder unifiedorder = new Unifiedorder();
        unifiedorder.setAppid(wxpayConfig.getAppid());
        unifiedorder.setBody("会员充值");
        unifiedorder.setMch_id(wxpayConfig.getMch_id());
        unifiedorder.setNonce_str(nonce_str);
        unifiedorder.setNotify_url(notify_url);
        unifiedorder.setOut_trade_no(pay.getOrderId());
        unifiedorder.setSpbill_create_ip(spbill_create_ip);
        unifiedorder.setTotal_fee(total_fee);
        unifiedorder.setTrade_type(trade_type);
        unifiedorder.setSign(sign);
        // 构造xml参数
        String xmlInfo = HttpXmlUtils.xmlInfo(unifiedorder);
        log.info("xmlInfo:" + xmlInfo);
        String wxUrl = "https://api.mch.weixin.qq.com/pay/unifiedorder";
        String method = "POST";
        // 请求微信
        String weixinPost = HttpXmlUtils.httpsRequest(wxUrl, method, xmlInfo).toString();
        log.info("weixinPost:" + weixinPost);
        // 解析微信的反馈
        UnifiedorderResult unifiedorderResult = ParseXMLUtils.jdomParseXml(weixinPost);
        if (unifiedorderResult != null) {
            if ("SUCCESS".equals(unifiedorderResult.getReturn_code())) {
                if ("INVALID_REQUEST".equals(unifiedorderResult.getErr_code())) {
                    map.put("code", 500);
                    map.put("apiCode", "操作失败");
                    return map;
                }
                // 开始拼接App调起微信的参数
                SortedMap<Object, Object> wxAppparameters = new TreeMap<Object, Object>();
                wxAppparameters.put("appid", unifiedorderResult.getAppid());
                wxAppparameters.put("partnerid", unifiedorderResult.getMch_id());
                wxAppparameters.put("prepayid", unifiedorderResult.getPrepay_id());
                wxAppparameters.put("package", wxpayConfig.getWx_package());
                wxAppparameters.put("noncestr", nonce_str);
                wxAppparameters.put("timestamp", System.currentTimeMillis() / 1000);
                wxAppparameters.put("sign", WXSignUtils.createSign(wxAppparameters));
                log.info("________-------------_________签名" + wxAppparameters.get("sign").toString());
                map.put("code", 200);
                map.put("msg", "success");
                map.put("data", wxAppparameters);
                return map;
            } else {
                log.info("错误原因为：" + unifiedorderResult.getReturn_msg());
                map.put("code", 500);
                map.put("msg", unifiedorderResult.getReturn_msg());
                return map;
            }
        } else {
            log.info("服务端请求微信的返回值异常。");
            map.put("code", 500);
            map.put("msg", "服务端请求微信的返回值异常。");
            return map;
        }
    }


    @Override
    @Transactional(rollbackFor = Exception.class)
    public String weChatNotify(HttpServletRequest request) throws Exception {
        Map<String, String> map = new HashMap<>();
        log.info("----------------微信回调开始----------------------");
        // 读取参数
        InputStream inputStream;
        StringBuffer sb = new StringBuffer();
        inputStream = request.getInputStream();
        String s;
        BufferedReader in = new BufferedReader(new InputStreamReader(inputStream, "UTF-8"));
        while ((s = in.readLine()) != null) {
            sb.append(s);
        }
        in.close();
        inputStream.close();
        // 解析xml成map
        Map<String, String> m = new HashMap<String, String>();
        m = WXSignUtils.doXMLParse(sb.toString());
        // 过滤空 设置 TreeMap
        SortedMap<Object, Object> packageParams = new TreeMap<>();
        Iterator<String> it = m.keySet().iterator();
        while (it.hasNext()) {
            String parameter = it.next();
            String parameterValue = m.get(parameter);

            String v = "";
            if (null != parameterValue) {
                v = parameterValue.trim();
            }
            packageParams.put(parameter, v);
        }
        // 微信支付的API密钥
        String key = wxpayConfig.getKey();
        if (!isTenpaySign("UTF-8", packageParams, key)) {
            map.put("return_code", "FAIL");
            map.put("return_msg", "return_code不正确");
            return StringUtils.GetMapToXML(map);
        }
        if (isTenpaySign("UTF-8", packageParams, key)) {
            // 验证通过
            if ("SUCCESS".equals(packageParams.get("result_code").toString())) {
                String out_trade_no = (String) packageParams.get("out_trade_no");
                /** 业务逻辑实现 begin**/
                boolean isSuccess = memberRecordsService.paySuccessHandle(out_trade_no);
                /** 业务逻辑实现 end**/
                if (isSuccess) {
                    map.put("return_code", "SUCCESS");
                    map.put("return_msg", "OK");
                } else {
                    log.info("支付失败");
                    map.put("return_code", "error");
                    map.put("return_msg", "支付失败");
                }
                return StringUtils.GetMapToXML(map);

            }
        } else {
            log.info("支付失败");
            map.put("return_code", "error");
            map.put("return_msg", "支付失败");
            return StringUtils.GetMapToXML(map);
        }
        log.info("支付失败");
        map.put("return_code", "error");
        map.put("return_msg", "支付失败");
        return StringUtils.GetMapToXML(map);
    }


    /**
     * @Function: 是否签名正确, 规则是:按参数名称a-z排序,遇到空值的参数不参加签名。
     */
    @SuppressWarnings("rawtypes")
    public static boolean isTenpaySign(String characterEncoding, SortedMap<Object, Object> packageParams,
                                       String API_KEY) {
        StringBuffer sb = new StringBuffer();
        Set es = packageParams.entrySet();
        Iterator it = es.iterator();
        while (it.hasNext()) {
            Map.Entry entry = (Map.Entry) it.next();
            String k = (String) entry.getKey();
            String v = (String) entry.getValue();
            if (!"sign".equals(k) && null != v && !"".equals(v)) {
                sb.append(k + "=" + v + "&");
            }
        }
        sb.append("key=" + API_KEY);
        // 算出摘要
        String mysign = EncryptUtil.md5Password(sb.toString()).toLowerCase();
        String tenpaySign = ((String) packageParams.get("sign")).toLowerCase();
        return tenpaySign.equals(mysign);
    }
}
