package com.cloud.web.service;

import com.cloud.web.entity.Article;
import com.cloud.web.param.ArticlePageParam;
import com.cloud.framework.common.service.BaseService;
import com.cloud.web.vo.ArticleQueryVo;
import java.io.Serializable;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.web.vo.MyArticleQueryVo;

/**
 *  服务类
 *
 * @author zyl
 * @since 2021-01-08
 */
public interface ArticleService extends BaseService<Article> {

    /**
     * 保存
     *
     * @param article
     * @return
     * @throws Exception
     */
    boolean saveArticle(Article article) throws Exception;

    /**
     * 修改
     *
     * @param article
     * @return
     * @throws Exception
     */
    boolean updateArticle(Article article) throws Exception;

    /**
     * 删除
     *
     * @param id
     * @return
     * @throws Exception
     */
    boolean deleteArticle(Long id) throws Exception;

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    ArticleQueryVo getArticleById(Serializable id) throws Exception;

    /**
     * 获取分页对象
     *
     * @param articlePageParam
     * @return
     * @throws Exception
     */
    Paging<Article> getArticlePageList(ArticlePageParam articlePageParam) throws Exception;
    /**
     * 获取分页对象通过sql
     *
     * @param articlePageParam
     * @return
     * @throws Exception
     */
    Paging<ArticleQueryVo> getArticlePageListBySql(ArticlePageParam articlePageParam) throws Exception;

    MyArticleQueryVo getMyArticleQueryVo(Integer type) throws Exception;

}
