package com.cloud.web.service;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.BlacklistRecords;
import com.cloud.web.param.BlacklistRecordsPageParam;
import com.cloud.framework.common.service.BaseService;
import com.cloud.web.params.page.MyBlacklistRecordsPageParam;
import com.cloud.web.vo.AddBlacklistRecordsVo;
import com.cloud.web.vo.BlacklistRecordsQueryVo;

import java.io.Serializable;

import com.cloud.framework.core.pagination.Paging;

/**
 * 黑名单记录 服务类
 *
 * @author zyl
 * @since 2022-11-08
 */
public interface BlacklistRecordsService extends BaseService<BlacklistRecords> {

    /**
     * 保存
     *
     * @param blacklistRecords
     * @return
     * @throws Exception
     */
    boolean saveBlacklistRecords(BlacklistRecords blacklistRecords) throws Exception;

    /**
     * 修改
     *
     * @param blacklistRecords
     * @return
     * @throws Exception
     */
    boolean updateBlacklistRecords(BlacklistRecords blacklistRecords) throws Exception;

    /**
     * 删除
     *
     * @param id
     * @return
     * @throws Exception
     */
    boolean deleteBlacklistRecords(Long id) throws Exception;

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    BlacklistRecordsQueryVo getBlacklistRecordsById(Serializable id) throws Exception;

    /**
     * 获取分页对象
     *
     * @param blacklistRecordsPageParam
     * @return
     * @throws Exception
     */
    Paging<BlacklistRecords> getBlacklistRecordsPageList(BlacklistRecordsPageParam blacklistRecordsPageParam) throws Exception;

    /**
     * 获取分页对象通过sql
     *
     * @param blacklistRecordsPageParam
     * @return
     * @throws Exception
     */
    Paging<BlacklistRecordsQueryVo> getBlacklistRecordsPageListBySql(BlacklistRecordsPageParam blacklistRecordsPageParam) throws Exception;


    /**
     * 获取分页对象
     *
     * @param myBlacklistRecordsPageParam
     * @return
     * @throws Exception
     */
    ApiPageResult<BlacklistRecordsQueryVo> getBlacklistRecordsPageLists(MyBlacklistRecordsPageParam myBlacklistRecordsPageParam) throws Exception;


    /**
     * 删除记录
     *
     * @param id
     */
    void deleteById(String id);

    /**
     * 添加黑名单
     *
     * @param addBlacklistRecordsVo
     * @return
     */
    boolean addBlacklistRecords(AddBlacklistRecordsVo addBlacklistRecordsVo);
}
