package com.cloud.web.service;

import java.util.Arrays;
import java.util.List;

import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.cloudauth.model.v20190307.DescribeFaceVerifyRequest;
import com.aliyuncs.cloudauth.model.v20190307.DescribeFaceVerifyResponse;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.http.HttpClientConfig;
import com.aliyuncs.http.HttpClientType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.regions.ProductDomain;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.web.entity.User;
import com.cloud.web.entity.UserAuthentication;
import com.cloud.web.exception.AndroidBusinessException;
import com.google.gson.Gson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class DescribeFaceVerifyService {

    @Autowired
    private UserAuthenticationService userAuthenticationService;
    @Autowired
    private UserService userService;


    public String getDescribeFaceVerify(Long userId, String certifyId) throws Exception {
        DescribeFaceVerifyRequest request = new DescribeFaceVerifyRequest();
        // 请输入SceneId+L。
        request.setSceneId(1000005619L);
        request.setCertifyId(certifyId);

        List<String> endpoints = Arrays.asList("cloudauth.cn-shanghai.aliyuncs.com",
                "cloudauth.cn-beijing.aliyuncs.com");

        // 推荐，支持服务路由。
        DescribeFaceVerifyResponse response = describeFaceVerifyAutoRoute(endpoints, request);

        // 不支持服务自动路由。
        //DescribeFaceVerifyResponse response = describeFaceVerify("cloudauth.cn-shanghai.aliyuncs.com", request);
        log.info("阿里人脸对比数据=========》》》》》》" + new Gson().toJson(response));
        String passed = response.getResultObject().getPassed();
        UserAuthentication authentication = userAuthenticationService.getOne(new LambdaQueryWrapper<UserAuthentication>().eq(UserAuthentication::getUserId, userId));
        if (passed.equals("T")) {
            if (authentication != null) {
                authentication.setStatus(1);
                userAuthenticationService.updateUserAuthentication(authentication);
                User user = userService.getById(userId);
                if (user != null) {
                    user.setIsUserAuthentication(1);
                    user.setRealName(authentication.getRealName());
                    user.setIdCard(authentication.getIdCard());
                    userService.updateUser(user);
                }
            }
        } else {
            if (authentication != null) {
                authentication.setStatus(2);
                userAuthenticationService.updateUserAuthentication(authentication);
            }
            throw new AndroidBusinessException(ApiCode.AUTHENTICATION_FAIL);
        }
        return null;
    }

    private static DescribeFaceVerifyResponse describeFaceVerifyAutoRoute(List<String> endpoints,
                                                                          DescribeFaceVerifyRequest request) {

        DescribeFaceVerifyResponse lastResponse = null;
        for (String endpoint : endpoints) {
            try {
                DescribeFaceVerifyResponse response = describeFaceVerify(endpoint, request);
                lastResponse = response;
                if ("500".equals(response.getCode())) {
                    continue;
                }

                return response;
            } catch (ClientException e) {
                if ("ServiceUnavailable".equals(e.getErrCode()) || "SDK.ServerUnreachable".equals(e.getErrCode())
                        || "SDK.ReadTimeout".equals(e.getErrCode())) {
                    continue;
                }

                System.out.println(e.getErrCode());
                System.out.println(e.getErrMsg());
                System.out.println(e.getRequestId());
            }
        }

        return lastResponse;
    }

    private static DescribeFaceVerifyResponse describeFaceVerify(String endpoint, DescribeFaceVerifyRequest request)
            throws ClientException {
        // Client代理配置。
        HttpClientConfig config = new HttpClientConfig();
        config.setClientType(HttpClientType.ApacheHttpClient);
        //config.setProtocolType(ProtocolType.HTTPS);
        //config.setHttpsProxy("https://47.100.XX.XX:XXXX");
        //config.setHttpProxy("http://47.100.XX.XX:XXXX");
        //config.setNoProxy("xx.xx.xx.xx,localhost");
        config.setConnectionTimeoutMillis(10000);
        config.setReadTimeoutMillis(10000);
        DefaultProfile profile = DefaultProfile.getProfile("", "LTAI5tC7Wd7kXmSM3UxxmoTy",
                "gtIXOdnBa8L2UWYlAWillqWIF5ChzJ");
        profile.setHttpClientConfig(config);
        DefaultAcsClient client = new DefaultAcsClient(profile);
        request.setSysProductDomain(new ProductDomain("cloudauth", endpoint));

        return client.getAcsResponse(request);
    }
}
