package com.cloud.web.service;

import java.util.Arrays;
import java.util.List;

import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.DefaultAcsClient;
import com.aliyuncs.exceptions.ClientException;
import com.aliyuncs.http.HttpClientConfig;
import com.aliyuncs.http.HttpClientType;
import com.aliyuncs.profile.DefaultProfile;
import com.aliyuncs.regions.ProductDomain;
import com.cloud.framework.util.UUIDUtil;
import com.cloud.web.params.param.InsertUserAuthenticationParam;
import com.google.gson.Gson;
import com.aliyuncs.cloudauth.model.v20190307.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class InitFaceVerifyService {

    public InitFaceVerifyResponse getInitFaceVerifyResponse(InsertUserAuthenticationParam insertUserAuthenticationParam) {
        InitFaceVerifyRequest request = new InitFaceVerifyRequest();
        // 请输入SceneId+L。
        request.setSceneId(1000005619L);
        // 设置商户请求的唯一标识。
        String orderNo = UUIDUtil.getUuid();
        request.setOuterOrderNo(orderNo);
        // 认证方案。
        request.setProductCode("ID_PRO");
        // 模式。
        request.setModel("LIVENESS");
        request.setCertType("IDENTITY_CARD");

        request.setCertName(insertUserAuthenticationParam.getRealName());
        request.setCertNo(insertUserAuthenticationParam.getIdCard());
        request.setFaceContrastPictureUrl(insertUserAuthenticationParam.getBeforeImage());
        // MetaInfo环境参数。
        JSONObject json = new JSONObject();
        json.put("zimVer", insertUserAuthenticationParam.getZimVer());
        json.put("appVersion", insertUserAuthenticationParam.getAppVersion());
        json.put("bioMetaInfo", insertUserAuthenticationParam.getBioMetaInfo());
        json.put("appName", insertUserAuthenticationParam.getAppName());
        json.put("deviceType", insertUserAuthenticationParam.getDeviceType());
        json.put("osVersion", insertUserAuthenticationParam.getOsVersion());
        json.put("sdkVersion", insertUserAuthenticationParam.getSdkVersion());
        json.put("apdidToken", insertUserAuthenticationParam.getApdidToken());
        json.put("deviceModel", insertUserAuthenticationParam.getDeviceModel());
        request.setMetaInfo(json.toJSONString());
        //request.setMobile("1390000****");
        //request.setIp("114.100.XX.XX");
        //request.setUserId("148562088256****");
        //request.setCallbackUrl("https://www.example.com");
        //request.setCallbackToken("NMjvQanQgplBSaEI0sL86Wn****");

        List<String> endpoints = Arrays.asList("cloudauth.cn-shanghai.aliyuncs.com",
                "cloudauth.cn-beijing.aliyuncs.com");

        // 推荐，支持服务路由。
        InitFaceVerifyResponse response = initFaceVerifyAutoRoute(endpoints, request);

        // 不支持服务自动路由。
        //InitFaceVerifyResponse response = initFaceVerify("cloudauth.cn-shanghai.aliyuncs.com", request);
        log.info("阿里人脸对比数据=========》》》》》》" + new Gson().toJson(response));
        log.debug("阿里人脸对比数据=========》》》》》》" + new Gson().toJson(response));
        System.out.println(new Gson().toJson(response));
        return response;
    }

    private static InitFaceVerifyResponse initFaceVerifyAutoRoute(List<String> endpoints,
                                                                  InitFaceVerifyRequest request) {

        InitFaceVerifyResponse lastResponse = null;
        for (String endpoint : endpoints) {
            try {
                InitFaceVerifyResponse response = initFaceVerify(endpoint, request);
                lastResponse = response;
                if ("500".equals(response.getCode())) {
                    continue;
                }

                return response;
            } catch (ClientException e) {
                if ("ServiceUnavailable".equals(e.getErrCode()) || "SDK.ServerUnreachable".equals(e.getErrCode())
                        || "SDK.ReadTimeout".equals(e.getErrCode())) {
                    continue;
                }

                System.out.println(e.getErrCode());
                System.out.println(e.getErrMsg());
                System.out.println(e.getRequestId());
            }
        }

        return lastResponse;
    }

    private static InitFaceVerifyResponse initFaceVerify(String endpoint, InitFaceVerifyRequest request)
            throws ClientException {
        // Client代理配置。
        HttpClientConfig config = new HttpClientConfig();
        config.setClientType(HttpClientType.ApacheHttpClient);
        //config.setProtocolType(ProtocolType.HTTPS);
        //config.setHttpsProxy("https://47.100.XX.XX:XXXX");
        //config.setHttpProxy("http://47.100.XX.XX:XXXX");
        //config.setNoProxy("xx.xx.xx.xx,localhost");
        config.setConnectionTimeoutMillis(10000);
        config.setReadTimeoutMillis(10000);
        DefaultProfile profile = DefaultProfile.getProfile("", "LTAI5tC7Wd7kXmSM3UxxmoTy",
                "gtIXOdnBa8L2UWYlAWillqWIF5ChzJ");
        profile.setHttpClientConfig(config);
        DefaultAcsClient client = new DefaultAcsClient(profile);
        request.setSysProductDomain(new ProductDomain("cloudauth", endpoint));

        return client.getAcsResponse(request);
    }


//    public static void main(String[] args) {
//        JSONObject json = new JSONObject();
//        json.put("zimVer", "3.0.0");
//        json.put("appVersion", "");
//        json.put("bioMetaInfo", "6.5.0:11501568,4");
//        json.put("appName", "com.pandajob");
//        json.put("deviceType", "ios");
//        json.put("osVersion", "iOS 12.5.6");
//        json.put("sdkVersion", "1.5.1");
//        json.put("apdidToken", "cuwhhouXKkx24lkD/ZPdu+Zjjn9OSpO+InKpV0vcOyQM/+2FhAEAAA==");
//        json.put("deviceModel", "iPhone7,2");
//        System.out.println(json.toJSONString());
//    }

}
