package com.cloud.web.service;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.Message;
import com.cloud.web.param.MessagePageParam;
import com.cloud.framework.common.service.BaseService;
import com.cloud.web.params.page.MyMessagePageParam;
import com.cloud.web.params.page.MyPostPageParam;
import com.cloud.web.vo.MessageQueryVo;
import java.io.Serializable;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.web.vo.PostQueryVo;

/**
 *  服务类
 *
 * @author zyl
 * @since 2021-01-08
 */
public interface MessageService extends BaseService<Message> {

    /**
     * 保存
     *
     * @param message
     * @return
     * @throws Exception
     */
    boolean saveMessage(Message message) throws Exception;

    /**
     * 修改
     *
     * @param message
     * @return
     * @throws Exception
     */
    boolean updateMessage(Message message) throws Exception;

    /**
     * 删除
     *
     * @param id
     * @return
     * @throws Exception
     */
    boolean deleteMessage(Long id) throws Exception;

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    MessageQueryVo getMessageById(Serializable id) throws Exception;

    /**
     * 获取分页对象
     *
     * @param messagePageParam
     * @return
     * @throws Exception
     */
    Paging<Message> getMessagePageList(MessagePageParam messagePageParam) throws Exception;
    /**
     * 获取分页对象通过sql
     *
     * @param messagePageParam
     * @return
     * @throws Exception
     */
    Paging<MessageQueryVo> getMessagePageListBySql(MessagePageParam messagePageParam) throws Exception;

    ApiPageResult<MessageQueryVo> getMessageQueryVo(MyMessagePageParam myMessagePageParam)throws Exception;
}
