package com.cloud.web.service;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.Post;
import com.cloud.web.param.AddPostParam;
import com.cloud.web.param.PostPageParam;
import com.cloud.framework.common.service.BaseService;
import com.cloud.web.params.page.MyPostPageParam;
import com.cloud.web.vo.PostInfoVo;
import com.cloud.web.vo.PostQueryVo;
import java.io.Serializable;
import com.cloud.framework.core.pagination.Paging;

/**
 *  服务类
 *
 * @author zyl
 * @since 2021-01-08
 */
public interface PostService extends BaseService<Post> {

    /**
     * 保存
     *
     * @param post
     * @return
     * @throws Exception
     */
    boolean savePost(Post post) throws Exception;

    /**
     * 修改
     *
     * @param post
     * @return
     * @throws Exception
     */
    boolean updatePost(Post post) throws Exception;

    /**
     * 删除
     *
     * @param id
     * @return
     * @throws Exception
     */
    boolean deletePost(Long id) throws Exception;

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    PostQueryVo getPostById(Serializable id) throws Exception;

    /**
     * 获取分页对象
     *
     * @param postPageParam
     * @return
     * @throws Exception
     */
    Paging<Post> getPostPageList(PostPageParam postPageParam) throws Exception;
    /**
     * 获取分页对象通过sql
     *
     * @param postPageParam
     * @return
     * @throws Exception
     */
    Paging<PostQueryVo> getPostPageListBySql(PostPageParam postPageParam) throws Exception;

    ApiPageResult<PostQueryVo> getPostVo(MyPostPageParam postPageParam)throws Exception;
    ApiPageResult<PostQueryVo> getPostVos(MyPostPageParam postPageParam)throws Exception;

    PostInfoVo getOne(Integer id,Long userId) throws Exception;

    /**
     * 删除职位
     * @param id
     */
    void delete(Integer id);

}
