package com.cloud.web.service;

import com.cloud.web.entity.UserAuthentication;
import com.cloud.web.param.UserAuthenticationPageParam;
import com.cloud.framework.common.service.BaseService;
import com.cloud.web.params.param.InsertUserAuthenticationParam;
import com.cloud.web.vo.IdCardVo;
import com.cloud.web.vo.UserAuthenticationQueryVo;

import java.io.IOException;
import java.io.Serializable;

import com.cloud.framework.core.pagination.Paging;

/**
 * 用户实名认证 服务类
 *
 * @author zyl
 * @since 2022-11-13
 */
public interface UserAuthenticationService extends BaseService<UserAuthentication> {

    /**
     * 保存
     *
     * @param userAuthentication
     * @return
     * @throws Exception
     */
    boolean saveUserAuthentication(UserAuthentication userAuthentication) throws Exception;

    /**
     * 修改
     *
     * @param userAuthentication
     * @return
     * @throws Exception
     */
    boolean updateUserAuthentication(UserAuthentication userAuthentication) throws Exception;

    /**
     * 删除
     *
     * @param id
     * @return
     * @throws Exception
     */
    boolean deleteUserAuthentication(Long id) throws Exception;

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    UserAuthenticationQueryVo getUserAuthenticationById(Serializable id) throws Exception;

    /**
     * 获取分页对象
     *
     * @param userAuthenticationPageParam
     * @return
     * @throws Exception
     */
    Paging<UserAuthentication> getUserAuthenticationPageList(UserAuthenticationPageParam userAuthenticationPageParam) throws Exception;

    /**
     * 获取分页对象通过sql
     *
     * @param userAuthenticationPageParam
     * @return
     * @throws Exception
     */
    Paging<UserAuthenticationQueryVo> getUserAuthenticationPageListBySql(UserAuthenticationPageParam userAuthenticationPageParam) throws Exception;


    /**
     * 用户实名认证
     * @param insertUserAuthenticationParam
     * @return
     */
    String addUserAuthentication(InsertUserAuthenticationParam insertUserAuthenticationParam) throws IOException;

    /**
     * 用户身份信息提取
     * @param insertUserAuthenticationParam
     * @return
     */
    IdCardVo getIdCard(InsertUserAuthenticationParam insertUserAuthenticationParam) throws IOException;
}
