package com.cloud.web.service;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.User;
import com.cloud.web.param.IntroUserPageParam;
import com.cloud.web.param.ModifyPasswordParam;
import com.cloud.web.param.UpdateCompanyParam;
import com.cloud.web.params.param.*;
import com.cloud.web.param.UserPageParam;
import com.cloud.framework.common.service.BaseService;
import com.cloud.web.vo.IntroUserVo;
import com.cloud.web.vo.LoginTokenVo;
import com.cloud.web.vo.UserQueryVo;
import java.io.Serializable;
import com.cloud.framework.core.pagination.Paging;

/**
 *  服务类
 *
 * @author zyl
 * @since 2021-01-08
 */
public interface UserService extends BaseService<User> {

    /**
     * 保存
     *
     * @param user
     * @return
     * @throws Exception
     */
    boolean saveUser(User user) throws Exception;

    /**
     * 修改
     *
     * @param user
     * @return
     * @throws Exception
     */
    boolean updateUser(User user) throws Exception;

    /**
     * 删除
     *
     * @param id
     * @return
     * @throws Exception
     */
    boolean deleteUser(Long id) throws Exception;

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    UserQueryVo getUserById(Serializable id) throws Exception;

    /**
     * 获取分页对象
     *
     * @param userPageParam
     * @return
     * @throws Exception
     */
    Paging<User> getUserPageList(UserPageParam userPageParam) throws Exception;
    /**
     * 获取分页对象通过sql
     *
     * @param userPageParam
     * @return
     * @throws Exception
     */
    Paging<UserQueryVo> getUserPageListBySql(UserPageParam userPageParam) throws Exception;

    String registerUser(RegisterParam registerParam) throws Exception;

    LoginTokenVo getOneByPhone(UserLoginParam userLoginParam)throws Exception;

    String registerUsers(RegistersParam registersParam) throws Exception;

    LoginTokenVo getOneByPhones(UserLoginsParam userLoginsParam)throws Exception;

    LoginTokenVo getOneByPhoneCode(UserLoginsParam userLoginsParam)throws Exception;

    void updateUser(UpdateUserParam updateUserParam, Long userId) throws Exception;

    /**
     * 邀请记录
     * @param introUserPageParam
     * @return
     * @throws Exception
     */
    ApiPageResult<IntroUserVo> getIntroRecord(IntroUserPageParam introUserPageParam)throws Exception;

    /**
     * 找回密码
     * @param modifyPasswordParam
     */
    void modifyPassword(ModifyPasswordParam modifyPasswordParam);

    /**
     * 更新企业信息
     * @param updateCompanyParam
     * @param userId
     * @throws Exception
     */
    void updateCompany(UpdateCompanyParam updateCompanyParam, Long userId) throws Exception;
}
