package com.cloud.web.service.impl;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.Announcement;

import com.cloud.web.mapper.AnnouncementMapper;
import com.cloud.web.param.PostPageParam;
import com.cloud.web.params.page.MyAnnouncementPageParam;
import com.cloud.web.service.AnnouncementService;
import com.cloud.web.param.AnnouncementPageParam;
import com.cloud.web.vo.AnnouncementQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cloud.web.vo.PostQueryVo;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 服务实现类
 *
 * @author zyl
 * @since 2022-10-24
 */
@Slf4j
@Service
public class AnnouncementServiceImpl extends BaseServiceImpl<AnnouncementMapper, Announcement> implements AnnouncementService {

    @Autowired
    private AnnouncementMapper announcementMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveAnnouncement(Announcement announcement) throws Exception {
        return super.save(announcement);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateAnnouncement(Announcement announcement) throws Exception {
        return super.updateById(announcement);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteAnnouncement(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public AnnouncementQueryVo getAnnouncementById(Serializable id) throws Exception {
        return announcementMapper.getAnnouncementById(id);
    }

    @Override
    public Paging<AnnouncementQueryVo> getAnnouncementPageListBySql(AnnouncementPageParam announcementPageParam) throws Exception {
        Page<AnnouncementQueryVo> page = new PageInfo<>(announcementPageParam, OrderItem.desc("create_time"));
        IPage<AnnouncementQueryVo> iPage = announcementMapper.getAnnouncementPageList(page, announcementPageParam);
        return new Paging<AnnouncementQueryVo>(iPage);
    }

    @Override
    public Paging<Announcement> getAnnouncementPageList(AnnouncementPageParam announcementPageParam) throws Exception {
        Page<Announcement> page = new PageInfo<>(announcementPageParam, OrderItem.desc("create_time"));
        IPage<Announcement> iPage = announcementMapper.selectPage(page, QueryHelpPlus.getPredicate(Announcement.class, announcementPageParam));
        return new Paging<Announcement>(iPage);
    }

    @Override
    public ApiPageResult<AnnouncementQueryVo> getAnnouncementPageLists(MyAnnouncementPageParam myAnnouncementPageParam) throws Exception {
        AnnouncementPageParam pageParam = new AnnouncementPageParam();
        pageParam.setPageIndex(myAnnouncementPageParam.getOffset());
        pageParam.setPageSize(myAnnouncementPageParam.getPageSize());
        Page<AnnouncementQueryVo> page = new PageInfo<>(pageParam);
        IPage<AnnouncementQueryVo> iPage = announcementMapper.getAnnouncementPageList(page, pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

}
