package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.util.IpUtil;
import com.cloud.web.entity.*;

import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.mapper.ApplyMapper;
import com.cloud.web.params.param.InsertApplyParam;
import com.cloud.web.service.*;
import com.cloud.web.param.ApplyPageParam;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.ApplyQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;
import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 服务实现类
 *
 * @author zyl
 * @since 2021-01-09
 */
@Slf4j
@Service
public class ApplyServiceImpl extends BaseServiceImpl<ApplyMapper, Apply> implements ApplyService {

    @Autowired
    private ApplyMapper applyMapper;
    @Autowired
    private UserService userService;
    @Autowired
    private PostService postService;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private MessageService messageService;
    @Autowired
    private PunchCardService punchCardService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveApply(Apply apply) throws Exception {
        return super.save(apply);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateApply(Apply apply) throws Exception {
        return super.updateById(apply);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteApply(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public ApplyQueryVo getApplyById(Serializable id) throws Exception {
        return applyMapper.getApplyById(id);
    }

    @Override
    public Paging<ApplyQueryVo> getApplyPageListBySql(ApplyPageParam applyPageParam) throws Exception {
        Page<ApplyQueryVo> page = new PageInfo<>(applyPageParam, OrderItem.desc("create_time"));
        IPage<ApplyQueryVo> iPage = applyMapper.getApplyPageList(page, applyPageParam);
        return new Paging<ApplyQueryVo>(iPage);
    }

    @Override
    public void addApply(InsertApplyParam insertApplyParam) throws Exception {
        //判断当前用户是否有报名记录
//        int count = applyMapper.selectCount(new LambdaQueryWrapper<Apply>().eq(Apply::getUserId,insertApplyParam.getUserId()));
//        if (count > 0){
//            throw new AndroidVerifyException(ApiCode.REPETITION_APPLY);
//        }
        User user = userService.getById(insertApplyParam.getUserId());
        Post post = postService.getById(insertApplyParam.getPostId());
        Company company = null;
        if (post.getCompanyId() != null) {
            company = companyService.getById(post.getCompanyId());
        }
        Apply apply = new Apply();
        if (user != null && post != null) {
            QueryWrapper<Apply> wrapper = new QueryWrapper();
            wrapper.eq("user_id", insertApplyParam.getUserId()).eq("post_id", insertApplyParam.getPostId()).eq("company_id", insertApplyParam.getCompanyId());
            Apply oldApply = applyMapper.selectOne(wrapper);
            if (!StringUtils.isEmpty(oldApply)) {
                throw new AndroidVerifyException(ApiCode.USER_APPLY_ERR);
            }
            apply.setCompanyId(post.getCompanyId());
            if (company != null) {
                apply.setCompanyName(company.getName());
            }
            apply.setPostId(post.getId());
            apply.setPostName(post.getName());
            apply.setUserId(user.getId());
            apply.setUserName(user.getUserName());
            apply.setUserPhone(user.getPhone());
            apply.setStatus(1);
            apply.setCreateTime(new Date());
            apply.setIsMember(user.getIsMember());
            String ipAddress = IpUtil.getRequestIp();
            if (StringUtils.isNotBlank(ipAddress)) {
                apply.setIpAddress(ipAddress);
            }
            applyMapper.insert(apply);
            //查询用户打卡记录
            List<PunchCard> punchCards = punchCardService.list(new LambdaQueryWrapper<PunchCard>().eq(PunchCard::getUserId, insertApplyParam.getUserId()));
            if (punchCards != null && punchCards.size() > 0) {
                for (PunchCard punchCard : punchCards) {
                    punchCardService.delete(punchCard.getId());
                }
            }
            //添加消息通知
            Message message = new Message();
            message.setTitle("报名成功通知");
            if (company != null) {
                message.setContent("尊敬的用户，您在" + company.getName() + "的" + post.getName() + "职位已经报名成功。");
            } else {
                message.setContent("尊敬的用户，您的" + post.getName() + "职位已经报名成功。");
            }
            message.setUid(user.getId());
            message.setStatus(1);
            message.setCreateTime(new Date());
            messageService.save(message);
        }
    }

    @Override
    public void delete(Long id) {
        applyMapper.removeApply(id);
    }

    @Override
    public Paging<Apply> getApplyPageList(ApplyPageParam applyPageParam) throws Exception {
        Page<Apply> page = new PageInfo<>(applyPageParam, OrderItem.desc("create_time"));
        IPage<Apply> iPage = applyMapper.selectPage(page, QueryHelpPlus.getPredicate(Apply.class, applyPageParam));
        return new Paging<Apply>(iPage);
    }

}
