package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.web.entity.Article;

import com.cloud.web.mapper.ArticleMapper;
import com.cloud.web.service.ArticleService;
import com.cloud.web.param.ArticlePageParam;
import com.cloud.web.vo.ArticleQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cloud.web.vo.MyArticleQueryVo;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class ArticleServiceImpl extends BaseServiceImpl<ArticleMapper, Article> implements ArticleService {

    @Autowired
    private ArticleMapper articleMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveArticle(Article article) throws Exception {
        return super.save(article);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateArticle(Article article)throws Exception{
            return super.updateById(article);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteArticle(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public ArticleQueryVo getArticleById(Serializable id)throws Exception{
            return articleMapper.getArticleById(id);
            }

    @Override
    public Paging<ArticleQueryVo> getArticlePageListBySql(ArticlePageParam articlePageParam)throws Exception{
            Page<ArticleQueryVo> page=new PageInfo<>(articlePageParam,OrderItem.desc("create_time"));
                IPage<ArticleQueryVo> iPage= articleMapper.getArticlePageList(page, articlePageParam);
            return new Paging<ArticleQueryVo>(iPage);
            }

    @Override
    public MyArticleQueryVo getMyArticleQueryVo(Integer type) throws Exception {
        LambdaQueryWrapper<Article> wrapper=new LambdaQueryWrapper<>();
        wrapper.eq(Article::getType,type);
        Article article=this.getOne(wrapper);
        MyArticleQueryVo vo=new MyArticleQueryVo();
        vo.setContent(article.getContent());
        return vo;
    }

    @Override
    public Paging<Article> getArticlePageList(ArticlePageParam articlePageParam)throws Exception{
        Page<Article> page=new PageInfo<>(articlePageParam,OrderItem.desc("create_time"));
            IPage<Article> iPage= articleMapper.selectPage(page,QueryHelpPlus.getPredicate(Article.class, articlePageParam));
        return new Paging<Article>(iPage);
        }

}
