package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cloud.web.entity.BaseSetting;

import com.cloud.web.mapper.BaseSettingMapper;
import com.cloud.web.service.BaseSettingService;
import com.cloud.web.param.BaseSettingPageParam;
import com.cloud.web.vo.BaseSettingQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class BaseSettingServiceImpl extends BaseServiceImpl<BaseSettingMapper, BaseSetting> implements BaseSettingService {

    @Autowired
    private BaseSettingMapper baseSettingMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveBaseSetting(BaseSetting baseSetting) throws Exception {
        return super.save(baseSetting);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateBaseSetting(BaseSetting baseSetting)throws Exception{
            return super.updateById(baseSetting);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteBaseSetting(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public BaseSettingQueryVo getBaseSettingById(Serializable id)throws Exception{
            return baseSettingMapper.getBaseSettingById(id);
            }

    @Override
    public Paging<BaseSettingQueryVo> getBaseSettingPageListBySql(BaseSettingPageParam baseSettingPageParam)throws Exception{
            Page<BaseSettingQueryVo> page=new PageInfo<>(baseSettingPageParam,OrderItem.desc("create_time"));
                IPage<BaseSettingQueryVo> iPage= baseSettingMapper.getBaseSettingPageList(page, baseSettingPageParam);
            return new Paging<BaseSettingQueryVo>(iPage);
            }

    @Override
    public BaseSetting getOne() {
        return baseSettingMapper.selectOne(new QueryWrapper<>());
    }

    @Override
    public Paging<BaseSetting> getBaseSettingPageList(BaseSettingPageParam baseSettingPageParam)throws Exception{
        Page<BaseSetting> page=new PageInfo<>(baseSettingPageParam,OrderItem.desc("create_time"));
            IPage<BaseSetting> iPage= baseSettingMapper.selectPage(page,QueryHelpPlus.getPredicate(BaseSetting.class, baseSettingPageParam));
        return new Paging<BaseSetting>(iPage);
        }

}
