package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.BlacklistRecords;

import com.cloud.web.entity.User;
import com.cloud.web.mapper.BlacklistRecordsMapper;
import com.cloud.web.param.AnnouncementPageParam;
import com.cloud.web.params.page.MyBlacklistRecordsPageParam;
import com.cloud.web.service.BlacklistRecordsService;
import com.cloud.web.param.BlacklistRecordsPageParam;
import com.cloud.web.service.UserService;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.AddBlacklistRecordsVo;
import com.cloud.web.vo.AnnouncementQueryVo;
import com.cloud.web.vo.BlacklistRecordsQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 黑名单记录 服务实现类
 *
 * @author zyl
 * @since 2022-11-08
 */
@Slf4j
@Service
public class BlacklistRecordsServiceImpl extends BaseServiceImpl<BlacklistRecordsMapper, BlacklistRecords> implements BlacklistRecordsService {

    @Autowired
    private BlacklistRecordsMapper blacklistRecordsMapper;
    @Autowired
    private UserService userService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveBlacklistRecords(BlacklistRecords blacklistRecords) throws Exception {
        return super.save(blacklistRecords);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateBlacklistRecords(BlacklistRecords blacklistRecords) throws Exception {
        return super.updateById(blacklistRecords);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteBlacklistRecords(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public BlacklistRecordsQueryVo getBlacklistRecordsById(Serializable id) throws Exception {
        return blacklistRecordsMapper.getBlacklistRecordsById(id);
    }

    @Override
    public Paging<BlacklistRecordsQueryVo> getBlacklistRecordsPageListBySql(BlacklistRecordsPageParam blacklistRecordsPageParam) throws Exception {
        Page<BlacklistRecordsQueryVo> page = new PageInfo<>(blacklistRecordsPageParam, OrderItem.desc("create_time"));
        IPage<BlacklistRecordsQueryVo> iPage = blacklistRecordsMapper.getBlacklistRecordsPageList(page, blacklistRecordsPageParam);
        return new Paging<BlacklistRecordsQueryVo>(iPage);
    }

    @Override
    public ApiPageResult<BlacklistRecordsQueryVo> getBlacklistRecordsPageLists(MyBlacklistRecordsPageParam myBlacklistRecordsPageParam) throws Exception {
        BlacklistRecordsPageParam pageParam = new BlacklistRecordsPageParam();
        pageParam.setPageIndex(myBlacklistRecordsPageParam.getOffset());
        pageParam.setPageSize(myBlacklistRecordsPageParam.getPageSize());
        Page<BlacklistRecordsQueryVo> page = new PageInfo<>(pageParam);
        IPage<BlacklistRecordsQueryVo> iPage = blacklistRecordsMapper.getBlacklistRecordsPageList(page, pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public void deleteById(String id) {
        blacklistRecordsMapper.deleteById(id);
    }

    @Override
    public boolean addBlacklistRecords(AddBlacklistRecordsVo addBlacklistRecordsVo) {
        BlacklistRecords records = blacklistRecordsMapper.selectOne(new LambdaQueryWrapper<BlacklistRecords>().eq(BlacklistRecords::getUserId, addBlacklistRecordsVo.getId()));
        if (records != null) {
            return true;
        }
        BlacklistRecords blacklistRecords = new BlacklistRecords();
        User u = userService.getById(addBlacklistRecordsVo.getUserId());
        if (u != null) {
            blacklistRecords.setCUserId(u.getId());
            blacklistRecords.setCUserName(u.getUserName());
            blacklistRecords.setCUserPhone(u.getPhone());
        }
        User us = userService.getById(addBlacklistRecordsVo.getId());
        if (us != null) {
            blacklistRecords.setUserId(us.getId());
            blacklistRecords.setUserName(us.getUserName());
            if (StringUtils.isNotBlank(us.getRealName())) {
                blacklistRecords.setRealName(us.getRealName());
            }
            blacklistRecords.setUserPhone(us.getPhone());
        }
        if (StringUtils.isNotBlank(addBlacklistRecordsVo.getRemark())) {
            blacklistRecords.setRemark(addBlacklistRecordsVo.getRemark());
        }
        blacklistRecords.setCreateTime(new Date());
        return super.save(blacklistRecords);
    }

    @Override
    public Paging<BlacklistRecords> getBlacklistRecordsPageList(BlacklistRecordsPageParam blacklistRecordsPageParam) throws Exception {
        Page<BlacklistRecords> page = new PageInfo<>(blacklistRecordsPageParam, OrderItem.desc("create_time"));
        IPage<BlacklistRecords> iPage = blacklistRecordsMapper.selectPage(page, QueryHelpPlus.getPredicate(BlacklistRecords.class, blacklistRecordsPageParam));
        return new Paging<BlacklistRecords>(iPage);
    }

}
