package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.system.service.UpLoadService;
import com.cloud.web.entity.CompanyAuthentication;

import com.cloud.web.entity.User;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.mapper.CompanyAuthenticationMapper;
import com.cloud.web.params.param.InsertCompanyAuthenticationParam;
import com.cloud.web.service.CompanyAuthenticationService;
import com.cloud.web.param.CompanyAuthenticationPageParam;
import com.cloud.web.service.UserService;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.CompanyAuthenticationQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.IOException;
import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import io.swagger.annotations.Api;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;

/**
 * 用户实名认证 服务实现类
 *
 * @author zyl
 * @since 2022-11-13
 */
@Slf4j
@Service
public class CompanyAuthenticationServiceImpl extends BaseServiceImpl<CompanyAuthenticationMapper, CompanyAuthentication> implements CompanyAuthenticationService {

    @Autowired
    private CompanyAuthenticationMapper companyAuthenticationMapper;
    @Resource
    private UpLoadService upLoadService;
    @Autowired
    private UserService userService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveCompanyAuthentication(CompanyAuthentication companyAuthentication) throws Exception {
        return super.save(companyAuthentication);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateCompanyAuthentication(CompanyAuthentication companyAuthentication) throws Exception {
        return super.updateById(companyAuthentication);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteCompanyAuthentication(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public CompanyAuthenticationQueryVo getCompanyAuthenticationById(Serializable id) throws Exception {
        return companyAuthenticationMapper.getCompanyAuthenticationById(id);
    }

    @Override
    public Paging<CompanyAuthenticationQueryVo> getCompanyAuthenticationPageListBySql(CompanyAuthenticationPageParam companyAuthenticationPageParam) throws Exception {
        Page<CompanyAuthenticationQueryVo> page = new PageInfo<>(companyAuthenticationPageParam, OrderItem.desc("create_time"));
        IPage<CompanyAuthenticationQueryVo> iPage = companyAuthenticationMapper.getCompanyAuthenticationPageList(page, companyAuthenticationPageParam);
        return new Paging<CompanyAuthenticationQueryVo>(iPage);
    }

    @Override
    public boolean addCompanyAuthentication(InsertCompanyAuthenticationParam insertCompanyAuthenticationParam) throws IOException {
        //图片上传
        String businessLicense = null;
        if (insertCompanyAuthenticationParam.getBusinessLicenseFile() != null) {
            String images = upLoadService.uploadToOOS(insertCompanyAuthenticationParam.getBusinessLicenseFile());
            if (StringUtils.isNotBlank(images)) {
                businessLicense = images;
            }
        }
//        else {
//            throw new AndroidBusinessException(ApiCode.BUSINESSLICENSE_ERR);
//        }
        CompanyAuthentication companyAuthentication = companyAuthenticationMapper.selectOne(new LambdaQueryWrapper<CompanyAuthentication>().eq(CompanyAuthentication::getUserId, insertCompanyAuthenticationParam.getUserId()));
        if (companyAuthentication != null) {
            BeanUtils.copyProperties(insertCompanyAuthenticationParam, companyAuthentication);
            companyAuthentication.setBusinessLicense(businessLicense);
            companyAuthentication.setStatus(0);
            companyAuthentication.setCauseFailure(null);
            companyAuthentication.setUpdateTime(new Date());
            return super.updateById(companyAuthentication);
        } else {
            CompanyAuthentication authentication = new CompanyAuthentication();
            BeanUtils.copyProperties(insertCompanyAuthenticationParam, authentication);
            authentication.setBusinessLicense(businessLicense);
            User user = userService.getById(insertCompanyAuthenticationParam.getUserId());
            if (user != null) {
                authentication.setUserId(user.getId());
                authentication.setUserName(user.getUserName());
                authentication.setUserPhone(user.getPhone());
            }
            authentication.setCreateTime(new Date());
            return super.save(authentication);
        }
    }

    @Override
    public Paging<CompanyAuthentication> getCompanyAuthenticationPageList(CompanyAuthenticationPageParam companyAuthenticationPageParam) throws Exception {
        Page<CompanyAuthentication> page = new PageInfo<>(companyAuthenticationPageParam, OrderItem.desc("create_time"));
        IPage<CompanyAuthentication> iPage = companyAuthenticationMapper.selectPage(page, QueryHelpPlus.getPredicate(CompanyAuthentication.class, companyAuthenticationPageParam));
        return new Paging<CompanyAuthentication>(iPage);
    }

}
