package com.cloud.web.service.impl;

import com.cloud.web.entity.Company;

import com.cloud.web.mapper.CompanyMapper;
import com.cloud.web.service.CompanyService;
import com.cloud.web.param.CompanyPageParam;
import com.cloud.web.vo.CompanyQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class CompanyServiceImpl extends BaseServiceImpl<CompanyMapper, Company> implements CompanyService {

    @Autowired
    private CompanyMapper companyMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveCompany(Company company) throws Exception {
        return super.save(company);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateCompany(Company company)throws Exception{
            return super.updateById(company);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteCompany(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public CompanyQueryVo getCompanyById(Serializable id)throws Exception{
            return companyMapper.getCompanyById(id);
            }

    @Override
    public Paging<CompanyQueryVo> getCompanyPageListBySql(CompanyPageParam companyPageParam)throws Exception{
            Page<CompanyQueryVo> page=new PageInfo<>(companyPageParam,OrderItem.desc("create_time"));
                IPage<CompanyQueryVo> iPage= companyMapper.getCompanyPageList(page, companyPageParam);
            return new Paging<CompanyQueryVo>(iPage);
            }
    @Override
    public Paging<Company> getCompanyPageList(CompanyPageParam companyPageParam)throws Exception{
        Page<Company> page=new PageInfo<>(companyPageParam,OrderItem.desc("create_time"));
            IPage<Company> iPage= companyMapper.selectPage(page,QueryHelpPlus.getPredicate(Company.class, companyPageParam));
        return new Paging<Company>(iPage);
        }

}
