package com.cloud.web.service.impl;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.DrawDocument;

import com.cloud.web.mapper.DrawDocumentMapper;
import com.cloud.web.params.page.MyDocumentPageParam;
import com.cloud.web.service.DrawDocumentService;
import com.cloud.web.param.DrawDocumentPageParam;
import com.cloud.web.vo.DrawDocumentQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class DrawDocumentServiceImpl extends BaseServiceImpl<DrawDocumentMapper, DrawDocument> implements DrawDocumentService {

    @Autowired
    private DrawDocumentMapper drawDocumentMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveDrawDocument(DrawDocument drawDocument) throws Exception {
        return super.save(drawDocument);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateDrawDocument(DrawDocument drawDocument)throws Exception{
            return super.updateById(drawDocument);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteDrawDocument(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public DrawDocumentQueryVo getDrawDocumentById(Serializable id)throws Exception{
            return drawDocumentMapper.getDrawDocumentById(id);
            }

    @Override
    public Paging<DrawDocumentQueryVo> getDrawDocumentPageListBySql(DrawDocumentPageParam drawDocumentPageParam)throws Exception{
            Page<DrawDocumentQueryVo> page=new PageInfo<>(drawDocumentPageParam,OrderItem.desc("create_time"));
                IPage<DrawDocumentQueryVo> iPage= drawDocumentMapper.getDrawDocumentPageList(page, drawDocumentPageParam);
            return new Paging<DrawDocumentQueryVo>(iPage);
            }

    @Override
    public ApiPageResult<DrawDocumentQueryVo> getList(MyDocumentPageParam myDocumentPageParam) throws Exception {
        DrawDocumentPageParam pageParam=new DrawDocumentPageParam();
        pageParam.setPageIndex(myDocumentPageParam.getOffset());
        pageParam.setPageSize(myDocumentPageParam.getPageSize());
        pageParam.setUid(myDocumentPageParam.getUserId());
        Page<DrawDocumentQueryVo> page=new PageInfo<>(pageParam);
        IPage<DrawDocumentQueryVo> iPage=drawDocumentMapper.getDrawDocumentPageList(page,pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public Paging<DrawDocument> getDrawDocumentPageList(DrawDocumentPageParam drawDocumentPageParam)throws Exception{
        Page<DrawDocument> page=new PageInfo<>(drawDocumentPageParam,OrderItem.desc("create_time"));
            IPage<DrawDocument> iPage= drawDocumentMapper.selectPage(page,QueryHelpPlus.getPredicate(DrawDocument.class, drawDocumentPageParam));
        return new Paging<DrawDocument>(iPage);
        }

}
