package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.web.entity.*;

import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.mapper.DrawMapper;
import com.cloud.web.service.*;
import com.cloud.web.param.DrawPageParam;
import com.cloud.web.vo.DrawQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cloud.web.vo.DrawVo;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class DrawServiceImpl extends BaseServiceImpl<DrawMapper, Draw> implements DrawService {

    @Autowired
    private DrawMapper drawMapper;
    @Autowired
    private UserService userService;
    @Autowired
    private IntegralBillService integralBillService;
    @Autowired
    private DrawDocumentService drawDocumentService;
    @Autowired
    private PunchCardService punchCardService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveDraw(Draw draw) throws Exception {
        return super.save(draw);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateDraw(Draw draw) throws Exception {
        return super.updateById(draw);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteDraw(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public DrawQueryVo getDrawById(Serializable id) throws Exception {
        return drawMapper.getDrawById(id);
    }

    @Override
    public Paging<DrawQueryVo> getDrawPageListBySql(DrawPageParam drawPageParam) throws Exception {
        Page<DrawQueryVo> page = new PageInfo<>(drawPageParam, OrderItem.desc("create_time"));
        IPage<DrawQueryVo> iPage = drawMapper.getDrawPageList(page, drawPageParam);
        return new Paging<DrawQueryVo>(iPage);
    }

    @Override
    public List<Draw> getList() {
        return drawMapper.selectList(new QueryWrapper<Draw>());
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public DrawVo raffle(Long userId) throws Exception {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        String nowDate = sdf.format(new Date());
        //当天抽奖次数
        int count = drawDocumentService.count(new LambdaQueryWrapper<DrawDocument>().eq(DrawDocument::getUserId, userId).like(DrawDocument::getCreateTime, nowDate));
        if (count > 0) {
            throw new AndroidVerifyException(ApiCode.DRAW_LIMITED);
        }
        User user = userService.getById(userId);
        if (user == null) {
            throw new AndroidVerifyException(ApiCode.UNAUTHORIZED);
        }
        if (user.getDrawNumber() <= 0 && user.getIntegral() <= 0) {
            throw new AndroidVerifyException(ApiCode.RAFFLE_INSUFFIVIENTS);
        }
        List<Draw> list = drawMapper.selectList(new QueryWrapper<>());
        if (list == null && list.size() == 0) {
            throw new AndroidVerifyException("没有奖品");
        }
        //模拟概率抽奖
        Collections.shuffle(list);
        int all = 0;
        for (int i = 0; i < list.size(); i++) {
            list.get(i).setProbability(all += list.get(i).getProbability());
        }
        int random = (int) (Math.random() * all);
        Draw draw = null;
        for (Draw d : list) {
            if (d.getProbability() > random) {
                draw = d;
                break;
            }
        }
        if (draw == null) {
            throw new AndroidVerifyException("抽奖失败");
        }

        //添加抽奖记录
        DrawDocument drawDocument = new DrawDocument();
        drawDocument.setUserId(user.getId());
        drawDocument.setUserName(user.getUserName());
        drawDocument.setDrawId(draw.getId());
        drawDocument.setDrawName(draw.getName());
        drawDocument.setDrawIntegral(draw.getIntegral());
        drawDocument.setCreateTime(new Date());
        boolean fg = drawDocumentService.saveDrawDocument(drawDocument);
        if (!fg) {
            throw new AndroidVerifyException("抽奖失败");
        }
        //添加积分变动记录
        if (draw.getIntegral() > 0) {
            IntegralBill integralBill = new IntegralBill();
            integralBill.setTitle("抽奖");
            integralBill.setAmount("+" + draw.getIntegral());
            integralBill.setInOut(1);
            integralBill.setUid(user.getId());
            integralBill.setUname(user.getUserName());
            integralBill.setCreateTime(new Date());
            integralBill.setType(1);
            integralBill.setPhone(user.getPhone());
            boolean flg = integralBillService.save(integralBill);
            if (!flg) {
                throw new AndroidVerifyException("抽奖失败");
            }
            user.setIntegral(user.getIntegral() + draw.getIntegral());
        }
        //用户已抽奖次数减一
        if (user.getDrawNumber() > 0) {
            user.setDrawNumber(user.getDrawNumber() - 1);
        } else if (user.getDrawNumber() <= 0 && user.getIntegral() > 0) {
            user.setIntegral(user.getIntegral() - 1);
            IntegralBill integralBill = new IntegralBill();
            integralBill.setTitle("抽奖");
            integralBill.setAmount("-1");
            integralBill.setInOut(2);
            integralBill.setUid(user.getId());
            integralBill.setUname(user.getUserName());
            integralBill.setCreateTime(new Date());
            integralBill.setType(1);
            integralBill.setPhone(user.getPhone());
            integralBillService.save(integralBill);
        }
        boolean flag = userService.updateById(user);
        if (!flag) {
            throw new AndroidVerifyException("抽奖失败");
        }
        DrawVo drawVo = new DrawVo();
        drawVo.setId(draw.getId());
        drawVo.setName(draw.getName());
        drawVo.setDrawNumber(user.getDrawNumber() + user.getIntegral());
        return drawVo;
    }

    @Override
    public Paging<Draw> getDrawPageList(DrawPageParam drawPageParam) throws Exception {
        Page<Draw> page = new PageInfo<>(drawPageParam, OrderItem.desc("create_time"));
        IPage<Draw> iPage = drawMapper.selectPage(page, QueryHelpPlus.getPredicate(Draw.class, drawPageParam));
        return new Paging<Draw>(iPage);
    }


    public static Date getStartTime() {
        Calendar todayStart = Calendar.getInstance();
        todayStart.set(Calendar.HOUR_OF_DAY, 0);
        todayStart.set(Calendar.MINUTE, 0);
        todayStart.set(Calendar.SECOND, 0);
        todayStart.set(Calendar.MILLISECOND, 0);
        return todayStart.getTime();
    }

    public static Date getEndTime() {
        Calendar todayEnd = Calendar.getInstance();
        todayEnd.set(Calendar.HOUR_OF_DAY, 23);
        todayEnd.set(Calendar.MINUTE, 59);
        todayEnd.set(Calendar.SECOND, 59);
        todayEnd.set(Calendar.MILLISECOND, 999);
        return todayEnd.getTime();
    }
}
