package com.cloud.web.service.impl;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.HelpCenter;

import com.cloud.web.mapper.HelpCenterMapper;
import com.cloud.web.params.page.MyHelpPageParam;
import com.cloud.web.service.HelpCenterService;
import com.cloud.web.param.HelpCenterPageParam;
import com.cloud.web.vo.HelpCenterQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class HelpCenterServiceImpl extends BaseServiceImpl<HelpCenterMapper, HelpCenter> implements HelpCenterService {

    @Autowired
    private HelpCenterMapper helpCenterMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveHelpCenter(HelpCenter helpCenter) throws Exception {
        return super.save(helpCenter);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateHelpCenter(HelpCenter helpCenter)throws Exception{
            return super.updateById(helpCenter);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteHelpCenter(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public HelpCenterQueryVo getHelpCenterById(Serializable id)throws Exception{
            return helpCenterMapper.getHelpCenterById(id);
            }

    @Override
    public Paging<HelpCenterQueryVo> getHelpCenterPageListBySql(HelpCenterPageParam helpCenterPageParam)throws Exception{
            Page<HelpCenterQueryVo> page=new PageInfo<>(helpCenterPageParam,OrderItem.desc("create_time"));
                IPage<HelpCenterQueryVo> iPage= helpCenterMapper.getHelpCenterPageList(page, helpCenterPageParam);
            return new Paging<HelpCenterQueryVo>(iPage);
            }

    @Override
    public ApiPageResult<HelpCenterQueryVo> getHelpCenterQueryVo(MyHelpPageParam myHelpPageParam) throws Exception {
        HelpCenterPageParam pageParam=new HelpCenterPageParam();
        pageParam.setPageIndex(myHelpPageParam.getOffset());
        pageParam.setPageSize(myHelpPageParam.getPageSize());
        Page<HelpCenterQueryVo> page=new PageInfo<>(pageParam);
        IPage<HelpCenterQueryVo> iPage=helpCenterMapper.getHelpCenterPageList(page,pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public Paging<HelpCenter> getHelpCenterPageList(HelpCenterPageParam helpCenterPageParam)throws Exception{
        Page<HelpCenter> page=new PageInfo<>(helpCenterPageParam,OrderItem.desc("create_time"));
            IPage<HelpCenter> iPage= helpCenterMapper.selectPage(page,QueryHelpPlus.getPredicate(HelpCenter.class, helpCenterPageParam));
        return new Paging<HelpCenter>(iPage);
        }

}
