package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cloud.web.entity.HotSearch;

import com.cloud.web.mapper.HotSearchMapper;
import com.cloud.web.service.HotSearchService;
import com.cloud.web.param.HotSearchPageParam;
import com.cloud.web.vo.HotSearchQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import java.util.List;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-14
 */
@Slf4j
@Service
public class HotSearchServiceImpl extends BaseServiceImpl<HotSearchMapper, HotSearch> implements HotSearchService {

    @Autowired
    private HotSearchMapper hotSearchMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveHotSearch(HotSearch hotSearch) throws Exception {
        return super.save(hotSearch);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateHotSearch(HotSearch hotSearch)throws Exception{
            return super.updateById(hotSearch);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteHotSearch(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public HotSearchQueryVo getHotSearchById(Serializable id)throws Exception{
            return hotSearchMapper.getHotSearchById(id);
            }

    @Override
    public Paging<HotSearchQueryVo> getHotSearchPageListBySql(HotSearchPageParam hotSearchPageParam)throws Exception{
            Page<HotSearchQueryVo> page=new PageInfo<>(hotSearchPageParam,OrderItem.desc("create_time"));
                IPage<HotSearchQueryVo> iPage= hotSearchMapper.getHotSearchPageList(page, hotSearchPageParam);
            return new Paging<HotSearchQueryVo>(iPage);
            }

    @Override
    public List<HotSearch> getSearch() {
        return hotSearchMapper.selectList(new QueryWrapper<>());
    }

    @Override
    public Paging<HotSearch> getHotSearchPageList(HotSearchPageParam hotSearchPageParam)throws Exception{
        Page<HotSearch> page=new PageInfo<>(hotSearchPageParam,OrderItem.desc("create_time"));
            IPage<HotSearch> iPage= hotSearchMapper.selectPage(page,QueryHelpPlus.getPredicate(HotSearch.class, hotSearchPageParam));
        return new Paging<HotSearch>(iPage);
        }

}
