package com.cloud.web.service.impl;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.IntegralBill;

import com.cloud.web.mapper.IntegralBillMapper;
import com.cloud.web.params.page.MyIntegralPageParam;
import com.cloud.web.service.IntegralBillService;
import com.cloud.web.param.IntegralBillPageParam;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.IntegralBillQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class IntegralBillServiceImpl extends BaseServiceImpl<IntegralBillMapper, IntegralBill> implements IntegralBillService {

    @Autowired
    private IntegralBillMapper integralBillMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveIntegralBill(IntegralBill integralBill) throws Exception {
        return super.save(integralBill);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateIntegralBill(IntegralBill integralBill)throws Exception{
            return super.updateById(integralBill);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteIntegralBill(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public IntegralBillQueryVo getIntegralBillById(Serializable id)throws Exception{
            return integralBillMapper.getIntegralBillById(id);
            }

    @Override
    public Paging<IntegralBillQueryVo> getIntegralBillPageListBySql(IntegralBillPageParam integralBillPageParam)throws Exception{
            Page<IntegralBillQueryVo> page=new PageInfo<>(integralBillPageParam,OrderItem.desc("create_time"));
                IPage<IntegralBillQueryVo> iPage= integralBillMapper.getIntegralBillPageList(page, integralBillPageParam);
            return new Paging<IntegralBillQueryVo>(iPage);
            }

    @Override
    public ApiPageResult<IntegralBillQueryVo> getBillVo(MyIntegralPageParam integralBillPageParam) throws Exception {
        IntegralBillPageParam pageParam=new IntegralBillPageParam();
        pageParam.setPageIndex(integralBillPageParam.getOffset());
        pageParam.setPageSize(integralBillPageParam.getPageSize());
        pageParam.setUid(integralBillPageParam.getUid());
        pageParam.setType(integralBillPageParam.getTypes());
        if (StringUtils.isNotBlank(integralBillPageParam.getBeginTime()) && StringUtils.isNotBlank(integralBillPageParam.getEndTime())){
          pageParam.setBeginTime(integralBillPageParam.getBeginTime());
          pageParam.setEndTime(integralBillPageParam.getEndTime());
        }
        Page<IntegralBillQueryVo> page=new PageInfo<>(pageParam);
        IPage<IntegralBillQueryVo> iPage=integralBillMapper.getIntegralBillPageList(page,pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public Paging<IntegralBill> getIntegralBillPageList(IntegralBillPageParam integralBillPageParam)throws Exception{
        Page<IntegralBill> page=new PageInfo<>(integralBillPageParam,OrderItem.desc("create_time"));
            IPage<IntegralBill> iPage= integralBillMapper.selectPage(page,QueryHelpPlus.getPredicate(IntegralBill.class, integralBillPageParam));
        return new Paging<IntegralBill>(iPage);
        }

}
