package com.cloud.web.service.impl;

import com.cloud.web.entity.Location;

import com.cloud.web.entity.User;
import com.cloud.web.mapper.LocationMapper;
import com.cloud.web.params.param.LocationParam;
import com.cloud.web.service.LocationService;
import com.cloud.web.param.LocationPageParam;
import com.cloud.web.service.UserService;
import com.cloud.web.vo.LocationQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-04-01
 */
@Slf4j
@Service
public class LocationServiceImpl extends BaseServiceImpl<LocationMapper, Location> implements LocationService {

    @Autowired
    private LocationMapper locationMapper;
    @Autowired
    private UserService userService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveLocation(Location location) throws Exception {
        return super.save(location);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateLocation(Location location)throws Exception{
            return super.updateById(location);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteLocation(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public LocationQueryVo getLocationById(Serializable id)throws Exception{
            return locationMapper.getLocationById(id);
            }

    @Override
    public Paging<LocationQueryVo> getLocationPageListBySql(LocationPageParam locationPageParam)throws Exception{
            Page<LocationQueryVo> page=new PageInfo<>(locationPageParam,OrderItem.desc("create_time"));
                IPage<LocationQueryVo> iPage= locationMapper.getLocationPageList(page, locationPageParam);
            return new Paging<LocationQueryVo>(iPage);
            }

    @Override
    public void addLocation(LocationParam locationParam) {
        User user = userService.getById(locationParam.getUserId());
        Location location = new Location();
        if (user != null){
            location.setUserId(user.getId());
            location.setUserName(user.getUserName());
            location.setAddress(locationParam.getAddress());
            locationMapper.insert(location);
        }
    }

    @Override
    public Paging<Location> getLocationPageList(LocationPageParam locationPageParam)throws Exception{
        Page<Location> page=new PageInfo<>(locationPageParam,OrderItem.desc("create_time"));
            IPage<Location> iPage= locationMapper.selectPage(page,QueryHelpPlus.getPredicate(Location.class, locationPageParam));
        return new Paging<Location>(iPage);
        }

}
