package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.web.entity.*;

import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.mapper.MemberRecordsMapper;
import com.cloud.web.modules.pay.wechat.service.WeChatPayServices;
import com.cloud.web.modules.send.utils.NoteSendUtil;
import com.cloud.web.service.*;
import com.cloud.web.param.MemberRecordsPageParam;
import com.cloud.web.util.RedisUtils;
import com.cloud.web.vo.MemberRecordsQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;
import java.util.Map;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

import javax.servlet.http.HttpServletRequest;

/**
 * 服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class MemberRecordsServiceImpl extends BaseServiceImpl<MemberRecordsMapper, MemberRecords> implements MemberRecordsService {

    @Autowired
    private MemberRecordsMapper memberRecordsMapper;
    @Autowired
    private BaseSettingService baseSettingService;
    @Autowired
    private RedisUtils redisUtils;
    @Autowired
    private WeChatPayServices weChatPayServices;
    @Autowired
    private MemberRecordsService memberRecordsService;
    @Autowired
    private UserService userService;
    @Autowired
    private MessageService messageService;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private IntegralBillService integralBillService;
    @Autowired
    private ApplePriceSettingService applePriceSettingService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveMemberRecords(MemberRecords memberRecords) throws Exception {
        return super.save(memberRecords);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateMemberRecords(MemberRecords memberRecords) throws Exception {
        return super.updateById(memberRecords);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteMemberRecords(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public MemberRecordsQueryVo getMemberRecordsById(Serializable id) throws Exception {
        return memberRecordsMapper.getMemberRecordsById(id);
    }

    @Override
    public Paging<MemberRecordsQueryVo> getMemberRecordsPageListBySql(MemberRecordsPageParam memberRecordsPageParam) throws Exception {
        Page<MemberRecordsQueryVo> page = new PageInfo<>(memberRecordsPageParam, OrderItem.desc("create_time"));
        IPage<MemberRecordsQueryVo> iPage = memberRecordsMapper.getMemberRecordsPageList(page, memberRecordsPageParam);
        return new Paging<MemberRecordsQueryVo>(iPage);
    }

    @Override
    public Map<String, Object> goWeChatPay(Long userId, int type, HttpServletRequest request) {
        User user = userService.getById(userId);
        if (user == null) {
            throw new AndroidVerifyException(ApiCode.USER_NOT_EXIT);
        }
        MemberRecords memberRecords = new MemberRecords();
        String orderNum = "SN" + NoteSendUtil.getRandom(13);
        if (type == 1) {
            memberRecords.setOrderId(orderNum);
            memberRecords.setMoney(baseSettingService.getOne(new QueryWrapper<>()).getMemberPrice());
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(1);
            memberRecords.setType(1);
            memberRecordsMapper.insert(memberRecords);
            redisUtils.set(orderNum, orderNum);
        } else if (type == 2) {
            memberRecords.setOrderId(orderNum);
            memberRecords.setMoney(baseSettingService.getOne(new QueryWrapper<>()).getReferrerPrice());
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(1);
            memberRecords.setType(2);
            memberRecordsMapper.insert(memberRecords);
            redisUtils.set(orderNum, orderNum);
        } else if (type == 3) {
            memberRecords.setOrderId(orderNum);
            memberRecords.setMoney(baseSettingService.getOne(new QueryWrapper<>()).getAuthenticationPrice());
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(1);
            memberRecords.setType(3);
            memberRecordsMapper.insert(memberRecords);
            redisUtils.set(orderNum, orderNum);
        } else if (type == 4) {
            memberRecords.setOrderId(orderNum);
            Object obj = redisUtils.get("recharge:" + userId);
            BigDecimal bigDecimal = new BigDecimal(obj.toString());
            memberRecords.setMoney(bigDecimal);
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(1);
            memberRecords.setType(4);
            memberRecordsMapper.insert(memberRecords);
            redisUtils.del("recharge:" + userId);
            redisUtils.set(orderNum, orderNum);
        }
        return weChatPayServices.goWeChatPay(memberRecords, request);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean paySuccessHandle(String payId) throws Exception {
        try {
            //获取订单
            MemberRecords pay = this.getOrderId(payId);
            if (redisUtils.hasKey(pay.getOrderId())) {
                if (null == pay) {
                    log.error("订单不存在");
                    return false;
                }
                //更新订单状态
                LambdaUpdateWrapper<MemberRecords> wrapper = new LambdaUpdateWrapper<>();
                wrapper.set(MemberRecords::getPaid, 2);
                wrapper.eq(MemberRecords::getId, pay.getId());
                memberRecordsService.update(wrapper);
                log.error("------------>>>>>>>>>>>>>>>修改订单状态完成");
                //当前用户
                User myUser = userService.getById(pay.getUid());
                if (pay.getType() == 1) {
                    myUser.setIsMember(2);
                    //修改会员状态为普通会员
                    userService.updateUser(myUser);
                    redisUtils.del(pay.getOrderId());
                    Message message = new Message();
                    message.setTitle("会员开通成功提醒");
                    message.setContent("尊敬的用户,您已成功开通会员");
                    message.setUid(myUser.getId());
                    message.setStatus(1);
                    message.setCreateTime(new Date());
                    messageService.save(message);
                } else if (pay.getType() == 2) {
                    myUser.setIsReferrer(2);
                    //修改推荐管状态
                    userService.updateUser(myUser);
                    redisUtils.del(pay.getOrderId());
                    Message message = new Message();
                    message.setTitle("推荐管开通成功提醒");
                    message.setContent("尊敬的用户,您已成功开通推荐管");
                    message.setUid(myUser.getId());
                    message.setStatus(1);
                    message.setCreateTime(new Date());
                    messageService.save(message);
                } else if (pay.getType() == 3) {
                    log.error("------------>>>>>>>>>>>>>>>进入用户信息修改");
                    Company company1 = companyService.getOne(new LambdaQueryWrapper<Company>().eq(Company::getName, myUser.getUserName()));
                    if (company1 != null && company1.getId() > 0) {
                        myUser.setCompanyId(company1.getId());
                    } else {
                        Company company = new Company();
                        company.setName(myUser.getUserName());
                        companyService.save(company);
                        Company c = companyService.getOne(new LambdaQueryWrapper<Company>().eq(Company::getName, myUser.getUserName()));
                        myUser.setCompanyId(c.getId());
                    }
                    myUser.setIsCompany(2);
                    //开通金额转换为积分给企业用户
                    BaseSetting baseSetting = baseSettingService.getOne(new QueryWrapper<>());
                    BigDecimal integral = pay.getMoney().multiply(baseSetting.getRechargeProportion()).setScale(0, BigDecimal.ROUND_UP);
                    myUser.setIntegral(myUser.getIntegral() + integral.intValue());
                    //修改账号状态
                    userService.updateUser(myUser);
                    redisUtils.del(pay.getOrderId());
                    Message message = new Message();
                    message.setTitle("企业号开通成功提醒");
                    message.setContent("尊敬的用户,您已成功开通企业号");
                    message.setUid(myUser.getId());
                    message.setStatus(1);
                    message.setCreateTime(new Date());
                    messageService.save(message);
                    log.error("------------>>>>>>>>>>>>>>>用户信息修改完成");
                } else if (pay.getType() == 4) {
                    BaseSetting baseSetting = baseSettingService.getOne(new QueryWrapper<>());
                    BigDecimal integral = pay.getMoney().multiply(baseSetting.getRechargeProportion()).setScale(0, BigDecimal.ROUND_UP);
                    myUser.setIntegral(myUser.getIntegral() + integral.intValue());
                    userService.updateUser(myUser);
                    //积分记录
                    IntegralBill integralBill = new IntegralBill();
                    integralBill.setTitle("企业充值");
                    integralBill.setAmount("+" + integral);
                    integralBill.setInOut(1);
                    integralBill.setUid(myUser.getId());
                    integralBill.setUname(myUser.getUserName());
                    integralBill.setPhone(myUser.getPhone());
                    integralBill.setCreateTime(new Date());
                    integralBill.setType(4);
                    integralBillService.save(integralBill);
                }
            } else {
                return false;
            }
        } catch (Exception e) {
            return false;
        }
        return true;
    }

    @Override
    public MemberRecords getOrderId(String payId) {
        LambdaQueryWrapper<MemberRecords> wrapper = new LambdaQueryWrapper<>();
        wrapper.eq(MemberRecords::getOrderId, payId);
        return this.getOne(wrapper);
    }

    @Override
    public void applePay(Long userId, int type, String productId, HttpServletRequest request) throws Exception {
        User user = userService.getById(userId);
        MemberRecords memberRecords = new MemberRecords();
        String orderNum = "SN" + NoteSendUtil.getRandom(13);
        Message message = new Message();
        if (type == 2) {
            memberRecords.setOrderId(orderNum);
            memberRecords.setMoney(baseSettingService.getOne(new QueryWrapper<>()).getMemberPrice());
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(2);
            memberRecords.setType(1);
            memberRecordsMapper.insert(memberRecords);
            user.setIsMember(2);
            //修改会员状态为普通会员
            userService.updateUser(user);
            message.setTitle("会员开通成功提醒");
            message.setContent("尊敬的用户,您已成功开通会员");
            message.setUid(user.getId());
            message.setStatus(1);
            message.setCreateTime(new Date());
            messageService.save(message);
        } else if (type == 3) {
            memberRecords.setOrderId(orderNum);
            memberRecords.setMoney(baseSettingService.getOne(new QueryWrapper<>()).getReferrerPrice());
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(2);
            memberRecords.setType(2);
            memberRecordsMapper.insert(memberRecords);
            user.setIsReferrer(2);
            //修改推荐管状态
            userService.updateUser(user);
            message.setTitle("推荐管开通成功提醒");
            message.setContent("尊敬的用户,您已成功开通推荐管");
            message.setUid(user.getId());
            message.setStatus(1);
            message.setCreateTime(new Date());
            messageService.save(message);
        } else if (type == 1) {
            memberRecords.setOrderId(orderNum);
            memberRecords.setMoney(baseSettingService.getOne(new QueryWrapper<>()).getAuthenticationPrice());
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(2);
            memberRecords.setType(3);
            memberRecordsMapper.insert(memberRecords);
            log.error("---->>>>>进入用户信息修改");
            Company company = new Company();
            company.setName(user.getUserName());
            companyService.save(company);
            Company company1 = companyService.getOne(new LambdaQueryWrapper<Company>().eq(Company::getName, user.getUserName()));
            if (company1 != null && company1.getId() > 0) {
                user.setCompanyId(company1.getId());
            }
            user.setIsCompany(2);
            //开通金额转换为积分给企业用户
            BaseSetting baseSetting = baseSettingService.getOne(new QueryWrapper<>());
            BigDecimal integral = baseSetting.getAuthenticationPrice().multiply(baseSetting.getRechargeProportion()).setScale(0, BigDecimal.ROUND_UP);
            user.setIntegral(user.getIntegral() + integral.intValue());
            //修改账号状态
            userService.updateUser(user);
            message.setTitle("企业号开通成功提醒");
            message.setContent("尊敬的用户,您已成功开通企业号");
            message.setUid(user.getId());
            message.setStatus(1);
            message.setCreateTime(new Date());
            messageService.save(message);
        } else if (type == 4) {
            memberRecords.setOrderId(orderNum);
            ApplePriceSetting applePriceSetting = applePriceSettingService.getOne(new LambdaQueryWrapper<ApplePriceSetting>().eq(ApplePriceSetting::getProductId, productId));
            memberRecords.setMoney(new BigDecimal(applePriceSetting.getIntergal()));
            memberRecords.setUid(user.getId());
            memberRecords.setUname(user.getUserName());
            memberRecords.setPhone(user.getPhone());
            memberRecords.setPaid(1);
            memberRecords.setType(4);
            memberRecordsMapper.insert(memberRecords);
            BaseSetting baseSetting = baseSettingService.getOne(new QueryWrapper<>());
            BigDecimal integral = new BigDecimal(applePriceSetting.getIntergal()).multiply(baseSetting.getRechargeProportion()).setScale(0, BigDecimal.ROUND_UP);
            user.setIntegral(user.getIntegral() + integral.intValue());
            userService.updateUser(user);
            //积分记录
            IntegralBill integralBill = new IntegralBill();
            integralBill.setTitle("企业充值");
            integralBill.setAmount("+" + integral);
            integralBill.setInOut(1);
            integralBill.setUid(user.getId());
            integralBill.setUname(user.getUserName());
            integralBill.setPhone(user.getPhone());
            integralBill.setCreateTime(new Date());
            integralBill.setType(4);
            integralBillService.save(integralBill);
        }
    }

    @Override
    public Paging<MemberRecords> getMemberRecordsPageList(MemberRecordsPageParam memberRecordsPageParam) throws Exception {
        Page<MemberRecords> page = new PageInfo<>(memberRecordsPageParam, OrderItem.desc("create_time"));
        IPage<MemberRecords> iPage = memberRecordsMapper.selectPage(page, QueryHelpPlus.getPredicate(MemberRecords.class, memberRecordsPageParam));
        return new Paging<MemberRecords>(iPage);
    }

}
