package com.cloud.web.service.impl;

import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.Message;

import com.cloud.web.mapper.MessageMapper;
import com.cloud.web.params.page.MyMessagePageParam;
import com.cloud.web.service.MessageService;
import com.cloud.web.param.MessagePageParam;
import com.cloud.web.vo.MessageQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class MessageServiceImpl extends BaseServiceImpl<MessageMapper, Message> implements MessageService {

    @Autowired
    private MessageMapper messageMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveMessage(Message message) throws Exception {
        return super.save(message);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateMessage(Message message)throws Exception{
            return super.updateById(message);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteMessage(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public MessageQueryVo getMessageById(Serializable id)throws Exception{
            return messageMapper.getMessageById(id);
            }

    @Override
    public Paging<MessageQueryVo> getMessagePageListBySql(MessagePageParam messagePageParam)throws Exception{
            Page<MessageQueryVo> page=new PageInfo<>(messagePageParam,OrderItem.desc("create_time"));
                IPage<MessageQueryVo> iPage= messageMapper.getMessagePageList(page, messagePageParam);
            return new Paging<MessageQueryVo>(iPage);
            }

    @Override
    public ApiPageResult<MessageQueryVo> getMessageQueryVo(MyMessagePageParam myMessagePageParam) throws Exception {
        MessagePageParam pageParam=new MessagePageParam();
        pageParam.setPageIndex(myMessagePageParam.getOffset());
        pageParam.setPageSize(myMessagePageParam.getPageSize());
        pageParam.setUserId(myMessagePageParam.getUserId());
        Page<MessageQueryVo> page=new PageInfo<>(pageParam);
        IPage<MessageQueryVo> iPage=messageMapper.getMessagePageList(page,pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public Paging<Message> getMessagePageList(MessagePageParam messagePageParam)throws Exception{
        Page<Message> page=new PageInfo<>(messagePageParam,OrderItem.desc("create_time"));
            IPage<Message> iPage= messageMapper.selectPage(page,QueryHelpPlus.getPredicate(Message.class, messagePageParam));
        return new Paging<Message>(iPage);
        }

}
