package com.cloud.web.service.impl;

import com.cloud.framework.common.api.ApiCode;
import com.cloud.web.entity.IntegralBill;
import com.cloud.web.entity.PayrollRecords;

import com.cloud.web.entity.Post;
import com.cloud.web.entity.User;
import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.mapper.PayrollRecordsMapper;
import com.cloud.web.param.AddPayrollRecordsParam;
import com.cloud.web.service.IntegralBillService;
import com.cloud.web.service.PayrollRecordsService;
import com.cloud.web.param.PayrollRecordsPageParam;
import com.cloud.web.service.PostService;
import com.cloud.web.service.UserService;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.PayrollRecordsQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;
import java.util.Date;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 工资记录 服务实现类
 *
 * @author zyl
 * @since 2022-11-01
 */
@Slf4j
@Service
public class PayrollRecordsServiceImpl extends BaseServiceImpl<PayrollRecordsMapper, PayrollRecords> implements PayrollRecordsService {

    @Autowired
    private PayrollRecordsMapper payrollRecordsMapper;
    @Autowired
    private PostService postService;
    @Autowired
    private UserService userService;
    @Autowired
    private IntegralBillService integralBillService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean savePayrollRecords(PayrollRecords payrollRecords) throws Exception {
        return super.save(payrollRecords);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updatePayrollRecords(PayrollRecords payrollRecords) throws Exception {
        return super.updateById(payrollRecords);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deletePayrollRecords(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public PayrollRecordsQueryVo getPayrollRecordsById(Serializable id) throws Exception {
        return payrollRecordsMapper.getPayrollRecordsById(id);
    }

    @Override
    public Paging<PayrollRecordsQueryVo> getPayrollRecordsPageListBySql(PayrollRecordsPageParam payrollRecordsPageParam) throws Exception {
        Page<PayrollRecordsQueryVo> page = new PageInfo<>(payrollRecordsPageParam, OrderItem.desc("create_time"));
        IPage<PayrollRecordsQueryVo> iPage = payrollRecordsMapper.getPayrollRecordsPageList(page, payrollRecordsPageParam);
        return new Paging<PayrollRecordsQueryVo>(iPage);
    }

    @Override
    public boolean addPayrollRecords(AddPayrollRecordsParam addPayrollRecordsParam) throws Exception {
        Post post = postService.getById(addPayrollRecordsParam.getPostId());
        if (post != null) {
            User user = userService.getById(addPayrollRecordsParam.getUserId());
            if (user.getIntegral() < addPayrollRecordsParam.getTotalPayroll().intValue()) {
                throw new AndroidBusinessException(ApiCode.NOT_ORDER);
            }
            PayrollRecords payrollRecords = new PayrollRecords();
            payrollRecords.setPostId(post.getId());
            payrollRecords.setPostName(post.getName());
            payrollRecords.setCompanyId(post.getCompanyId());
            if (StringUtils.isNotBlank(post.getCompanyName())) {
                payrollRecords.setCompanyName(post.getCompanyName());
            }
            payrollRecords.setUserId(user.getId());
            payrollRecords.setUserName(user.getUserName());
            payrollRecords.setUserPhone(user.getPhone());
            payrollRecords.setUserIds(addPayrollRecordsParam.getUserIds());
            payrollRecords.setUserNumber(addPayrollRecordsParam.getUserNumber());
            payrollRecords.setPayroll(addPayrollRecordsParam.getPayroll());
            payrollRecords.setTotalPayroll(addPayrollRecordsParam.getTotalPayroll());
            payrollRecords.setCreateTime(new Date());
            if (StringUtils.isNotBlank(addPayrollRecordsParam.getRemark())) {
                payrollRecords.setRemark(addPayrollRecordsParam.getRemark());
            }
            payrollRecordsMapper.insert(payrollRecords);
            if (StringUtils.isNotBlank(addPayrollRecordsParam.getUserIds())) {
                String[] ids = addPayrollRecordsParam.getUserIds().split(",");
                for (String id : ids) {
                    User u = userService.getById(id);
                    if (u != null) {
                        //企业积分更新
                        user.setIntegral(user.getIntegral() - addPayrollRecordsParam.getPayroll().intValue());
                        user.setUpdateTime(new Date());
                        userService.updateUser(user);
                        IntegralBill integralBill = new IntegralBill();
                        integralBill.setTitle("工资结算");
                        integralBill.setAmount("-" + addPayrollRecordsParam.getTotalPayroll());
                        integralBill.setInOut(2);
                        integralBill.setUid(user.getId());
                        integralBill.setUname(user.getUserName());
                        integralBill.setPhone(user.getPhone());
                        integralBill.setCreateTime(new Date());
                        integralBill.setType(6);
                        integralBill.setRemark(post.getName() + "-" + u.getUserName() + "-" + addPayrollRecordsParam.getRemark());
                        integralBillService.save(integralBill);

                        //员工
                        u.setIntegral(u.getIntegral() + addPayrollRecordsParam.getPayroll().intValue());
                        u.setUpdateTime(new Date());
                        userService.updateUser(u);
                        IntegralBill bill = new IntegralBill();
                        bill.setTitle("工资");
                        bill.setAmount("+" + addPayrollRecordsParam.getPayroll());
                        bill.setInOut(1);
                        bill.setUid(u.getId());
                        bill.setUname(u.getUserName());
                        bill.setPhone(u.getPhone());
                        bill.setCreateTime(new Date());
                        bill.setType(6);
                        if (StringUtils.isNotBlank(post.getCompanyName())) {
                            bill.setRemark(post.getCompanyName() + "-" + post.getName() + "-" + addPayrollRecordsParam.getRemark());
                        } else {
                            bill.setRemark(post.getName() + "-" + addPayrollRecordsParam.getRemark());
                        }
                        integralBillService.save(bill);
                    }
                }
                return true;
            }
        }
        return false;
    }

    @Override
    public Paging<PayrollRecords> getPayrollRecordsPageList(PayrollRecordsPageParam payrollRecordsPageParam) throws Exception {
        Page<PayrollRecords> page = new PageInfo<>(payrollRecordsPageParam, OrderItem.desc("create_time"));
        IPage<PayrollRecords> iPage = payrollRecordsMapper.selectPage(page, QueryHelpPlus.getPredicate(PayrollRecords.class, payrollRecordsPageParam));
        return new Paging<PayrollRecords>(iPage);
    }

}
