package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.web.entity.*;

import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.mapper.PostMapper;
import com.cloud.web.param.AddPostParam;
import com.cloud.web.params.page.MyPostPageParam;
import com.cloud.web.service.*;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.CompanyQueryVo;
import com.cloud.web.vo.PostInfoVo;
import com.cloud.web.param.PostPageParam;
import com.cloud.web.vo.PostQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cloud.web.vo.PostVo;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class PostServiceImpl extends BaseServiceImpl<PostMapper, Post> implements PostService {

    @Autowired
    private PostMapper postMapper;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private ApplyService applyService;
    @Autowired
    private UserService userService;
    @Autowired
    private BaseSettingService baseSettingService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean savePost(Post post) throws Exception {
        return super.save(post);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updatePost(Post post) throws Exception {
        return super.updateById(post);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deletePost(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public PostQueryVo getPostById(Serializable id) throws Exception {
        return postMapper.getPostById(id);
    }

    @Override
    public Paging<PostQueryVo> getPostPageListBySql(PostPageParam postPageParam) throws Exception {
        Page<PostQueryVo> page = new PageInfo<>(postPageParam, OrderItem.desc("create_time"));
        IPage<PostQueryVo> iPage = postMapper.getPostPageList(page, postPageParam);
        return new Paging<PostQueryVo>(iPage);
    }

    @Override
    public ApiPageResult<PostQueryVo> getPostVo(MyPostPageParam myPostPageParam) throws Exception {
        User user = userService.getById(myPostPageParam.getUserId());
        PostPageParam pageParam = new PostPageParam();
        pageParam.setPageIndex(myPostPageParam.getOffset());
        pageParam.setPageSize(myPostPageParam.getPageSize());
        if (user != null && user.getIsCompany() == 2) {
            pageParam.setCompanyId(user.getCompanyId());
        }
        if (StringUtils.isNotBlank(myPostPageParam.getName())) {
            pageParam.setName(myPostPageParam.getName());
        }
        if (StringUtils.isNotBlank(myPostPageParam.getAddress())) {
            pageParam.setAddress(myPostPageParam.getAddress());
        }
        if (StringUtils.isNotBlank(myPostPageParam.getLongitude())) {
            pageParam.setLongitude(myPostPageParam.getLongitude());
        }
        if (StringUtils.isNotBlank(myPostPageParam.getLatitude())) {
            pageParam.setLatitude(myPostPageParam.getLatitude());
        }
        pageParam.setType(myPostPageParam.getType());
        Page<PostQueryVo> page = new PageInfo<>(pageParam);
        IPage<PostQueryVo> iPage = postMapper.getPostPageList(page, pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public ApiPageResult<PostQueryVo> getPostVos(MyPostPageParam myPostPageParam) throws Exception {
        User user = userService.getById(myPostPageParam.getUserId());
        PostPageParam pageParam = new PostPageParam();
        pageParam.setPageIndex(myPostPageParam.getOffset());
        pageParam.setPageSize(myPostPageParam.getPageSize());
        if (user != null && user.getIsCompany() == 2) {
            pageParam.setCompanyId(user.getCompanyId());
        }
        if (StringUtils.isNotBlank(myPostPageParam.getName())) {
            pageParam.setName(myPostPageParam.getName());
        }
        if (StringUtils.isNotBlank(myPostPageParam.getAddress())) {
            pageParam.setAddress(myPostPageParam.getAddress());
        }
        pageParam.setType(myPostPageParam.getType());
        Page<PostQueryVo> page = new PageInfo<>(pageParam);
        IPage<PostQueryVo> iPage = postMapper.getPostPageLists(page, pageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public Paging<Post> getPostPageList(PostPageParam postPageParam) throws Exception {
        Page<Post> page = new PageInfo<>(postPageParam, OrderItem.desc("create_time"));
        IPage<Post> iPage = postMapper.selectPage(page, QueryHelpPlus.getPredicate(Post.class, postPageParam));
        return new Paging<Post>(iPage);
    }

    @Override
    public PostInfoVo getOne(Integer id, Long userId) throws Exception {
        PostQueryVo post = postMapper.getPostById(id);
        if (StringUtils.isEmpty(post)) {
            throw new AndroidVerifyException(ApiCode.IS_DELETED);
        }
        PostVo postVo = new PostVo();
        BeanUtils.copyProperties(post, postVo);
        BaseSetting baseSetting = baseSettingService.getOne();
        if (baseSetting != null && StringUtils.isNotBlank(baseSetting.getConsumeExplain())) {
            postVo.setConsumeExplain(baseSetting.getConsumeExplain());
        }
        PostInfoVo postInfoVo = new PostInfoVo();
        postInfoVo.setPostVo(postVo);
        CompanyQueryVo companyQueryVo = companyService.getCompanyById(postVo.getCompanyId());
        if (companyQueryVo != null) {
            postInfoVo.setIntrodutions(companyQueryVo.getIntrodution());
            postInfoVo.setLogo(companyQueryVo.getLogo());
        }
        QueryWrapper<Apply> wrapper = new QueryWrapper();
        wrapper.eq("user_id", userId).eq("post_id", post.getId());
        Apply oldApply = applyService.getOne(wrapper);
        if (StringUtils.isEmpty(oldApply)) {
            postInfoVo.setStatus(1);
        } else if (oldApply != null && oldApply.getStatus() < 3) {
            postInfoVo.setStatus(0);
        } else if (oldApply != null && oldApply.getStatus() == 3) {
            postInfoVo.setStatus(2);
        }
        return postInfoVo;
    }

    @Override
    public void delete(Integer id) {
        postMapper.remove(id);
    }

}
