package com.cloud.web.service.impl;

import com.cloud.web.entity.PunchCard;

import com.cloud.web.mapper.PunchCardMapper;
import com.cloud.web.service.PunchCardService;
import com.cloud.web.param.PunchCardPageParam;
import com.cloud.web.vo.PunchCardQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;
import java.io.Serializable;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 *  服务实现类
 *
 * @author zyl
 * @since 2021-10-08
 */
@Slf4j
@Service
public class PunchCardServiceImpl extends BaseServiceImpl<PunchCardMapper, PunchCard> implements PunchCardService {

    @Autowired
    private PunchCardMapper punchCardMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean savePunchCard(PunchCard punchCard) throws Exception {
        return super.save(punchCard);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updatePunchCard(PunchCard punchCard)throws Exception{
            return super.updateById(punchCard);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deletePunchCard(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public PunchCardQueryVo getPunchCardById(Serializable id)throws Exception{
            return punchCardMapper.getPunchCardById(id);
            }

    @Override
    public Paging<PunchCardQueryVo> getPunchCardPageListBySql(PunchCardPageParam punchCardPageParam)throws Exception{
            Page<PunchCardQueryVo> page=new PageInfo<>(punchCardPageParam,OrderItem.desc("create_time"));
                IPage<PunchCardQueryVo> iPage= punchCardMapper.getPunchCardPageList(page, punchCardPageParam);
            return new Paging<PunchCardQueryVo>(iPage);
            }

    @Override
    public void delete(Integer id) {
            punchCardMapper.delete(id);
    }

    @Override
    public Paging<PunchCard> getPunchCardPageList(PunchCardPageParam punchCardPageParam)throws Exception{
        Page<PunchCard> page=new PageInfo<>(punchCardPageParam,OrderItem.desc("create_time"));
            IPage<PunchCard> iPage= punchCardMapper.selectPage(page,QueryHelpPlus.getPredicate(PunchCard.class, punchCardPageParam));
        return new Paging<PunchCard>(iPage);
        }

}
