package com.cloud.web.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyuncs.cloudauth.model.v20190307.InitFaceVerifyResponse;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.api.R;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.system.service.UpLoadService;
import com.cloud.web.entity.User;
import com.cloud.web.entity.UserAuthentication;

import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.mapper.UserAuthenticationMapper;
import com.cloud.web.params.param.InsertUserAuthenticationParam;
import com.cloud.web.service.InitFaceVerifyService;
import com.cloud.web.service.UserAuthenticationService;
import com.cloud.web.param.UserAuthenticationPageParam;
import com.cloud.web.service.UserService;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.IdCardVo;
import com.cloud.web.vo.UserAuthenticationQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.IOException;
import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.gson.Gson;
import io.swagger.annotations.Api;
import okhttp3.FormBody;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

import javax.annotation.Resource;

/**
 * 用户实名认证 服务实现类
 *
 * @author zyl
 * @since 2022-11-13
 */
@Slf4j
@Service
public class UserAuthenticationServiceImpl extends BaseServiceImpl<UserAuthenticationMapper, UserAuthentication> implements UserAuthenticationService {

    @Autowired
    private UserAuthenticationMapper userAuthenticationMapper;
    @Resource
    private UpLoadService upLoadService;
    @Autowired
    private UserService userService;
    @Autowired
    private InitFaceVerifyService initFaceVerifyService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveUserAuthentication(UserAuthentication userAuthentication) throws Exception {
        return super.save(userAuthentication);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateUserAuthentication(UserAuthentication userAuthentication) throws Exception {
        return super.updateById(userAuthentication);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteUserAuthentication(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public UserAuthenticationQueryVo getUserAuthenticationById(Serializable id) throws Exception {
        return userAuthenticationMapper.getUserAuthenticationById(id);
    }

    @Override
    public Paging<UserAuthenticationQueryVo> getUserAuthenticationPageListBySql(UserAuthenticationPageParam userAuthenticationPageParam) throws Exception {
        Page<UserAuthenticationQueryVo> page = new PageInfo<>(userAuthenticationPageParam, OrderItem.desc("create_time"));
        IPage<UserAuthenticationQueryVo> iPage = userAuthenticationMapper.getUserAuthenticationPageList(page, userAuthenticationPageParam);
        return new Paging<UserAuthenticationQueryVo>(iPage);
    }

    @Override
    public String addUserAuthentication(InsertUserAuthenticationParam insertUserAuthenticationParam) throws IOException {
        if (StringUtils.isBlank(insertUserAuthenticationParam.getIdCard()) || insertUserAuthenticationParam.getIdCard().length() < 18) {
            throw new AndroidBusinessException(ApiCode.ID_CARD_FOREVER_ERR);
        }
        String beforeImage = null;
        String afterImage = null;
        if (insertUserAuthenticationParam.getBeforeImageFile() != null) {
            String images = upLoadService.uploadToOOS(insertUserAuthenticationParam.getBeforeImageFile());
            if (StringUtils.isNotBlank(images)) {
                beforeImage = images;
                insertUserAuthenticationParam.setBeforeImage(images);
            }
        }
        if (insertUserAuthenticationParam.getAfterImageFile() != null) {
            String images = upLoadService.uploadToOOS(insertUserAuthenticationParam.getAfterImageFile());
            if (StringUtils.isNotBlank(images)) {
                afterImage = images;
            }
        }
        UserAuthentication authentication = userAuthenticationMapper.selectOne(new LambdaQueryWrapper<UserAuthentication>().eq(UserAuthentication::getUserId, insertUserAuthenticationParam.getUserId()));
        if (authentication != null) {
            BeanUtils.copyProperties(insertUserAuthenticationParam, authentication);
            if (StringUtils.isNotBlank(beforeImage)) {
                authentication.setBeforeImage(beforeImage);
            }
            if (StringUtils.isNotBlank(afterImage)) {
                authentication.setAfterImage(afterImage);
            }
            authentication.setStatus(0);
            authentication.setUpdateTime(new Date());
            super.updateById(authentication);
            InitFaceVerifyResponse initFaceVerifyResponse = initFaceVerifyService.getInitFaceVerifyResponse(insertUserAuthenticationParam);
            log.info("阿里人脸对比数据=========》》》》》》====" + new Gson().toJson(initFaceVerifyResponse));
            log.debug("阿里人脸对比数据=========》》》》》》" + new Gson().toJson(initFaceVerifyResponse));
            return new Gson().toJson(initFaceVerifyResponse);
        } else {
            UserAuthentication userAuthentication = new UserAuthentication();
            BeanUtils.copyProperties(insertUserAuthenticationParam, userAuthentication);
            if (StringUtils.isNotBlank(beforeImage)) {
                userAuthentication.setBeforeImage(beforeImage);
            }
            if (StringUtils.isNotBlank(afterImage)) {
                userAuthentication.setAfterImage(afterImage);
            }
            User user = userService.getById(insertUserAuthenticationParam.getUserId());
            if (user != null) {
                userAuthentication.setUserId(user.getId());
                userAuthentication.setUserName(user.getUserName());
                userAuthentication.setUserPhone(user.getPhone());
            }
            userAuthentication.setCreateTime(new Date());
            super.save(userAuthentication);

            InitFaceVerifyResponse initFaceVerifyResponse = initFaceVerifyService.getInitFaceVerifyResponse(insertUserAuthenticationParam);
            return new Gson().toJson(initFaceVerifyResponse);
        }

    }

    @Override
    public IdCardVo getIdCard(InsertUserAuthenticationParam insertUserAuthenticationParam) throws IOException {
        IdCardVo idCardVo = new IdCardVo();

        String url = "https://idcardocrc.shumaidata.com/getidcardocrc";
        String appCode = "121ec835c42941cd9d9f877ed61bb635";

        Map<String, String> params = new HashMap<>();
        //image 和 url 二选一 ，选择其中一个时，请注释另外一个参数,默认使用image
        String beforeImage = null;
        String afterImage = null;
        String images = upLoadService.uploadToOOS(insertUserAuthenticationParam.getBeforeImageFile());
        if (StringUtils.isNotBlank(images)) {
            beforeImage = images;
        }
        params.put("url", beforeImage);
//        params.put("image", "data:image/jpeg;base64,/9j/4AAQSkZJRgABAQAAS");
        String result = postForm(appCode, url, params);
        JSONObject jsonObject = JSON.parseObject(result);
        JSONObject data = jsonObject.getJSONObject("data");
        JSONObject info = data.getJSONObject("info");
        if (info == null) {
            throw new AndroidBusinessException(ApiCode.ID_CARD_IMAGE_ERR);
        }
        String realName = info.getString("name");
        String idCard = info.getString("number");
        idCardVo.setRealName(realName);
        idCardVo.setIdCard(idCard);

        String imagess = upLoadService.uploadToOOS(insertUserAuthenticationParam.getAfterImageFile());
        if (StringUtils.isNotBlank(imagess)) {
            afterImage = imagess;
        }
        params.put("url", afterImage);
        String results = postForm(appCode, url, params);
        JSONObject jsonObjects = JSON.parseObject(results);
        JSONObject datas = jsonObjects.getJSONObject("data");
        JSONObject infos = datas.getJSONObject("info");
        if (infos == null) {
            throw new AndroidBusinessException(ApiCode.ID_CARD_IMAGE_ERR);
        }
        String authority = info.getString("authority");
        String timelimit = info.getString("timelimit");
        idCardVo.setAuthority(authority);
        idCardVo.setTimelimit(timelimit);
        return idCardVo;
    }

    @Override
    public Paging<UserAuthentication> getUserAuthenticationPageList(UserAuthenticationPageParam userAuthenticationPageParam) throws Exception {
        Page<UserAuthentication> page = new PageInfo<>(userAuthenticationPageParam, OrderItem.desc("create_time"));
        IPage<UserAuthentication> iPage = userAuthenticationMapper.selectPage(page, QueryHelpPlus.getPredicate(UserAuthentication.class, userAuthenticationPageParam));
        return new Paging<UserAuthentication>(iPage);
    }

    /**
     * 用到的HTTP工具包：okhttp 3.13.1
     * <dependency>
     * <groupId>com.squareup.okhttp3</groupId>
     * <artifactId>okhttp</artifactId>
     * <version>3.13.1</version>
     * </dependency>
     */
    public static String postForm(String appCode, String url, Map<String, String> params) throws IOException {
        OkHttpClient client = new OkHttpClient.Builder().build();
        FormBody.Builder formbuilder = new FormBody.Builder();
        Iterator<String> it = params.keySet().iterator();
        while (it.hasNext()) {
            String key = it.next();
            formbuilder.add(key, params.get(key));
        }
        FormBody body = formbuilder.build();
        Request request = new Request.Builder().url(url).addHeader("Authorization", "APPCODE " + appCode).post(body).build();
        Response response = client.newCall(request).execute();
        System.out.println("返回状态码" + response.code() + ",message:" + response.message());
        String result = response.body().string();
        return result;
    }
}
