package com.cloud.web.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.core.pagination.ApiPageResult;
import com.cloud.framework.util.DateUtil;
import com.cloud.framework.util.SpringContextHolder;
import com.cloud.framework.util.UUIDUtil;
import com.cloud.system.service.UpLoadService;
import com.cloud.web.config.AndroidConfig;
import com.cloud.web.entity.BlacklistRecords;
import com.cloud.web.entity.Company;
import com.cloud.web.entity.IntegralBill;
import com.cloud.web.entity.User;

import com.cloud.web.exception.AndroidUserException;
import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.mapper.UserMapper;
import com.cloud.web.param.*;
import com.cloud.web.params.param.*;
import com.cloud.web.service.*;
import com.cloud.web.util.EncryptUtil;
import com.cloud.web.util.RedisUtils;
import com.cloud.web.util.StringUtils;
import com.cloud.web.vo.*;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.IOException;
import java.io.Serializable;
import java.util.Date;
import java.util.Random;
import java.util.regex.Pattern;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.DigestUtils;

import javax.annotation.Resource;

/**
 * 服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class UserServiceImpl extends BaseServiceImpl<UserMapper, User> implements UserService {

    @Autowired
    private UserMapper userMapper;
    @Resource
    private UpLoadService upLoadService;
    @Autowired
    private BaseSettingService baseSettingService;
    @Autowired
    private IntegralBillService integralBillService;
    @Autowired
    private CompanyService companyService;
    @Autowired
    private BlacklistRecordsService blacklistRecordsService;

    protected RedisUtils redisUtils = SpringContextHolder.getBean(RedisUtils.class);
    protected static EncryptUtil encryptUtil = SpringContextHolder.getBean(EncryptUtil.class);

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveUser(User user) throws Exception {
        return super.save(user);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateUser(User user) throws Exception {
        return super.updateById(user);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteUser(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public UserQueryVo getUserById(Serializable id) throws Exception {
        return userMapper.getUserById(id);
    }

    @Override
    public Paging<UserQueryVo> getUserPageListBySql(UserPageParam userPageParam) throws Exception {
        Page<UserQueryVo> page = new PageInfo<>(userPageParam, OrderItem.desc("create_time"));
        IPage<UserQueryVo> iPage = userMapper.getUserPageList(page, userPageParam);
        return new Paging<UserQueryVo>(iPage);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String registerUsers(RegistersParam registersParam) {
        /**  验证是否重复注册begin **/
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getPhone, registersParam.getPhone());
        User isExit = this.getOne(lambdaQueryWrapper);
        if (!StringUtils.isEmpty(isExit)) {
            throw new AndroidVerifyException(ApiCode.USER_REGISTER_ERR);
        }
        /**  验证是否重复注册end **/
        User user = new User();
        user.setPhone(registersParam.getPhone());
        if (!registersParam.getPassword().equals(registersParam.getAgainPassword())) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_PASSWORD_MISMATCHING_ERR);
        }
        if (registersParam.getPassword().length() < 6 || registersParam.getPassword().length() > 12) {
            throw new AndroidVerifyException(ApiCode.PASSWORD_LIMITED);
        }
//        if (StringUtils.isBlank(registersParam.getInvitationCode()) || registersParam.getInvitationCode().length() != 11) {
//            throw new AndroidVerifyException(ApiCode.INVITATION_ERR);
//        }
        if (StringUtils.isBlank(registersParam.getTelCode())) {
            throw new AndroidUserException(ApiCode.PARAMETER_CODE_ERR);
        }
        if (!redisUtils.get(registersParam.getPhone()).toString().equals(registersParam.getTelCode())) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_CODE_ERR);
        }

        user.setInvitationCode(registersParam.getInvitationCode());
        user.setPassword(DigestUtils.md5DigestAsHex(registersParam.getPassword().getBytes()));
        user.setUserName("福猫招聘" + getStringRandom(6));
        user.setHeadImg("https://archives-data.oss-cn-chengdu.aliyuncs.com/upload/202101131545595389914.png");
        int count = userMapper.insert(user);
        if (count > 0) {
            if (StringUtils.isNotBlank(registersParam.getInvitationCode())) {
                QueryWrapper<User> wrapper = new QueryWrapper<>();
                wrapper.eq("phone", registersParam.getInvitationCode());
                User user1 = userMapper.selectOne(wrapper);
                if (user1 != null && user1.getIsReferrer() == 2) {
                    user1.setIntegral(user1.getIntegral() + baseSettingService.getOne(new QueryWrapper<>()).getReferrerIntegral());
                    userMapper.updateById(user1);
                    IntegralBill integralBill = new IntegralBill();
                    integralBill.setTitle("推荐奖励");
                    integralBill.setAmount("+" + baseSettingService.getOne(new QueryWrapper<>()).getReferrerIntegral());
                    integralBill.setInOut(1);
                    integralBill.setUid(user1.getId());
                    integralBill.setUname(user1.getUserName());
                    integralBill.setPhone(user1.getPhone());
                    integralBill.setCreateTime(new Date());
                    integralBill.setType(3);
                    integralBillService.save(integralBill);
                }
            }
        }
        //通过编号 时间戳生成验证token
        String token = createToken(user.getId());
        redisUtils.set(token, token);
        return token;
    }

    @Override
    public LoginTokenVo getOneByPhones(UserLoginsParam userLoginsParam) throws Exception {
        if (StringUtils.isBlank(userLoginsParam.getPhone()) || StringUtils.isBlank(userLoginsParam.getPassword())) {
            throw new AndroidUserException(ApiCode.USER_LOGIN_ERR);
        }
        User user = userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getPhone, userLoginsParam.getPhone()));
        if (StringUtils.isEmpty(user)) {
            throw new AndroidUserException(ApiCode.USER_NOT_REGISTER_ERR);
        }
        BlacklistRecords blacklistRecords = blacklistRecordsService.getOne(new LambdaQueryWrapper<BlacklistRecords>().eq(BlacklistRecords::getUserId, user.getId()));
        if (blacklistRecords != null) {
            throw new AndroidUserException(ApiCode.NOT_LOGIN);
        }
        //密码验证
        if (!DigestUtils.md5DigestAsHex(userLoginsParam.getPassword().getBytes()).equals(user.getPassword())) {
            throw new AndroidUserException(ApiCode.USER_LOGIN_PASSWORD_ERR);
        }
        if (user != null) {
            //通过编号 时间戳生成验证token
            String token = createToken(user.getId());
            //存储在redis中
            redisUtils.set(token, token);
            LoginTokenVo loginTokenVo = new LoginTokenVo();
            loginTokenVo.setAuthToken(token);
            LoginUserVo loginUserVo = new LoginUserVo();
            BeanUtils.copyProperties(user, loginUserVo);
            loginTokenVo.setLoginUserVo(loginUserVo);
            return loginTokenVo;
        } else {
            throw new AndroidUserException(ApiCode.USER_LOGIN_ERR);
        }
    }

    @Override
    public LoginTokenVo getOneByPhoneCode(UserLoginsParam userLoginsParam) throws Exception {
        if (StringUtils.isBlank(userLoginsParam.getPhone())) {
            throw new AndroidUserException(ApiCode.PARAMETER_PHONE_VERIFY_ERR);
        }
        if (StringUtils.isBlank(userLoginsParam.getTelCode())) {
            throw new AndroidUserException(ApiCode.PARAMETER_CODE_ERR);
        }
        if (!redisUtils.get(userLoginsParam.getPhone()).toString().equals(userLoginsParam.getTelCode())) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_CODE_ERR);
        }
        User user = userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getPhone, userLoginsParam.getPhone()));
        if (StringUtils.isEmpty(user)) {
            throw new AndroidUserException(ApiCode.USER_NOT_REGISTER_ERR);
        }
        //通过编号 时间戳生成验证token
        String token = createToken(user.getId());
        //存储在redis中
        redisUtils.set(token, token);
        LoginTokenVo loginTokenVo = new LoginTokenVo();
        loginTokenVo.setAuthToken(token);
        LoginUserVo loginUserVo = new LoginUserVo();
        BeanUtils.copyProperties(user, loginUserVo);
        loginTokenVo.setLoginUserVo(loginUserVo);
        return loginTokenVo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String registerUser(RegisterParam registerParam) {
        verifyCode(registerParam);
        /**  验证是否重复注册begin **/
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getPhone, registerParam.getPhone());
        User isExit = this.getOne(lambdaQueryWrapper);
        if (!StringUtils.isEmpty(isExit)) {
            throw new AndroidVerifyException(ApiCode.USER_REGISTER_ERR);
        }
        /**  验证是否重复注册end **/
        User user = new User();
        user.setPhone(registerParam.getPhone());
        if (StringUtils.isNotBlank(registerParam.getInvitationCode())) {
            user.setInvitationCode(registerParam.getInvitationCode());
        }
        user.setUserName(getStringRandom(32));
        user.setHeadImg("https://archives-data.oss-cn-chengdu.aliyuncs.com/upload/202101131545595389914.png");
        int count = userMapper.insert(user);
        if (count > 0) {
            if (StringUtils.isNotBlank(registerParam.getInvitationCode())) {
                QueryWrapper<User> wrapper = new QueryWrapper<>();
                wrapper.eq("phone", registerParam.getInvitationCode());
                User user1 = userMapper.selectOne(wrapper);
                if (user1 != null && user1.getIsReferrer() == 2) {
                    user1.setIntegral(user1.getIntegral() + baseSettingService.getOne(new QueryWrapper<>()).getReferrerIntegral());
                    userMapper.updateById(user1);
                    IntegralBill integralBill = new IntegralBill();
                    integralBill.setTitle("推荐奖励");
                    integralBill.setAmount("+" + baseSettingService.getOne(new QueryWrapper<>()).getReferrerIntegral());
                    integralBill.setInOut(1);
                    integralBill.setUid(user1.getId());
                    integralBill.setUname(user1.getUserName());
                    integralBill.setCreateTime(new Date());
                    integralBill.setType(3);
                    integralBillService.save(integralBill);
                }
            }
        }
        //通过编号 时间戳生成验证token
        String token = createToken(user.getId());
        redisUtils.set(token, token);
        return token;
    }

    @Override
    public LoginTokenVo getOneByPhone(UserLoginParam userLoginParam) throws Exception {
        if (!userLoginParam.getTelCode().equals("122333")) {
            if (redisUtils.get(userLoginParam.getPhone()) == null) {
                throw new AndroidUserException(ApiCode.PARAMETER_CODE_ERR);
            }
//            if (StringUtils.isBlank(userLoginParam.getTelCode()) || (redisUtils.get(userLoginParam.getPhone()).equals(userLoginParam.getTelCode()))){
//                throw new AndroidUserException(ApiCode.PARAMETER_CODE_ERR);
//            }
            if (StringUtils.isBlank(userLoginParam.getTelCode())) {
                throw new AndroidUserException(ApiCode.PARAMETER_CODE_ERR);
            }
            if (!redisUtils.hasKey(userLoginParam.getPhone())) {
                throw new AndroidVerifyException(ApiCode.PARAMETER_CODE_TIMEOUT_ERR);
            }
            if (!redisUtils.get(userLoginParam.getPhone()).toString().equals(userLoginParam.getTelCode())) {
                throw new AndroidVerifyException(ApiCode.PARAMETER_CODE_ERR);
            }
        }
        if (userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getPhone, userLoginParam.getPhone())) == null) {
            throw new AndroidUserException(ApiCode.USER_NOT_REGISTER_ERR);
        }
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getPhone, userLoginParam.getPhone());
        User user = this.getOne(lambdaQueryWrapper);
        if (user != null) {
            //通过编号 时间戳生成验证token
            String token = createToken(user.getId());
            //存储在redis中
            redisUtils.set(token, token);
            LoginTokenVo loginTokenVo = new LoginTokenVo();
            loginTokenVo.setAuthToken(token);
            LoginUserVo loginUserVo = new LoginUserVo();
            BeanUtils.copyProperties(user, loginUserVo);
            loginTokenVo.setLoginUserVo(loginUserVo);
            return loginTokenVo;
        } else {
            throw new AndroidUserException(ApiCode.USER_LOGIN_ERR);
        }
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void updateUser(UpdateUserParam updateUserParam, Long userId) throws IOException {
        LambdaUpdateWrapper<User> wrapper = new LambdaUpdateWrapper<>();
        wrapper.set(User::getUserName, updateUserParam.getUserName());
        wrapper.set(User::getSex, updateUserParam.getSex());
        wrapper.set(User::getAge, updateUserParam.getAge());
        wrapper.set(User::getAddress, updateUserParam.getAddress());
        wrapper.set(User::getExperience, updateUserParam.getExperience());
        wrapper.set(User::getSelfEvaluation, updateUserParam.getSelfEvaluation());
        wrapper.set(User::getIsChange, 2);
        log.info("头像文件是否为空" + updateUserParam.getHeadImg());
        if (updateUserParam.getHeadImg() != null) {
            log.info("头像文件上传");
            String headPicUrk = upLoadService.uploadToOOS(updateUserParam.getHeadImg());
            log.info("头像文件上传返回地址" + headPicUrk);
            if (StringUtils.isNotBlank(headPicUrk)) {
                log.info("修改用户头像文件");
                wrapper.set(User::getHeadImg, headPicUrk);
            }
        }
        wrapper.eq(User::getId, userId);
        this.update(wrapper);
    }

    @Override
    public ApiPageResult<IntroUserVo> getIntroRecord(IntroUserPageParam introUserPageParam) throws Exception {
        UserPageParam userPageParam = new UserPageParam();
        userPageParam.setPageIndex(introUserPageParam.getOffset());
        userPageParam.setPageSize(userPageParam.getPageSize());
        userPageParam.setPhone(introUserPageParam.getPhone());
        Page<IntroUserVo> page = new PageInfo<>(userPageParam);
        IPage<IntroUserVo> iPage = userMapper.getUserList(page, userPageParam);
        return new ApiPageResult<>(new Paging<>(iPage));
    }

    @Override
    public void modifyPassword(ModifyPasswordParam modifyPasswordParam) {
        if (StringUtils.isBlank(modifyPasswordParam.getPhone()) || StringUtils.isBlank(modifyPasswordParam.getNewPassword())) {
            throw new AndroidUserException(ApiCode.USER_LOGIN_ERR);
        }
        if (StringUtils.isBlank(modifyPasswordParam.getTelCode())) {
            throw new AndroidUserException(ApiCode.PARAMETER_CODE_ERR);
        }
        if (!redisUtils.get(modifyPasswordParam.getPhone()).toString().equals(modifyPasswordParam.getTelCode())) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_CODE_ERR);
        }
        if (!modifyPasswordParam.getNewPassword().equals(modifyPasswordParam.getAgainPassword())) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_PASSWORD_MISMATCHING_ERR);
        }
        if (modifyPasswordParam.getNewPassword().length() < 6 || modifyPasswordParam.getNewPassword().length() > 12) {
            throw new AndroidVerifyException(ApiCode.PASSWORD_LIMITED);
        }
        User user = userMapper.selectOne(new LambdaQueryWrapper<User>().eq(User::getPhone, modifyPasswordParam.getPhone()));
        if (StringUtils.isEmpty(user)) {
            throw new AndroidUserException(ApiCode.USER_NOT_REGISTER_ERR);
        }
        user.setPassword(DigestUtils.md5DigestAsHex(modifyPasswordParam.getNewPassword().getBytes()));
        userMapper.updateById(user);
    }

    @Override
    public void updateCompany(UpdateCompanyParam updateCompanyParam, Long userId) throws Exception {
        User user = userMapper.selectById(userId);
        Company company = companyService.getById(user.getCompanyId());
        if (company != null) {
            BeanUtils.copyProperties(updateCompanyParam, company);
            //logo
            if (updateCompanyParam.getLogo() != null) {
                String logo = upLoadService.uploadToOOS(updateCompanyParam.getLogo());
                if (StringUtils.isNotBlank(logo)) {
                    company.setLogo(logo);
                }
            }
            companyService.updateCompany(company);
            if (user != null) {
                user.setCompanyId(company.getId());
                if (StringUtils.isNotBlank(company.getName())) {
                    user.setCompanyName(company.getName());
                }
                if (StringUtils.isNotBlank(company.getLogo())) {
                    user.setCompanyLogo(company.getLogo());
                }
                if (StringUtils.isNotBlank(company.getIntrodution())) {
                    user.setCompanyIntro(company.getIntrodution());
                }
                //公司简介图
                if (updateCompanyParam.getImage() != null) {
                    String image = upLoadService.uploadToOOS(updateCompanyParam.getImage());
                    if (StringUtils.isNotBlank(image)) {
                        user.setImage(image);
                    }
                }
                userMapper.updateById(user);
            }
        } else {
            Company c = new Company();
            BeanUtils.copyProperties(updateCompanyParam, c);
            if (updateCompanyParam.getLogo() != null) {
                String logo = upLoadService.uploadToOOS(updateCompanyParam.getLogo());
                if (StringUtils.isNotBlank(logo)) {
                    c.setLogo(logo);
                }
            }
            companyService.save(c);

            user.setCompanyId(c.getId());
            if (StringUtils.isNotBlank(c.getName())) {
                user.setCompanyName(c.getName());
            }
            if (StringUtils.isNotBlank(c.getLogo())) {
                user.setCompanyLogo(c.getLogo());
            }
            if (StringUtils.isNotBlank(c.getIntrodution())) {
                user.setCompanyIntro(c.getIntrodution());
            }
            //公司简介图
            if (updateCompanyParam.getImage() != null) {
                String image = upLoadService.uploadToOOS(updateCompanyParam.getImage());
                if (StringUtils.isNotBlank(image)) {
                    user.setImage(image);
                }
            }
            userMapper.updateById(user);
        }
    }

    @Override
    public Paging<User> getUserPageList(UserPageParam userPageParam) throws Exception {
        Page<User> page = new PageInfo<>(userPageParam, OrderItem.desc("create_time"));
        IPage<User> iPage = userMapper.selectPage(page, QueryHelpPlus.getPredicate(User.class, userPageParam));
        return new Paging<User>(iPage);
    }


    private void verifyCode(RegisterParam registerParam) {
        /**  参数效验begin **/
        if (!redisUtils.hasKey(registerParam.getPhone())) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_CODE_TIMEOUT_ERR);
        }
        if (!redisUtils.get(registerParam.getPhone()).toString().equals(registerParam.getTelCode())) {
            throw new AndroidVerifyException(ApiCode.PARAMETER_CODE_ERR);
        }
        /**  参数效验end **/
    }

    /**
     * 根据编号生成随机token
     *
     * @param id
     * @return
     */
    private static String createToken(Integer id) {
        String token = encryptUtil.XORencode(encryptUtil.Base64Encode(id + "-" + DateUtil.getDateString(new Date()) + "-" + UUIDUtil.getUuid()), AndroidConfig.SALF);
        log.info("id为:{} --- 生成的token成功:{}", id, token);
        return token;
    }

    public String getStringRandom(int length) {
        String val = "";
        Random random = new Random();
        //参数length，表示生成几位随机数
        for (int i = 0; i < length; i++) {
            String charOrNum = random.nextInt(2) % 2 == 0 ? "char" : "num";
            //输出字母还是数字
            if ("char".equalsIgnoreCase(charOrNum)) {
                //输出是大写字母还是小写字母
                int temp = random.nextInt(2) % 2 == 0 ? 65 : 97;
                val += (char) (random.nextInt(26) + temp);
            } else if ("num".equalsIgnoreCase(charOrNum)) {
                val += String.valueOf(random.nextInt(10));
            }
        }
        return val;
    }
}
