package com.cloud.web.service.impl;

import com.aliyun.oss.OSS;
import com.aliyun.oss.OSSClientBuilder;
import com.cloud.config.properties.CosProperties;
import com.cloud.config.properties.OssProperties;
import com.cloud.framework.util.RegularUtil;
import com.cloud.web.service.WebUpLoadService;
import com.cloud.web.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.IOException;

/**
 * <pre>
 * 上传文件到腾讯云对象存储
 * </pre>
 *
 * @author cyty
 * @since 2019-10-24
 */
@Slf4j
@Service
public class WebUpLoadServiceImpl implements WebUpLoadService {

    @Autowired
    private CosProperties cosProperties;
    @Autowired
    private OssProperties ossProperties;

    /**
     * 上传文件到阿里云对象存储
     *
     * @param bytes
     * @return
     */
    @Override
    public String uploadToOOS(byte[] bytes, Integer type) throws IOException {
        String file = "upload/" + StringUtils.getRandomString(3) + ".jpeg";
        if (type != null) {
            file = file + ".html";
        }
        //阿里云对象存储配置
        String endpoint = ossProperties.getEndpoint();
        String accessKeyId = ossProperties.getAccessKeyId();
        String accessKeySecret = ossProperties.getAccessKeySecret();
        String bucket = ossProperties.getBucket();
        OSS ossClient = new OSSClientBuilder().build(endpoint, accessKeyId, accessKeySecret);
        ossClient.putObject(bucket, file, new ByteArrayInputStream(bytes));
        ossClient.shutdown();
        //返回文件路径
        String url = "https://" + bucket + "." + RegularUtil.convertUrl(endpoint) + "/" + file;
        return url;
    }

}
