package com.cloud.web.service.impl;

import cn.hutool.http.HttpUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.cloud.framework.common.api.ApiCode;
import com.cloud.framework.common.exception.BusinessException;
import com.cloud.framework.util.IpUtil;
import com.cloud.web.entity.BaseSetting;
import com.cloud.web.entity.IntegralBill;
import com.cloud.web.entity.User;
import com.cloud.web.entity.WithdrawDeposit;

import com.cloud.web.exception.AndroidBusinessException;
import com.cloud.web.exception.AndroidVerifyException;
import com.cloud.web.mapper.WithdrawDepositMapper;
import com.cloud.web.params.param.InsertDepositParam;
import com.cloud.web.service.BaseSettingService;
import com.cloud.web.service.IntegralBillService;
import com.cloud.web.service.UserService;
import com.cloud.web.service.WithdrawDepositService;
import com.cloud.web.param.WithdrawDepositPageParam;
import com.cloud.web.util.MyStringUtils;
import com.cloud.web.vo.WithdrawDepositQueryVo;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.core.pagination.PageInfo;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.binarywang.wxpay.bean.entpay.EntPayRequest;
import com.github.binarywang.wxpay.bean.entpay.EntPayResult;
import com.github.binarywang.wxpay.exception.WxPayException;
import com.github.binarywang.wxpay.service.WxPayService;
import org.springframework.transaction.annotation.Transactional;
import lombok.extern.slf4j.Slf4j;
import com.cloud.framework.util.QueryHelpPlus;
import org.springframework.stereotype.Service;
import org.springframework.beans.factory.annotation.Autowired;

/**
 * 服务实现类
 *
 * @author zyl
 * @since 2021-01-08
 */
@Slf4j
@Service
public class WithdrawDepositServiceImpl extends BaseServiceImpl<WithdrawDepositMapper, WithdrawDeposit> implements WithdrawDepositService {

    @Autowired
    private WithdrawDepositMapper withdrawDepositMapper;
    @Autowired
    private UserService userService;
    @Autowired
    private IntegralBillService integralBillService;
    @Autowired
    private WxPayService wxPayService;
    @Autowired
    private BaseSettingService baseSettingService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveWithdrawDeposit(WithdrawDeposit withdrawDeposit) throws Exception {
        return super.save(withdrawDeposit);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateWithdrawDeposit(WithdrawDeposit withdrawDeposit) throws Exception {
        return super.updateById(withdrawDeposit);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteWithdrawDeposit(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public WithdrawDepositQueryVo getWithdrawDepositById(Serializable id) throws Exception {
        return withdrawDepositMapper.getWithdrawDepositById(id);
    }

    @Override
    public Paging<WithdrawDepositQueryVo> getWithdrawDepositPageListBySql(WithdrawDepositPageParam withdrawDepositPageParam) throws Exception {
        Page<WithdrawDepositQueryVo> page = new PageInfo<>(withdrawDepositPageParam, OrderItem.desc("create_time"));
        IPage<WithdrawDepositQueryVo> iPage = withdrawDepositMapper.getWithdrawDepositPageList(page, withdrawDepositPageParam);
        return new Paging<WithdrawDepositQueryVo>(iPage);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void insertDeposit(Long userId, InsertDepositParam insertDepositParam) throws Exception {
        String openAppId = "wxa3355ee024c4b87d";
        String openSecret = "b35f4d5d9da382493d15f5880944df11";
        //获取access_token
        String url = "https://api.weixin.qq.com/sns/oauth2/access_token" +
                "?appid=" + openAppId +
                "&secret=" + openSecret +
                "&code=" + insertDepositParam.getCode() +
                "&grant_type=authorization_code";
        String result = HttpUtil.get(url);
        //返回结果的json对象
        JSONObject resultObject = JSON.parseObject(result);
        String openId = resultObject.getString("openid");

        WithdrawDeposit withdrawDeposit = new WithdrawDeposit();
        withdrawDeposit.setBank(openId);
        withdrawDeposit.setAmount(insertDepositParam.getAmount());
        User user = userService.getById(userId);
        if (user == null) throw new AndroidBusinessException(ApiCode.USER_NOT_EXIT);
        if ((user.getIntegral() - insertDepositParam.getAmount()) < 0)
            throw new AndroidBusinessException(ApiCode.MONEY_INSUFFIVIENT);
        withdrawDeposit.setUid(user.getId());
        withdrawDeposit.setUname(user.getUserName());
        withdrawDeposit.setPhone(user.getPhone());
        //直接提现  状态由1改为2
        withdrawDeposit.setStatus(2);
        withdrawDeposit.setCreateTime(new Date());
        withdrawDepositMapper.insert(withdrawDeposit);
        //提现后，用户积分相应减少并添加积分记录
        //添加积分变动记录
        IntegralBill integralBill = new IntegralBill();
        integralBill.setTitle("提现");
        integralBill.setAmount("-" + insertDepositParam.getAmount());
        integralBill.setInOut(2);
        integralBill.setUid(user.getId());
        integralBill.setUname(user.getUserName());
        integralBill.setCreateTime(new Date());
        integralBill.setType(2);
        boolean flg = integralBillService.save(integralBill);
        if (!flg) {
            throw new AndroidVerifyException("提现失败");
        }
        //用户积分调整
        int integral = user.getIntegral() - insertDepositParam.getAmount();
        user.setIntegral(integral);
        boolean flag = userService.updateUser(user);
        if (!flag) throw new AndroidVerifyException("提现失败");
        //提现到微信
        EntPayRequest entPayRequest = new EntPayRequest();
        entPayRequest.setAppid("wxa3355ee024c4b87d");
        entPayRequest.setMchId("1608112070");
        entPayRequest.setPartnerTradeNo(MyStringUtils.getRandomString(10));
        //orIxS6K5OUzCs4XTIIwv7-4M9tHM
        entPayRequest.setOpenid(openId);
        entPayRequest.setCheckName("NO_CHECK");
        //提现金额
        BaseSetting baseSetting = baseSettingService.getOne();
        int price = new BigDecimal(insertDepositParam.getAmount()).multiply(baseSetting.getWithdrawProportion()).multiply(new BigDecimal(100)).intValue();
        entPayRequest.setAmount(price);
        entPayRequest.setDescription("提现");
        entPayRequest.setSpbillCreateIp(IpUtil.getRequestIp());
        EntPayResult entPayResult = null;
        try {
            entPayResult = wxPayService.getEntPayService().entPay(entPayRequest);
            if (entPayResult != null) {
                System.out.println("提现成功");
            }
            System.out.println("entPayResult : " + entPayResult);
        } catch (WxPayException e) {
            System.out.println("提现失败，返回报文" + e);
            throw new AndroidBusinessException(ApiCode.WITHDRAWDEPOSIT_ERR);
        }
    }

    @Override
    public Paging<WithdrawDeposit> getWithdrawDepositPageList(WithdrawDepositPageParam withdrawDepositPageParam) throws Exception {
        Page<WithdrawDeposit> page = new PageInfo<>(withdrawDepositPageParam, OrderItem.desc("create_time"));
        IPage<WithdrawDeposit> iPage = withdrawDepositMapper.selectPage(page, QueryHelpPlus.getPredicate(WithdrawDeposit.class, withdrawDepositPageParam));
        return new Paging<WithdrawDeposit>(iPage);
    }

}
