package com.cloud.web.util;

import org.apache.lucene.index.PersistentSnapshotDeletionPolicy;

import java.util.*;
import java.util.function.BiConsumer;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * @Author sj
 * Date on 2020/10/12  11:34
 * @Version 1.0
 */
public class CollectionUtils {

    /**
     * 集合分块方法
     *
     * @param collection 可拆分集合
     * @param maxSize    集合最大数量
     * @param splitSize  拆分份数
     * @param <T>
     * @return
     */
    public static <T> List<Collection<T>> splitList(Collection<T> collection, int maxSize, int splitSize) {
        if (null == collection || collection.size() == 0) {
            return Collections.emptyList();
        }
        return Stream.iterate(0, f -> f + 1)
                .limit(maxSize)
                .parallel()
                .map(a -> collection.parallelStream().skip(a * splitSize).limit(splitSize).collect(Collectors.toList()))
                .filter(b -> !b.isEmpty())
                .collect(Collectors.toList());
    }

    public static void main(String[] args) {
        List<Integer> list = new ArrayList<>();
        list.add(1);
        List<Collection<Integer>> list1 = splitList(list, list.size(), 5);
        list1.forEach(System.out::println);
    }

    /**
     * @param <T>
     * @param startIndex 开始遍历的索引
     * @param elements   集合
     * @param action
     */
    public static <T> void forEach(int startIndex, Iterable<? extends T> elements, BiConsumer<Integer, ? super T> action) {
        Objects.requireNonNull(elements);
        Objects.requireNonNull(action);
        if (startIndex < 0) {
            startIndex = 0;
        }
        int index = 0;
        for (T element : elements) {
            index++;
            if (index <= startIndex) {
                continue;
            }
            action.accept(index - 1, element);
        }
    }
}
