package com.cloud.web.util;

import com.cloud.framework.util.SpringContextUtil;
import com.cloud.web.service.WebUpLoadService;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.model.PicturesTable;
import org.apache.poi.hwpf.usermodel.*;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.springframework.stereotype.Component;

import java.io.*;
import java.util.Arrays;
import java.util.stream.Stream;

@Component
public class WordExcelToHtml {

    private static WebUpLoadService webUpLoadService = SpringContextUtil.getBean(WebUpLoadService.class);
    /**
     * 回车符ASCII码
     */
    private static final short ENTER_ASCII = 13;

    /**
     * 空格符ASCII码
     */
    private static final short SPACE_ASCII = 32;

    /**
     * 水平制表符ASCII码
     */
    private static final short TABULATION_ASCII = 9;

    public String htmlText = "";
    public String htmlTextTbl = "";
    public int counter = 0;
    public int beginPosi = 0;
    public int endPosi = 0;
    public int[] beginArray;
    public int[] endArray;
    public String[] htmlTextArray;
    public boolean tblExist = false;

    /**
     * 读取每个文字样式
     *
     * @param file
     * @throws Exception
     */
    @SuppressWarnings("all")
    public String getWordAndStyle(File file) throws Exception {
        htmlText = "";
        htmlTextTbl = "";
        counter = 0;
        beginPosi = 0;
        endPosi = 0;
        tblExist = false;
        FileInputStream in = new FileInputStream(file);
        ByteArrayOutputStream byteOS = new ByteArrayOutputStream();
        HWPFDocument doc = new HWPFDocument(in);
        Range rangetbl = doc.getRange();// 得到文档的读取范围
        TableIterator it = new TableIterator(rangetbl);
        int num = 100;
        beginArray = new int[num];
        endArray = new int[num];
        htmlTextArray = new String[num];

        // 取得文档中字符的总数
        int length = doc.characterLength();
        // 创建图片容器
        PicturesTable pTable = doc.getPicturesTable();

//        htmlText = "<html><head><title>" + doc.getSummaryInformation().getTitle() + "</title>";
//
//        htmlText += "<style type=\"text/css\">";
//        htmlText += "table.gridtable {font-family: verdana,arial,sans-serif;";
//        htmlText +=
//                "font-size:11px;color:#333333;border-width: 1px;border-color: #666666;border-collapse: collapse;}";
//        htmlText +=
//                "table.gridtable th {border-width: 1px;	padding: 8px;border-style: solid;border-color: #666666;background-color: #dedede;}";
//        htmlText +=
//                "table.gridtable td {border-width: 1px;padding: 8px;border-style: solid;border-color: #666666;background-color: #ffffff;}";
//        htmlText += "</style></head><body>";
//        htmlText += "</head><body>";
        // 创建临时字符串,好加以判断一串字符是否存在相同格式

        if (it.hasNext()) {
            readTable(it, rangetbl);
        }

        int cur = 0;

        String tempString = "";
        for (int i = 0; i < length - 1; i++) {
            // 整篇文章的字符通过一个个字符的来判断,range为得到文档的范围
            Range range = new Range(i, i + 1, doc);

            CharacterRun cr = range.getCharacterRun(0);
            if (tblExist) {
                if (i == beginArray[cur]) {
                    htmlText += tempString + htmlTextArray[cur];
                    tempString = "";
                    i = endArray[cur] - 1;
                    cur++;
                    continue;
                }
            }
            if (pTable.hasPicture(cr)) {
                htmlText += tempString;
                // 读写图片
                readPicture(pTable, cr);
                tempString = "";
            } else {

                Range range2 = new Range(i + 1, i + 2, doc);
                // 第二个字符
                CharacterRun cr2 = range2.getCharacterRun(0);
                char c = cr.text().charAt(0);

//                System.out.println(i + "::" + range.getEndOffset() + "::" + range.getStartOffset() + "::" + c);
                // 判断是否为回车符
                if (c == ENTER_ASCII) {
                    tempString += "<br/>";

                }
                // 判断是否为空格符
                else if (c == SPACE_ASCII)
                    tempString += " ";
                    // 判断是否为水平制表符
                else if (c == TABULATION_ASCII) tempString += "    ";
                // 比较前后2个字符是否具有相同的格式
                boolean flag = compareCharStyle(cr, cr2);
                if (flag)
                    tempString += cr.text();
                else {
                    String fontStyle = "<span style=\"font-family:" + cr.getFontName() + ";font-size:"
                            + cr.getFontSize() / 2 + "pt;";

                    if (cr.isBold()) fontStyle += "font-weight:bold;";
                    if (cr.isItalic()) fontStyle += "font-style:italic;";

                    htmlText += fontStyle + "\">" + tempString + cr.text() + "</span>";
                    tempString = "";
                }
            }
        }
        htmlText += "<div style='width:100%;height:20px'></div>";
//        htmlText += tempString + "</body></html>";
        //创建并上传生成的html文件到阿里云oss对象存储中
//        String url = writeFile(htmlText, StringUtils.getRandomString(3));
        return htmlText;
    }

    /**
     * 读写文档中的表格
     *
     * @throws Exception
     */
    public void readTable(TableIterator it, Range rangetbl) throws Exception {

        htmlTextTbl = "";
        // 迭代文档中的表格

        counter = -1;
        while (it.hasNext()) {
            tblExist = true;
            htmlTextTbl = "";
            Table tb = (Table) it.next();
            beginPosi = tb.getStartOffset();
            endPosi = tb.getEndOffset();

//            System.out.println("............" + beginPosi + "...." + endPosi);
            counter = counter + 1;
            // 迭代行，默认从0开始
            beginArray[counter] = beginPosi;
            endArray[counter] = endPosi;

            htmlTextTbl += "<table class=\"gridtable\">";
            for (int i = 0; i < tb.numRows(); i++) {
                TableRow tr = tb.getRow(i);

                htmlTextTbl += "<tr>";
                // 迭代列，默认从0开始
                for (int j = 0; j < tr.numCells(); j++) {
                    TableCell td = tr.getCell(j);// 取得单元格
                    int cellWidth = td.getWidth();

                    // 取得单元格的内容
                    for (int k = 0; k < td.numParagraphs(); k++) {
                        Paragraph para = td.getParagraph(k);
                        String s = para.text().toString().trim();
                        if (s == "") {
                            s = " ";
                        }
                        System.out.println(s);
                        htmlTextTbl += "<td width=" + cellWidth + ">" + s + "</td>";
//                        System.out.println(i + ":" + j + ":" + cellWidth + ":" + s);
                    } // end for
                } // end for
                htmlTextTbl += "</tr>";
            } // end for
            htmlTextTbl += "</table>";
            htmlTextArray[counter] = htmlTextTbl;

        } // end while
    }

    /**
     * 读写文档中的图片
     *
     * @param pTable
     * @param cr
     * @throws Exception
     */
    public void readPicture(PicturesTable pTable, CharacterRun cr) throws Exception {
        // 提取图片
        Picture pic = pTable.extractPicture(cr, false);
        //上传文件到阿里云oss上获取链接地址
        String fileUrl = webUpLoadService.uploadToOOS(pic.getContent(), null);
        htmlText += "<div style='max-width:100%'><img  src=\"" + fileUrl + "\"></img></div>";
    }

    public static boolean compareCharStyle(CharacterRun cr1, CharacterRun cr2) {
        boolean flag = false;
        if (cr1.isBold() == cr2.isBold() && cr1.isItalic() == cr2.isItalic()
                && cr1.getFontName().equals(cr2.getFontName()) && cr1.getFontSize() == cr2.getFontSize()) {
            flag = true;
        }
        return flag;
    }

    /**
     * 写文件
     *
     * @param s
     */
    public String writeFile(String s, String fileName) {
        BufferedWriter bw = null;
        System.out.println();
        System.out.println(s);
        System.out.println();
        try {
            //创建一个html临时文件
            File file = File.createTempFile(fileName, ".html");//创建临时文件
            bw = new BufferedWriter(new FileWriter(file));
            //将word文档中读取出来的数据写入临时文件中
            bw.write(s);
            bw.flush();
            //将临时文件上传到阿里云oss上获取地址
            String url = webUpLoadService.uploadToOOS(FileUtils.getBytesByFile(file), 1);
            //删除本地临时存储文件
            FileUtils.delFiles(file);
            return url;
        } catch (FileNotFoundException fnfe) {
            fnfe.printStackTrace();
        } catch (IOException ioe) {
            ioe.printStackTrace();
        } finally {
            try {
                if (bw != null) {
                    bw.close();
                }
            } catch (IOException ie) {
            }
        }
        return "";
    }


    /**
     * 解析word文档转换为字符串
     *
     * @param file
     * @throws Exception
     */
    @SuppressWarnings("all")
    public String getWordToStr(File file) throws Exception {
        htmlText = "";
        htmlTextTbl = "";
        counter = 0;
        beginPosi = 0;
        endPosi = 0;
        tblExist = false;
        FileInputStream in = new FileInputStream(file);
        HWPFDocument doc = new HWPFDocument(in);
        Range rangetbl = doc.getRange();// 得到文档的读取范围
        TableIterator it = new TableIterator(rangetbl);
        int num = 100;
        beginArray = new int[num];
        endArray = new int[num];
        htmlTextArray = new String[num];
        // 取得文档中字符的总数
        int length = doc.characterLength();
        // 创建图片容器
        PicturesTable pTable = doc.getPicturesTable();
        if (it.hasNext()) {
            readTable(it, rangetbl);
        }
        int cur = 0;
        String tempString = "";
        for (int i = 0; i < length - 1; i++) {
            // 整篇文章的字符通过一个个字符的来判断,range为得到文档的范围
            Range range = new Range(i, i + 1, doc);
            CharacterRun cr = range.getCharacterRun(0);
            if (tblExist) {
                if (i == beginArray[cur]) {
                    htmlText += tempString + htmlTextArray[cur];
                    tempString = "";
                    i = endArray[cur] - 1;
                    cur++;
                    continue;
                }
            }
            Range range2 = new Range(i + 1, i + 2, doc);
            // 第二个字符
            CharacterRun cr2 = range2.getCharacterRun(0);
            char c = cr.text().charAt(0);
            // 判断是否为回车符
            if (c == ENTER_ASCII) {
                continue;
            }
            // 判断是否为空格符
            else if (c == SPACE_ASCII)
                continue;
//                tempString += "";
                // 判断是否为水平制表符
            else if (c == TABULATION_ASCII) {
                continue;
//                tempString += "";
            }
            // 比较前后2个字符是否具有相同的格式
            boolean flag = compareCharStyle(cr, cr2);
            if (flag)
                tempString += cr.text();
            else {
                htmlText += tempString + cr.text();
                tempString = "";
            }
        }
        System.out.println(htmlText);
        return htmlText;
    }
}
