/**
 * Copyright (C) 2018-2020
 * All rights reserved, Designed By www.yixiang.co
 */
package com.cloud.scheduled.config;

import com.cloud.scheduled.entity.QuartzJob;
import com.cloud.scheduled.exception.QuartzRunTimeException;
import com.cloud.scheduled.service.QuartzJobService;
import com.cloud.scheduled.util.QuartzManage;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.quartz.impl.matchers.GroupMatcher;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.util.List;

/**
 * 定时任务注入
 */
@Component
@Slf4j
public class JobRunner implements ApplicationRunner {

    private final QuartzJobService quartzJobService;

    private final QuartzManage quartzManage;

    public JobRunner(QuartzJobService quartzJobService, QuartzManage quartzManage) {
        this.quartzJobService = quartzJobService;
        this.quartzManage = quartzManage;
    }


    /**
     * 项目启动时重新激活启用的定时任务
     *
     * @param applicationArguments
     */
    @Override
    public void run(ApplicationArguments applicationArguments) throws QuartzRunTimeException, SchedulerException {
        log.info("--------------------开始注入定时任务---------------------");
        List<QuartzJob> quartzJobs = quartzJobService.findByIsPauseIsFalse();
        for (QuartzJob quartzJob : quartzJobs) {
            if (quartzJob.getTriggerType().equals(QuartzTriggerTypeEnums.SIMPLE_TRIGGER.getCode())) {
                quartzManage.addSimpleJob(quartzJob);
            }
            if (quartzJob.getTriggerType().equals(QuartzTriggerTypeEnums.CRON_TRIGGER.getCode())) {
                quartzManage.addCronJob(quartzJob);
            }
            log.info("定时任务名称：{}---注入成功", quartzJob.getJobName());
        }
        log.info("定时任务注入数量：{}", quartzJobs.size());
        log.info("--------------------定时任务注入完成---------------------");
    }
}
