package com.cloud.scheduled.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.cloud.framework.common.entity.BaseEntity;
import com.cloud.framework.core.validator.groups.Update;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;

/**
 *
 * @author sj
 * @since 2020-10-07
 */
@Data
@Accessors(chain = true)
@EqualsAndHashCode(callSuper = true)
@ApiModel(value = "QuartzJob对象")
public class QuartzJob extends BaseEntity {
    private static final long serialVersionUID = 1L;

    public static final String JOB_KEY = "JOB_KEY";
    public static final String EXAM_JOB_NAME = "EXAM_";


    @NotNull(message = "id不能为空", groups = {Update.class})
    @ApiModelProperty("编号")
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    @NotNull(message = "触发器类型不能为空")
    @ApiModelProperty("触发器类型（1为SimpleTrigger触发器 2为CoreTrigger触发器)")
    private Integer triggerType;

    @NotBlank(message = "任务名称不能为空")
    @ApiModelProperty("任务名称")
    private String jobName;

    @ApiModelProperty("corn表达式")
    private String cronExpression;

    @ApiModelProperty("状态：1暂停、0启用")
    private Boolean isPause;

    @NotBlank(message = "javaBean名称不能为空")
    @ApiModelProperty("javaBean名称")
    private String beanName;

    @NotBlank(message = "方法名称不能为空")
    @ApiModelProperty("方法名称")
    private String methodName;

    @ApiModelProperty("方法参数 多参数使用json")
    private String params;

    @ApiModelProperty("开始执行时间")
    private Date executeStartTime;

    @ApiModelProperty("结束时间")
    private Date endTime;

    @ApiModelProperty("执行次数")
    private Integer forEver;

    @ApiModelProperty("创建时间")
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    @ApiModelProperty("更新时间")
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    @ApiModelProperty("逻辑删除")
    @TableLogic
    private Integer deleted;

    @ApiModelProperty("备注")
    private String remark;

}
