package com.cloud.scheduled.service;

import com.cloud.framework.common.service.BaseService;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.scheduled.entity.QuartzJob;
import com.cloud.scheduled.exception.QuartzRunTimeException;
import com.cloud.scheduled.param.QuartzJobPageParam;
import com.cloud.scheduled.vo.QuartzJobQueryVo;

import java.io.Serializable;
import java.util.List;

/**
 * 服务类
 *
 * @author sj
 * @since 2020-10-07
 */
public interface QuartzJobService extends BaseService<QuartzJob> {

    /**
     * 保存
     *
     * @param quartzJob
     * @return
     * @throws Exception
     */
    boolean saveQuartzJob(QuartzJob quartzJob) throws Exception;

    /**
     * 修改
     *
     * @param quartzJob
     * @return
     * @throws Exception
     */
    boolean updateQuartzJob(QuartzJob quartzJob) throws Exception;

    /**
     * 删除
     *
     * @param id
     * @return
     * @throws Exception
     */
    boolean deleteQuartzJob(Long id) throws Exception;

    /**
     * 根据ID获取查询对象
     *
     * @param id
     * @return
     * @throws Exception
     */
    QuartzJobQueryVo getQuartzJobById(Serializable id) throws Exception;

    /**
     * 获取分页对象
     *
     * @param quartzJobPageParam
     * @return
     * @throws Exception
     */
    Paging<QuartzJob> getQuartzJobPageList(QuartzJobPageParam quartzJobPageParam) throws Exception;

    /**
     * 获取分页对象通过sql
     *
     * @param quartzJobPageParam
     * @return
     * @throws Exception
     */
    Paging<QuartzJobQueryVo> getQuartzJobPageListBySql(QuartzJobPageParam quartzJobPageParam) throws Exception;


    /**
     * 更改定时任务状态
     *
     * @param quartzJob
     */
    void updateIsPause(QuartzJob quartzJob) throws QuartzRunTimeException;


    /**
     * 立即执行定时任务
     *
     * @param quartzJob
     */
    void execution(QuartzJob quartzJob) throws QuartzRunTimeException;

    /**
     * 查询启用的任务
     *
     * @return List
     */
    List<QuartzJob> findByIsPauseIsFalse();

    /**
     * 更新参数
     * @param id
     * @param params
     * @return
     */
    boolean updateParams(Long id, String params);

    /**
     * 通过模考大赛编号删除定时器
     * @param id
     * @return
     */
    List<QuartzJob> getListByExamId(Long id);

    /**
     * 通过任务名称和模考大赛编号获取任务
     * @param jobName
     * @param examId
     * @return
     */
    QuartzJob getJobByExamIdAndType(String jobName, Integer examId);

    boolean updateStateTime(QuartzJob quartzJob);

    boolean updateFinishTime(QuartzJob quartzJob);

    boolean updatePublishTime(QuartzJob quartzJob);
}
