package com.cloud.scheduled.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.PageInfo;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.util.QueryHelpPlus;
import com.cloud.scheduled.entity.QuartzJob;
import com.cloud.scheduled.exception.QuartzRunTimeException;
import com.cloud.scheduled.mapper.QuartzJobMapper;
import com.cloud.scheduled.param.QuartzJobPageParam;
import com.cloud.scheduled.service.QuartzJobService;
import com.cloud.scheduled.util.QuartzManage;
import com.cloud.scheduled.vo.QuartzJobQueryVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.List;

/**
 * 服务实现类
 *
 * @author sj
 * @since 2020-10-07
 */
@Slf4j
@Service
public class QuartzJobServiceImpl extends BaseServiceImpl<QuartzJobMapper, QuartzJob> implements QuartzJobService {

    @Autowired
    private QuartzJobMapper quartzJobMapper;

    @Autowired
    private QuartzManage quartzManage;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveQuartzJob(QuartzJob quartzJob) throws Exception {
        return quartzJobMapper.insert(quartzJob) > 0;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateQuartzJob(QuartzJob quartzJob) throws Exception {
        return super.updateById(quartzJob);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteQuartzJob(Long id) throws Exception {
        return super.removeById(id);
    }

    @Override
    public QuartzJobQueryVo getQuartzJobById(Serializable id) throws Exception {
        return quartzJobMapper.getQuartzJobById(id);
    }

    @Override
    public Paging<QuartzJobQueryVo> getQuartzJobPageListBySql(QuartzJobPageParam quartzJobPageParam) throws Exception {
        Page<QuartzJobQueryVo> page = new PageInfo<>(quartzJobPageParam, OrderItem.desc("create_time"));
        IPage<QuartzJobQueryVo> iPage = quartzJobMapper.getQuartzJobPageList(page, quartzJobPageParam);
        return new Paging<QuartzJobQueryVo>(iPage);
    }

    /**
     * 更改定时任务状态
     *
     * @param quartzJob /
     */
    @Override
    public void updateIsPause(QuartzJob quartzJob) throws QuartzRunTimeException {
        if (quartzJob.getIsPause()) {
            quartzManage.resumeJob(quartzJob);
            quartzJob.setIsPause(false);
        } else {
            quartzManage.pauseJob(quartzJob);
            quartzJob.setIsPause(true);
        }
        this.saveOrUpdate(quartzJob);
    }

    @Override
    public void execution(QuartzJob quartzJob) throws QuartzRunTimeException {
        quartzManage.runJobNow(quartzJob);
    }

    @Override
    public List<QuartzJob> findByIsPauseIsFalse() {
        LambdaQueryWrapper<QuartzJob> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(QuartzJob::getIsPause, false);
        lambdaQueryWrapper.eq(QuartzJob::getDeleted, 0);
        return baseMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public boolean updateParams(Long id, String params) {
        LambdaUpdateWrapper<QuartzJob> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        lambdaUpdateWrapper.set(QuartzJob::getParams, params);
        lambdaUpdateWrapper.eq(QuartzJob::getId, id);
        return this.update(lambdaUpdateWrapper);
    }

    @Override
    public List<QuartzJob> getListByExamId(Long id) {
        LambdaQueryWrapper<QuartzJob> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.like(QuartzJob::getParams, "\"examId\": " + id);
        return baseMapper.selectList(lambdaQueryWrapper);
    }

    @Override
    public QuartzJob getJobByExamIdAndType(String jobName, Integer examId) {
        LambdaQueryWrapper<QuartzJob> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(QuartzJob::getJobName, jobName + examId);
        return baseMapper.selectOne(lambdaQueryWrapper);
    }

    @Override
    public boolean updateStateTime(QuartzJob quartzJob) {
        LambdaUpdateWrapper<QuartzJob> lambdaUpdateWrapper = new LambdaUpdateWrapper<>();
        lambdaUpdateWrapper.set(QuartzJob::getExecuteStartTime, quartzJob.getExecuteStartTime());
        lambdaUpdateWrapper.eq(QuartzJob::getId, quartzJob.getId());
        return this.update(lambdaUpdateWrapper);
    }

    @Override
    public boolean updateFinishTime(QuartzJob quartzJob) {
        return false;
    }

    @Override
    public boolean updatePublishTime(QuartzJob quartzJob) {
        return false;
    }

    @Override
    public Paging<QuartzJob> getQuartzJobPageList(QuartzJobPageParam quartzJobPageParam) throws Exception {
        Page<QuartzJob> page = new PageInfo<>(quartzJobPageParam, OrderItem.desc("create_time"));
        IPage<QuartzJob> iPage = quartzJobMapper.selectPage(page, QueryHelpPlus.getPredicate(QuartzJob.class, quartzJobPageParam));
        return new Paging<QuartzJob>(iPage);
    }


}
