package com.cloud.scheduled.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.PageInfo;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.framework.util.QueryHelpPlus;
import com.cloud.scheduled.entity.QuartzLog;
import com.cloud.scheduled.mapper.QuartzLogMapper;
import com.cloud.scheduled.param.QuartzLogPageParam;
import com.cloud.scheduled.service.QuartzLogService;
import com.cloud.scheduled.vo.QuartzLogQueryVo;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;

/**
 * 定时任务日志 服务实现类
 *
 * @author sj
 * @since 2020-10-07
 */
@Slf4j
@Service
public class QuartzLogServiceImpl extends BaseServiceImpl<QuartzLogMapper, QuartzLog> implements QuartzLogService {

    @Autowired
    private QuartzLogMapper quartzLogMapper;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveQuartzLog(QuartzLog quartzLog) throws Exception {
        return super.save(quartzLog);
        }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateQuartzLog(QuartzLog quartzLog)throws Exception{
            return super.updateById(quartzLog);
            }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteQuartzLog(Long id)throws Exception{
            return super.removeById(id);
            }

    @Override
    public QuartzLogQueryVo getQuartzLogById(Serializable id)throws Exception{
            return quartzLogMapper.getQuartzLogById(id);
            }

    @Override
    public Paging<QuartzLogQueryVo> getQuartzLogPageListBySql(QuartzLogPageParam quartzLogPageParam)throws Exception{
            Page<QuartzLogQueryVo> page=new PageInfo<>(quartzLogPageParam,OrderItem.desc("create_time"));
                IPage<QuartzLogQueryVo> iPage= quartzLogMapper.getQuartzLogPageList(page, quartzLogPageParam);
            return new Paging<QuartzLogQueryVo>(iPage);
            }
    @Override
    public Paging<QuartzLog> getQuartzLogPageList(QuartzLogPageParam quartzLogPageParam)throws Exception{
        Page<QuartzLog> page=new PageInfo<>(quartzLogPageParam,OrderItem.desc("create_time"));
            IPage<QuartzLog> iPage= quartzLogMapper.selectPage(page,QueryHelpPlus.getPredicate(QuartzLog.class, quartzLogPageParam));
        return new Paging<QuartzLog>(iPage);
        }

}
