package com.cloud.scheduled.util;

import com.cloud.framework.util.SpringContextHolder;
import com.cloud.scheduled.entity.QuartzJob;
import com.cloud.scheduled.entity.QuartzLog;
import com.cloud.scheduled.service.QuartzJobService;
import com.cloud.scheduled.service.QuartzLogService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.quartz.*;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.scheduling.quartz.QuartzJobBean;

import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Future;

/**
 * @Author sj
 * Date on 2020/10/7  14:06
 * @Version 1.0
 * 在线考试job
 */
@Slf4j
public class ExecutionJob extends QuartzJobBean {

    private final static ThreadPoolTaskExecutor EXECUTOR = SpringContextHolder.getBean(ThreadPoolTaskExecutor.class);

    @SneakyThrows
    @Override
    protected void executeInternal(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        QuartzJob quartzJob = (QuartzJob) jobExecutionContext.getMergedJobDataMap().get(QuartzJob.JOB_KEY);
        // 获取spring bean
        QuartzLogService quartzLogService = SpringContextHolder.getBean(QuartzLogService.class);
        QuartzJobService quartzJobService = SpringContextHolder.getBean(QuartzJobService.class);

        QuartzLog qLog = new QuartzLog();
        qLog.setJobName(quartzJob.getJobName());
        qLog.setBeanName(quartzJob.getBeanName());
        qLog.setMethodName(quartzJob.getMethodName());
        qLog.setParams(quartzJob.getParams());
        long startTime = System.currentTimeMillis();
        qLog.setCronExpression(quartzJob.getCronExpression());
        try {
            // 执行任务
            log.info("任务准备执行，任务名称：{}", quartzJob.getJobName());
            QuartzRunnable task = new QuartzRunnable(quartzJob.getBeanName(), quartzJob.getMethodName(),
                    quartzJob.getParams());
            CompletableFuture completableFuture = CompletableFuture.supplyAsync(task, EXECUTOR);
            completableFuture.get();
            long times = System.currentTimeMillis() - startTime;
            qLog.setTime(times);
            // 任务状态
            qLog.setIsSuccess(true);
            log.info("任务执行完毕，任务名称：{} 总共耗时：{} 毫秒", quartzJob.getJobName(), times);
        } catch (Exception e) {
            log.error("任务执行失败，任务名称：{}" + quartzJob.getJobName(), e);
            long times = System.currentTimeMillis() - startTime;
            qLog.setTime(times);
            // 任务状态 0：成功 1：失败
            qLog.setIsSuccess(false);
            qLog.setExceptionDetail(ThrowableUtil.getStackTrace(e));
            quartzJob.setIsPause(false);
            //更新状态
            quartzJobService.updateIsPause(quartzJob);
        } finally {
            quartzLogService.save(qLog);
        }
    }
}
