

package com.cloud.system.controller;

import com.cloud.config.properties.SpringBootPlusProperties;
import com.cloud.framework.common.api.ApiResult;
import com.cloud.framework.log.annotation.Module;
import com.cloud.framework.log.annotation.OperationLog;
import com.cloud.framework.log.enums.OperationLogType;
import com.cloud.framework.util.UploadUtil;
import com.cloud.system.service.UpLoadService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FilenameUtils;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.commons.lang3.RandomStringUtils;
import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.converter.PicturesManager;
import org.apache.poi.hwpf.converter.WordToHtmlConverter;
import org.apache.poi.hwpf.usermodel.Picture;
import org.apache.poi.hwpf.usermodel.PictureType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.w3c.dom.Document;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import java.io.*;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * 上传控制器
 *
 * @author cyty
 * @date 2019/8/20
 * @since 1.2.1-RELEASE
 */
@Slf4j
@RestController
@RequestMapping("/upload")
@Module("system")
@Api(value = "文件上传", tags = {"文件上传"})
public class UploadController {

    @Autowired
    private SpringBootPlusProperties springBootPlusProperties;
    @Autowired
    private UpLoadService upLoadService;

    /**
     * 上传单个文件
     *
     * @return
     */
    @PostMapping
    @OperationLog(name = "上传单个文件", type = OperationLogType.UPLOAD)
    @ApiOperation(value = "上传单个文件", response = ApiResult.class)
    @ApiImplicitParams({
            @ApiImplicitParam(name = "file", value = "文件", required = true, dataType = "__file"),
            @ApiImplicitParam(name = "type", value = "类型 head:头像", required = true)
    })
    public ApiResult<String> upload(@RequestParam("file") MultipartFile multipartFile,
                                    @RequestParam("type") String type) throws Exception {
        log.info("multipartFile = " + multipartFile);
        log.info("ContentType = " + multipartFile.getContentType());
        log.info("OriginalFilename = " + multipartFile.getOriginalFilename());
        log.info("Name = " + multipartFile.getName());
        log.info("Size = " + multipartFile.getSize());
        log.info("type = " + type);

        // 上传文件，返回保存的文件名称
        String saveFileName = UploadUtil.upload(springBootPlusProperties.getUploadPath(), multipartFile, originalFilename -> {
            // 文件后缀
            String fileExtension = FilenameUtils.getExtension(originalFilename);
            // 这里可自定义文件名称，比如按照业务类型/文件格式/日期
            String dateString = LocalDateTime.now().format(DateTimeFormatter.ofPattern("yyyyMMddHHmmssS")) + RandomStringUtils.randomNumeric(6);
            String fileName = dateString + "." + fileExtension;
            return fileName;
        });

        // 上传成功之后，返回访问路径，请根据实际情况设置

        String fileAccessPath = springBootPlusProperties.getResourceAccessUrl() + saveFileName;
        log.info("fileAccessPath:{}", fileAccessPath);

        return ApiResult.ok(fileAccessPath);
    }


    /**
     * 上传文件到腾讯云cos
     *
     * @return
     */
    @PostMapping("/uploadToCOS")
    @ApiOperation(value = "上传文件到腾讯云cos", response = ApiResult.class)
    public ApiResult uploadToCOS(@RequestParam("file") MultipartFile multipartFile) throws Exception {

        String url = upLoadService.upToOss(multipartFile);
        return ApiResult.ok(url);
    }


    /**
     * 上传文件到阿里云oos
     *
     * @return
     */
    @PostMapping("/uploadToOOS")
    @ApiOperation(value = "上传文件到阿里云oos", response = ApiResult.class)
    public ApiResult uploadToOOS(@RequestParam("file") MultipartFile multipartFile) throws Exception {
        String url = upLoadService.uploadToOOS(multipartFile);
        return ApiResult.ok(url);
    }

    /**
     * 删除阿里云上文件
     *
     * @return
     */
    @GetMapping("/delToOOS")
    @ApiOperation(value = "删除阿里云上的文件", response = ApiResult.class)
    public ApiResult delToOOS(@RequestParam("filePath") String filePath) throws Exception {
        boolean flag = upLoadService.deleteFile(filePath);
        return ApiResult.ok(flag);
    }

}
