package com.cloud.system.service.impl;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cloud.framework.common.exception.BusinessException;
import com.cloud.framework.common.service.impl.BaseServiceImpl;
import com.cloud.framework.core.pagination.PageInfo;
import com.cloud.framework.core.pagination.Paging;
import com.cloud.system.convert.SysPermissionConvert;
import com.cloud.system.entity.SysPermission;
import com.cloud.system.enums.MenuLevelEnum;
import com.cloud.system.enums.StateEnum;
import com.cloud.system.mapper.SysPermissionMapper;
import com.cloud.system.param.SysPermissionPageParam;
import com.cloud.system.service.SysPermissionService;
import com.cloud.system.service.SysRolePermissionService;
import com.cloud.system.vo.MetaVo;
import com.cloud.system.vo.RouterVo;
import com.cloud.system.vo.SysPermissionQueryVo;
import com.cloud.system.vo.SysPermissionTreeVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.*;
import java.util.stream.Collectors;


/**
 * <pre>
 * 系统权限 服务实现类
 * </pre>
 *
 * @author cyty
 * @since 2019-10-25
 */
@Slf4j
@Service
public class SysPermissionServiceImpl extends BaseServiceImpl<SysPermissionMapper, SysPermission> implements SysPermissionService {

    @Autowired
    private SysPermissionMapper sysPermissionMapper;

    @Autowired
    private SysRolePermissionService sysRolePermissionService;

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean saveSysPermission(SysPermission sysPermission) throws Exception {
        sysPermission.setId(null);
        return super.save(sysPermission);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean updateSysPermission(SysPermission sysPermission) throws Exception {
        // 获取权限
        if (getById(sysPermission.getId()) == null) {
            throw new BusinessException("权限不存在");
        }
        sysPermission.setUpdateTime(new Date());
        return super.updateById(sysPermission);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean deleteSysPermission(Long id) throws Exception {
        boolean isExists = sysRolePermissionService.isExistsByPermissionId(id);
        if (isExists) {
            throw new BusinessException("该权限存在角色关联关系，不能删除");
        }
        return super.removeById(id);
    }

    @Override
    public SysPermissionQueryVo getSysPermissionById(Serializable id) throws Exception {
        return sysPermissionMapper.getSysPermissionById(id);
    }

    @Override
    public Paging<SysPermissionQueryVo> getSysPermissionPageList(SysPermissionPageParam sysPermissionPageParam) throws Exception {
        Page<SysPermissionQueryVo> page = new PageInfo<>(sysPermissionPageParam, OrderItem.desc(getLambdaColumn(SysPermission::getCreateTime)));
        IPage<SysPermissionQueryVo> iPage = sysPermissionMapper.getSysPermissionPageList(page, sysPermissionPageParam);
        return new Paging(iPage);
    }

    @Override
    public boolean isExistsByPermissionIds(List<Long> permissionIds) {
        QueryWrapper queryWrapper = new QueryWrapper();
        queryWrapper.in("id", permissionIds);
        return sysPermissionMapper.selectCount(queryWrapper).intValue() == permissionIds.size();
    }

    @Override
    public List<SysPermission> getAllMenuList() throws Exception {
        SysPermission sysPermission = new SysPermission().setState(StateEnum.ENABLE.getCode());
        // 获取所有已启用的权限列表
        return sysPermissionMapper.selectList(new QueryWrapper(sysPermission));
    }

    @Override
    public List<SysPermissionTreeVo> getAllMenuTree() throws Exception {
        List<SysPermission> list = getAllMenuList();
        // 转换成树形菜单
        List<SysPermissionTreeVo> treeVos = convertSysPermissionTreeVoList(list);
        return treeVos;
    }

    @Override
    public List<SysPermissionTreeVo> convertSysPermissionTreeVoList(List<SysPermission> list) {
        // 按level分组获取map
        Map<Integer, List<SysPermission>> map = list.stream().collect(Collectors.groupingBy(SysPermission::getLevel));
        List<SysPermissionTreeVo> treeVos = new ArrayList<>();
        // 循环获取三级菜单树形集合
        for (SysPermission one : map.get(1)) {
            SysPermissionTreeVo oneVo = SysPermissionConvert.INSTANCE.permissionToTreeVo(one);
            Long oneParentId = oneVo.getParentId();
            if (oneParentId == null || oneParentId == 0) {
                treeVos.add(oneVo);
            }
            List<SysPermission> twoList = map.get(2);
            if (org.apache.commons.collections.CollectionUtils.isNotEmpty(twoList)) {
                for (SysPermission two : twoList) {
                    SysPermissionTreeVo twoVo = SysPermissionConvert.INSTANCE.permissionToTreeVo(two);
                    if (two.getParentId().equals(one.getId())) {
                        oneVo.getChildren().add(twoVo);
                    }
                    List<SysPermission> threeList = map.get(3);
                    if (org.apache.commons.collections.CollectionUtils.isNotEmpty(threeList)) {
                        for (SysPermission three : threeList) {
                            if (three.getParentId().equals(two.getId())) {
                                SysPermissionTreeVo threeVo = SysPermissionConvert.INSTANCE.permissionToTreeVo(three);
                                twoVo.getChildren().add(threeVo);
                            }
                        }
                    }
                }
            }

        }
        return treeVos;
    }

    @Override
    public List<String> getPermissionCodesByUserId(Long userId) throws Exception {
        return sysPermissionMapper.getPermissionCodesByUserId(userId);
    }

    @Override
    public List<SysPermission> getMenuListByUserId(Long userId) throws Exception {
        return sysPermissionMapper.getMenuListByUserId(userId);
    }

    @Override
    public List<SysPermissionTreeVo> getMenuTreeByUserId(Long userId) throws Exception {
        // 所有节点元素集合
        List<SysPermissionTreeVo> sysPermissionTreeVo = sysPermissionMapper.getSysPermissionTreeVo(userId);
        // 转换成树形菜单
        //List<SysPermissionTreeVo> treeVos = convertSysPermissionTreeVoList(list);
        return getChildPerms2(sysPermissionTreeVo, 0);
    }

//    /**
//     * 根据父节点的ID获取所有子节点
//     *
//     * @param list 分类表
//     * @param parentId 传入的父节点ID
//     * @return String
//     */
//    public List<SysPermissionTreeVo> getChildPerms(List<SysPermissionTreeVo> list, int parentId)
//    {
//        List<SysPermissionTreeVo> returnList = new ArrayList<SysPermissionTreeVo>();
//        for (Iterator<SysPermissionTreeVo> iterator = list.iterator(); iterator.hasNext();)
//        {
//            SysPermissionTreeVo t = (SysPermissionTreeVo) iterator.next();
//            // 一、根据传入的某个父节点ID,遍历该父节点的所有子节点
//            if (t.getParentId() == parentId)
//            {
//                recursionFn(list, t);
//                returnList.add(t);
//            }
//        }
//        return returnList;
//    }

    /**
     * 根据父节点的ID获取所有子节点
     *
     * @param list     所有节点集合
     * @param parentId 传入定义的父节点ID
     * @return
     */
    public List<SysPermissionTreeVo> getChildPerms2(List<SysPermissionTreeVo> list, int parentId) {
        return list.stream()
                // 筛选出所有为父节点的元素集合
                // 即遍历集合中所有的元素，根据parentId==我们定义的父节点ID
                .filter(f -> f.getParentId() == parentId)
                .map(m -> {
                    recursionFn2(list, m);
                    return m;
                })
                .collect(Collectors.toList());
    }

//    /**
//     * 递归列表
//     *
//     * @param list
//     * @param t
//     */
//    private void recursionFn(List<SysPermissionTreeVo> list, SysPermissionTreeVo t)
//    {
//        // 得到子节点列表
//        List<SysPermissionTreeVo> childList = getChildList(list, t);
//        t.setChildren(childList);
//        for (SysPermissionTreeVo tChild : childList)
//        {
//            if (hasChild(list, tChild))
//            {
//                // 判断是否有子节点
//                Iterator<SysPermissionTreeVo> it = childList.iterator();
//                while (it.hasNext())
//                {
//                    SysPermissionTreeVo n = (SysPermissionTreeVo) it.next();
//                    recursionFn(list, n);
//                }
//            }
//        }
//    }

    /**
     * 递归 通过父节点获取子节点集合 通过子节点获取孙节点集合 ...
     *
     * @param list 所有节点集合
     * @param t    已经筛选出的父节点元素
     */
    private void recursionFn2(List<SysPermissionTreeVo> list, SysPermissionTreeVo t) {
        // 得到该父节点元素下的所有子节点集合
        List<SysPermissionTreeVo> childList = getChildList2(list, t);
        // set子节点集合
        t.setChildren(childList);
        childList.stream()
                // 遍历该父节点下个所有子节点集合
                // 筛选判断每一个子节点是否有孙节点
                .filter(f -> hasChild(list, f))
                // 遍历有孙节点的子节点集合
                // 递归调用
                .forEach(fo -> recursionFn2(list, fo));
    }

//    /**
//     * 得到子节点列表
//     */
//    private List<SysPermissionTreeVo> getChildList(List<SysPermissionTreeVo> list, SysPermissionTreeVo t)
//    {
//        List<SysPermissionTreeVo> tlist = new ArrayList<SysPermissionTreeVo>();
//        Iterator<SysPermissionTreeVo> it = list.iterator();
//        while (it.hasNext())
//        {
//            SysPermissionTreeVo n = (SysPermissionTreeVo) it.next();
//            if (n.getParentId().longValue() == t.getId().longValue())
//            {
//                tlist.add(n);
//            }
//        }
//        return tlist;
//    }

    /**
     * 通过父节点获取子节点集合
     *
     * @param list 所有节点集合
     * @param t    父节点元素
     * @return
     */
    private List<SysPermissionTreeVo> getChildList2(List<SysPermissionTreeVo> list, SysPermissionTreeVo t) {
        return list.stream()
                .filter(f -> f.getParentId().equals(t.getId()))
                .collect(Collectors.toList());
    }

    /**
     * @param list 所有节点集合
     * @param t    父节点元素
     * @return
     */
    private boolean hasChild(List<SysPermissionTreeVo> list, SysPermissionTreeVo t) {
        return getChildList2(list, t).size() > 0 ? true : false;
    }

    @Override
    public List<Long> getPermissionIdsByRoleId(Long roleId) throws Exception {

        // 根据角色id获取该对应的所有三级权限ID

        return null;
    }


    @Override
    public List<SysPermissionTreeVo> getNavMenuTree() throws Exception {
        List<Integer> levels = Arrays.asList(MenuLevelEnum.ONE.getCode(), MenuLevelEnum.TWO.getCode());
        Wrapper wrapper = lambdaQuery()
                .in(SysPermission::getLevel, levels)
                .eq(SysPermission::getState, StateEnum.ENABLE.getCode())
                .getWrapper();


        List<SysPermission> list = sysPermissionMapper.selectList(wrapper);

        return convertSysPermissionTreeVoList(list);

    }

    /**
     * 创建路由菜单
     */
    @Override
    public List<RouterVo> buildMenus(List<SysPermissionTreeVo> menus) {
        List<RouterVo> routers = new LinkedList<RouterVo>();
        for (SysPermissionTreeVo menu : menus) {
            RouterVo router = new RouterVo();
            router.setName(menu.getName());
            router.setPath(getRouterPath(menu));

            router.setComponent(StringUtils.isEmpty(menu.getComponent()) ? "Layout" : menu.getComponent());
            router.setMeta(new MetaVo(menu.getName(), menu.getIcon()));
            router.setName(menu.getName());
            List<SysPermissionTreeVo> cMenus = menu.getChildren();

            if (cMenus != null && cMenus.size() > 0 && "M".equals(menu.getType())) {
                router.setAlwaysShow(true);
                router.setRedirect("noRedirect");
                router.setChildren(buildMenus(cMenus));
            }
            routers.add(router);
        }
        return routers;
    }

    /**
     * 获取路由地址
     *
     * @param menu 菜单信息
     * @return 路由地址
     */
    public String getRouterPath(SysPermissionTreeVo menu) {
        String routerPath = menu.getPath();
        // 非外链并且是一级目录
        if (0 == menu.getParentId() && "1".equals(menu.getIsFrame())) {
            routerPath = "/" + menu.getPath();
        }
        return routerPath;
    }
}
